@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -26,7 +26,7 @@
  * (c) 2001 James Courtier-Dutton <James@superbug.demon.co.uk>
  *
  * 
- * $Id: audio_alsa_out.c,v 1.160 2006/05/03 19:46:06 dsalt Exp $
+ * $Id: audio_alsa_out.c,v 1.165 2006/09/08 20:40:34 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,13 +45,7 @@
 
 #define ALSA_PCM_NEW_HW_PARAMS_API
 #define ALSA_PCM_NEW_SW_PARAMS_API
-#ifdef HAVE_ALSA_ASOUNDLIB_H
-#include <alsa/asoundlib.h>
-#elif HAVE_SYS_ASOUNDLIB_H
-#include <sys/asoundlib.h>
-#else
-#error "required asoundlib.h neither in sys/ nor alsa/ - unable to compile."
-#endif
+#include <asoundlib.h>
 
 #include <sys/ioctl.h>
 #include <inttypes.h>
@@ -280,36 +274,13 @@ static void
 			   const char *function, int err, const char *fmt, ...) {
 #ifdef DEBUG
   va_list   args;
-  char     *buf;
-  int       n, size = 100;
-  
-  printf("%s:%s:%d entered\n", __FILE__, __FUNCTION__, __LINE__ );
-  if(!(buf = xine_xmalloc(size))) 
-    return;
-  
-  while(1) {
-    
-    va_start(args, fmt);
-    n = vsnprintf(buf, size, fmt, args);
-    va_end(args);
-    
-    if(n > -1 && n < size)
-      break;
-    
-    if(n > -1)
-      size = n + 1;
-    else
-      size *= 2;
-    
-    if((buf = realloc(buf, size)) == NULL)
-      return;
-  }
-  
-   
-  printf("%s: %s() %s.\n", file, function, buf);
+  char     *buf = NULL;
 
-  if(buf)
-    free(buf);
+  va_start(args, fmt);
+  vasprintf(&buf, fmt, args);
+  va_end(args);
+  printf("%s: %s() %s.\n", file, function, buf);
+  free(buf);
 #endif
 }
 
@@ -338,6 +309,8 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
   int                   err, dir;
   int                 open_mode=1; /* NONBLOCK */
   /* int                   open_mode=0;  BLOCK */
+  struct timeval start_time;
+  struct timeval end_time;
 
   snd_pcm_hw_params_alloca(&params);
   snd_pcm_sw_params_alloca(&swparams);
@@ -390,8 +363,21 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
   this->bytes_in_buffer        = 0;
   /*
    * open audio device
+   * When switching to surround, dmix blocks the device some time, so we just keep trying for 0.8sec.
    */
-  err=snd_pcm_open(&this->audio_fd, pcm_device, direction, open_mode);      
+  gettimeofday(&start_time, NULL);
+  do {
+    err = snd_pcm_open(&this->audio_fd, pcm_device, direction, open_mode);      
+    gettimeofday(&end_time, NULL);
+    if( err == -EBUSY ) {
+      if( (double)end_time.tv_sec + 1E-6*end_time.tv_usec 
+          - (double)start_time.tv_sec - 1E-6*start_time.tv_usec > 0.8)
+        break;
+      else
+        usleep(10000);
+    }
+  } while( err == -EBUSY );
+  
   if(err <0 ) {                                                           
     xprintf (this->class->xine, XINE_VERBOSITY_LOG, 
 	     _("audio_alsa_out: snd_pcm_open() of %s failed: %s\n"), pcm_device, snd_strerror(err));               
@@ -1354,6 +1340,8 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   int speakers;
 
   this = (alsa_driver_t *) xine_xmalloc (sizeof (alsa_driver_t));
+  if (!this)
+    return NULL;
 
   this->class = class;
 
@@ -1438,6 +1426,7 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
           _("snd_pcm_open() failed:%d:%s\n"), err, snd_strerror(err)); 
     xine_log (this->class->xine, XINE_LOG_MSG,
           _(">>> Check if another program already uses PCM <<<\n")); 
+    free(this);
     return NULL; 
   }
 
@@ -1448,6 +1437,8 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   if (err < 0) {
     xprintf (this->class->xine, XINE_VERBOSITY_DEBUG, 
 	     "audio_alsa_out: broken configuration for this PCM: no configurations available\n");
+    snd_pcm_close(this->audio_fd);
+    free(this);
     return NULL;
   }
   err = snd_pcm_hw_params_set_access(this->audio_fd, params,
@@ -1455,6 +1446,8 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
   if (err < 0) {
     xprintf (this->class->xine, XINE_VERBOSITY_DEBUG, 
 	     "audio_alsa_out: access type not available");
+    snd_pcm_close(this->audio_fd);
+    free(this);
     return NULL;
   }
 
@@ -1588,6 +1581,12 @@ static ao_driver_t *open_plugin (audio_driver_class_t *class_gen, const void *da
                                                "information on alsa devices."),
                                              10, NULL,
                                              NULL);
+  if (!this->mixer.name) {
+    if (this->audio_fd)
+      snd_pcm_close (this->audio_fd);
+    free(this);
+    return NULL;
+  }
 
   pthread_mutex_init(&this->mixer.mutex, NULL);
   ao_alsa_mixer_init(&this->ao_driver);
@@ -1666,6 +1665,8 @@ static void *init_class (xine_t *xine, void *data) {
   alsa_class_t        *this;
 
   this = (alsa_class_t *) xine_xmalloc (sizeof (alsa_class_t));
+  if (!this)
+    return NULL;
 
   this->driver_class.open_plugin     = open_plugin;
   this->driver_class.get_identifier  = get_identifier;
@@ -1685,7 +1686,7 @@ static ao_info_t ao_info_alsa = {
  * exported plugin catalog entry
  */
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_OUT, AO_OUT_ALSA_IFACE_VERSION, "alsa", XINE_VERSION_CODE, &ao_info_alsa, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
