@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2005 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_matroska.c,v 1.32.2.3 2005/07/17 21:49:40 miguelfreitas Exp $
+ * $Id: demux_matroska.c,v 1.49 2006/11/14 14:17:31 dgp85 Exp $
  *
  * demultiplexer for matroska streams
  *
@@ -48,6 +48,7 @@
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "demux.h"
+#include "buffer.h"
 #include "bswap.h"
 
 #include "ebml.h"
@@ -67,6 +68,13 @@
 #define MAX(a, b)	((a)>(b)?(a):(b))
 #endif
 
+/* FOURCC will be manipulated using machine endian */
+#ifdef WORDS_BIGENDIAN
+#define meFOURCC BE_FOURCC
+#else
+#define meFOURCC LE_FOURCC
+#endif
+
 typedef struct {
   int                  track_num;
   off_t               *pos;
@@ -155,7 +163,7 @@ static void check_newpts (demux_matroska_t *this, int64_t pts,
     if (pts && (this->send_newpts || (track->last_pts && abs(diff)>WRAP_THRESHOLD)) ) {
       int i;
 
-      lprintf ("sending newpts %lld, diff %lld, track %d\n", pts, diff, track->track_num);
+      lprintf ("sending newpts %" PRId64 ", diff %" PRId64 ", track %d\n", pts, diff, track->track_num);
 
       if (this->buf_flag_seek) {
         _x_demux_control_newpts(this->stream, pts, BUF_FLAG_SEEK);
@@ -171,7 +179,7 @@ static void check_newpts (demux_matroska_t *this, int64_t pts,
     } else {
   #ifdef LOG
       if (pts)
-        lprintf ("diff %lld, track %d\n", diff, track->track_num);
+        lprintf ("diff %" PRId64 ", track %d\n", diff, track->track_num);
   #endif
     }
 
@@ -247,7 +255,7 @@ static int parse_info(demux_matroska_t *this) {
     this->timecode_scale = 1000000;
   }
   this->duration = (int)(duration * (double)this->timecode_scale / 1000000.0);
-  lprintf("timecode_scale: %lld\n", this->timecode_scale);
+  lprintf("timecode_scale: %" PRId64 "\n", this->timecode_scale);
   lprintf("duration: %d\n", this->duration);
   return 1;
 }
@@ -361,7 +369,7 @@ static void init_codec_video(demux_matroska_t *this, matroska_track_t *track) {
   
   if (track->codec_private_len > buf->max_size) {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-            "demux_matroska: private decoder data length (%d) is greater than fifo buffer length (%d)\n",
+            "demux_matroska: private decoder data length (%d) is greater than fifo buffer length (%" PRId32 ")\n",
              track->codec_private_len, buf->max_size);
     buf->free_buffer(buf);
     return;
@@ -400,7 +408,7 @@ static void init_codec_audio(demux_matroska_t *this, matroska_track_t *track) {
 
   if (track->codec_private_len > buf->max_size) {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-            "demux_matroska: private decoder data length (%d) is greater than fifo buffer length (%d)\n",
+            "demux_matroska: private decoder data length (%d) is greater than fifo buffer length (%" PRId32 ")\n",
              track->codec_private_len, buf->max_size);
     buf->free_buffer(buf);
     return;
@@ -443,7 +451,7 @@ static void init_codec_real(demux_matroska_t *this, matroska_track_t * track) {
   
   if (track->codec_private_len > buf->max_size) {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-            "demux_matroska: private decoder data length (%d) is greater than fifo buffer length (%d)\n",
+            "demux_matroska: private decoder data length (%d) is greater than fifo buffer length (%" PRId32 ")\n",
              track->codec_private_len, buf->max_size);
     buf->free_buffer(buf);
     return;
@@ -497,7 +505,7 @@ static void init_codec_vorbis(demux_matroska_t *this, matroska_track_t *track) {
     
     if (frame[i] > buf->max_size) {
       xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-              "demux_matroska: private decoder data length (%d) is greater than fifo buffer length (%d)\n",
+              "demux_matroska: private decoder data length (%d) is greater than fifo buffer length (%" PRId32 ")\n",
               frame[i], buf->max_size);
       buf->free_buffer(buf);
       return;
@@ -518,6 +526,80 @@ static void init_codec_vorbis(demux_matroska_t *this, matroska_track_t *track) {
 }
 
 
+static int aac_get_sr_index (uint32_t sample_rate) {
+  if (92017 <= sample_rate)
+    return 0;
+  else if (75132 <= sample_rate)
+    return 1;
+  else if (55426 <= sample_rate)
+    return 2;
+  else if (46009 <= sample_rate)
+    return 3;
+  else if (37566 <= sample_rate)
+    return 4;
+  else if (27713 <= sample_rate)
+    return 5;
+  else if (23004 <= sample_rate)
+    return 6;
+  else if (18783 <= sample_rate)
+    return 7;
+  else if (13856 <= sample_rate)
+    return 8;
+  else if (11502 <= sample_rate)
+    return 9;
+  else if (9391 <= sample_rate)
+    return 10;
+  else
+    return 11;
+}
+
+#define AAC_SYNC_EXTENSION_TYPE 0x02b7
+static void init_codec_aac(demux_matroska_t *this, matroska_track_t *track) {
+  matroska_audio_track_t *atrack = track->audio_track;
+  buf_element_t *buf;
+  int profile;
+  int sr_index;
+
+  /* Create a DecoderSpecificInfo for initialising libfaad */
+  sr_index = aac_get_sr_index(atrack->sampling_freq);
+  if (!strncmp (&track->codec_id[12], "MAIN", 4))
+    profile = 0;
+  else if (!strncmp (&track->codec_id[12], "LC", 2))
+    profile = 1;
+  else if (!strncmp (&track->codec_id[12], "SSR", 3))
+    profile = 2;
+  else
+    profile = 3;
+
+  buf = track->fifo->buffer_pool_alloc (track->fifo);
+
+  buf->size = 0;
+  buf->type = track->buf_type;
+  buf->pts = 0;
+
+  buf->decoder_flags = BUF_FLAG_SPECIAL|BUF_FLAG_HEADER;
+  buf->decoder_info[1] = BUF_SPECIAL_DECODER_CONFIG;
+  buf->decoder_info_ptr[2] = buf->mem;
+
+  buf->mem[0] = ((profile + 1) << 3) | ((sr_index & 0x0e) >> 1);
+  buf->mem[1] = ((sr_index & 0x01) << 7) | (atrack->channels << 3);
+
+  if (strstr(track->codec_id, "SBR") == NULL)
+    buf->decoder_info[2] = 2;
+  else {
+    /* HE-AAC (aka SBR AAC) */
+    sr_index = aac_get_sr_index(atrack->sampling_freq*2);
+    buf->mem[2] = AAC_SYNC_EXTENSION_TYPE >> 3;
+    buf->mem[3] = ((AAC_SYNC_EXTENSION_TYPE & 0x07) << 5) | 5;
+    buf->mem[4] = (1 << 7) | (sr_index << 3);
+
+    buf->decoder_info[2] = 5;
+  }
+
+  track->fifo->put(track->fifo, buf);
+}
+
+
 static int vobsub_parse_size(matroska_track_t *t, const char *start) {
   if (sscanf(&start[6], "%dx%d", &t->sub_track->width,
              &t->sub_track->height) == 2) {
@@ -706,7 +788,7 @@ static void handle_realvideo (demux_plugin_t *this_gen, matroska_track_t *track,
 
     if (chunk_tab_size > buf->max_size) {
       xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-              "demux_matroska: Real Chunk Table length (%d) is greater than fifo buffer length (%d)\n",
+              "demux_matroska: Real Chunk Table length (%d) is greater than fifo buffer length (%" PRId32 ")\n",
               chunk_tab_size, buf->max_size);
       buf->free_buffer(buf);
       return;
@@ -740,7 +822,7 @@ static void handle_sub_ssa (demux_plugin_t *this_gen, matroska_track_t *track,
   int dest_len;
   int skip = 0;
 
-  lprintf ("pts: %lld, duration: %d\n", data_pts, data_duration);
+  lprintf ("pts: %" PRId64 ", duration: %d\n", data_pts, data_duration);
   /* skip ',' */
   while (data_len && (commas < 8)) {
     if (*data == ',') commas++;
@@ -1049,7 +1131,6 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
           track->track_num,
           (track->codec_id ? track->codec_id : ""),
           (track->language ? track->language : ""));
-
   if (track->codec_id) {
     void (*init_codec)(demux_matroska_t *, matroska_track_t *) = NULL;
 
@@ -1064,12 +1145,60 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       init_codec = init_codec_video;
 
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_UNCOMPRESSED)) {
-    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_SP)) {
-    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_ASP)) {
-    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_AP)) {
+    } else if ((!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_SP)) ||
+               (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_ASP)) ||
+               (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_AP))) {
+      xine_bmiheader *bih;
+      
+      lprintf("MATROSKA_CODEC_ID_V_MPEG4_*\n");
+      /* create a bitmap info header struct for MPEG 4 */
+      bih = malloc(sizeof(xine_bmiheader) + track->codec_private_len);
+      bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
+      bih->biCompression = meFOURCC('M', 'P', '4', 'S');
+      bih->biWidth = track->video_track->pixel_width;
+      bih->biHeight = track->video_track->pixel_height;
+      _x_bmiheader_le2me(bih);
+      
+      /* add bih extra data */
+      memcpy(bih + 1, track->codec_private, track->codec_private_len);
+      free(track->codec_private);
+      track->codec_private = (uint8_t *)bih;
+      track->codec_private_len = bih->biSize;
+      track->buf_type = BUF_VIDEO_MPEG4;
+      
+      /* init as a vfw decoder */
+      init_codec = init_codec_video;
+    } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG4_AVC)) {
+      xine_bmiheader *bih;
+      
+      lprintf("MATROSKA_CODEC_ID_V_MPEG4_AVC\n");
+      /* create a bitmap info header struct for h264 */
+      bih = malloc(sizeof(xine_bmiheader) + track->codec_private_len);
+      bih->biSize = sizeof(xine_bmiheader) + track->codec_private_len;
+      bih->biCompression = meFOURCC('a', 'v', 'c', '1');
+      bih->biWidth = track->video_track->pixel_width;
+      bih->biHeight = track->video_track->pixel_height;
+      _x_bmiheader_le2me(bih);
+      
+      /* add bih extra data */
+      memcpy(bih + 1, track->codec_private, track->codec_private_len);
+      free(track->codec_private);
+      track->codec_private = (uint8_t *)bih;
+      track->codec_private_len = bih->biSize;
+      track->buf_type = BUF_VIDEO_H264;
+      
+      /* init as a vfw decoder */
+      init_codec = init_codec_video;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MSMPEG4V3)) {
+      track->buf_type = BUF_VIDEO_MSMPEG4_V3;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG1)) {
+      lprintf("MATROSKA_CODEC_ID_V_MPEG1\n");
+      track->buf_type = BUF_VIDEO_MPEG;
+      init_codec = init_codec_video;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_MPEG2)) {
+      lprintf("MATROSKA_CODEC_ID_V_MPEG2\n");
+      track->buf_type = BUF_VIDEO_MPEG;
+      init_codec = init_codec_video;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_REAL_RV10)) {
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_REAL_RV20)) {
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_V_REAL_RV30)) {
@@ -1108,12 +1237,19 @@ static int parse_track_entry(demux_matroska_t *this, matroska_track_t *track) {
       init_codec = init_codec_vorbis;
 
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_ACM)) {
+      xine_waveformatex *wfh;
+      lprintf("MATROSKA_CODEC_ID_A_ACM\n");
+
+      wfh = (xine_waveformatex*)track->codec_private;
+      _x_waveformatex_le2me(wfh);
+
+      track->buf_type = _x_formattag_to_buf_audio(wfh->wFormatTag);
+      init_codec = init_codec_audio;
     } else if (!strncmp(track->codec_id, MATROSKA_CODEC_ID_A_AAC,
-                        sizeof(MATROSKA_CODEC_ID_A_AAC - 1))) {
+                        sizeof(MATROSKA_CODEC_ID_A_AAC) - 1)) {
       lprintf("MATROSKA_CODEC_ID_A_AAC\n");
       track->buf_type = BUF_AUDIO_AAC;
-      init_codec = init_codec_audio;
-      
+      init_codec = init_codec_aac;
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_REAL_14_4)) {
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_REAL_28_8)) {
     } else if (!strcmp(track->codec_id, MATROSKA_CODEC_ID_A_REAL_COOK)) {
@@ -1537,14 +1673,14 @@ static int parse_block (demux_matroska_t *this, uint64_t block_size,
   flags = *data;
   data += 1;
   
-  lprintf("track_num: %lld, timecode_diff: %d, flags: 0x%x\n", track_num, timecode_diff, flags);
+  lprintf("track_num: %" PRId64 ", timecode_diff: %d, flags: 0x%x\n", track_num, timecode_diff, flags);
 
   gap = flags & 1;
   lacing = (flags >> 1) & 0x3;
 
   if (!find_track_by_id(this, (int)track_num, &track)) {
      xprintf(this->stream->xine, XINE_VERBOSITY_LOG,
-             "demux_matroska: invalid track id: %lld\n", track_num);
+             "demux_matroska: invalid track id: %" PRId64 "\n", track_num);
      return 0;
   }
 
@@ -1568,7 +1704,7 @@ static int parse_block (demux_matroska_t *this, uint64_t block_size,
     block_duration = track->default_duration;
     xduration = (int64_t)block_duration * (int64_t)90 / (int64_t)1000000;
   }
-  lprintf("pts: %lld, duration: %lld\n", pts, xduration);
+  lprintf("pts: %" PRId64 ", duration: %" PRId64 "\n", pts, xduration);
 
   check_newpts(this, pts, track);
 
@@ -1582,7 +1718,7 @@ static int parse_block (demux_matroska_t *this, uint64_t block_size,
     lprintf("no lacing\n");
 
     block_size_left = (this->block_data + block_size) - data;
-    lprintf("size: %d, block_size: %lld\n", block_size_left, block_size);
+    lprintf("size: %d, block_size: %" PRIu64 "\n", block_size_left, block_size);
 
     if (track->handle_content != NULL) {
       track->handle_content((demux_plugin_t *)this, track,
@@ -1622,12 +1758,13 @@ static int parse_block (demux_matroska_t *this, uint64_t block_size,
         /* size of each frame */
         for (i = 0; i < lace_num; i++) {
           int size = 0;
-          while (*data == 255) {
-            size += *data;
+          int partial_size;
+          do
+          {
+            partial_size = *data;
+            size += partial_size;
             data++; block_size_left--;
-          }
-          size += *data;
-          data++; block_size_left--;
+          } while (partial_size == 255);
           frame[i] = size;
           block_size_left -= size;
         }
@@ -1637,14 +1774,19 @@ static int parse_block (demux_matroska_t *this, uint64_t block_size,
       }
       break;
 
-      case MATROSKA_FIXED_SIZE_LACING:
+      case MATROSKA_FIXED_SIZE_LACING: {
+        int frame_size;
+
         lprintf("fixed size lacing\n");
+        
+        frame_size = block_size_left / (lace_num + 1);
         for (i = 0; i < lace_num; i++) {
-          frame[i] = block_size / (lace_num + 1);
-          block_size_left -= frame[i];
+          frame[i] = frame_size;
         }
-        frame[lace_num] = block_size_left;
-        break;
+        frame[lace_num] = block_size_left - (lace_num * frame_size);
+        block_size_left = 0;
+      }
+      break;
 
       case MATROSKA_EBML_LACING: {
         int64_t tmp;
@@ -1735,7 +1877,7 @@ static int parse_block_group(demux_matroska_t *this,
         /* should override track duration */
         if (!ebml_read_uint(ebml, &elem, &block_duration))
           return 0;
-        lprintf("duration: %lld\n", block_duration);
+        lprintf("duration: %" PRIu64 "\n", block_duration);
         break;
       case MATROSKA_ID_CL_REFERENCEBLOCK:
         is_key = 0;
@@ -1825,7 +1967,8 @@ static int parse_seek_entry(demux_matroska_t *this) {
   int next_level = 3;
   int has_id = 0;
   int has_position = 0;
-  uint64_t id, pos;
+  uint64_t id = 0;
+  uint64_t pos;
   
   while (next_level == 3) {
     ebml_elem_t elem;
@@ -1950,7 +2093,7 @@ static int parse_top_level_head(demux_matroska_t *this, int *next_level) {
   off_t current_pos;
   
   current_pos = this->input->get_current_pos(this->input);
-  lprintf("current_pos: %lld\n", current_pos);
+  lprintf("current_pos: %" PRIdMAX "\n", (intmax_t)current_pos);
   
   if (!ebml_read_elem_head(ebml, &elem))
     return 0;
@@ -2319,16 +2462,16 @@ static int demux_matroska_seek (demux_plugin_t *this_gen,
 
   entry = binary_seek(index, start_pos, start_time);
   if (entry == -1) {
-    lprintf("seeking for track %d to %s %lld - no entry found/EOS.\n",
+    lprintf("seeking for track %d to %s %" PRIdMAX " - no entry found/EOS.\n",
             index->track_num, start_pos ? "pos" : "time",
-            start_pos ? (int64_t)start_pos : (int64_t)start_time);
+            start_pos ? (intmax_t)start_pos : (intmax_t)start_time);
     this->status = DEMUX_FINISHED;
 
   } else {
-    lprintf("seeking for track %d to %s %lld. decision is #%d at %lld/%lld\n",
+    lprintf("seeking for track %d to %s %" PRIdMAX ". decision is #%d at %" PRIu64 "/%" PRIdMAX "\n",
             index->track_num, start_pos ? "pos" : "time",
-            start_pos ? (int64_t)start_pos : (int64_t)start_time,
-            index->track_num, index->timecode[entry], index->pos[entry]);
+            start_pos ? (intmax_t)start_pos : (intmax_t)start_time,
+            index->track_num, index->timecode[entry], (intmax_t)index->pos[entry]);
     
     if (this->input->seek(this->input, index->pos[entry], SEEK_SET) < 0)
       this->status = DEMUX_FINISHED;
@@ -2534,8 +2677,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 
 error:
-  if (ebml)
-    dispose_ebml_parser(ebml);
+  dispose_ebml_parser(ebml);
   free(this);
   return NULL;
 }
@@ -2592,11 +2734,11 @@ static void *init_class (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_matroska = {
+static const demuxer_info_t demux_info_matroska = {
   10                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_DEMUX, 26, "matroska", XINE_VERSION_CODE, &demux_info_matroska, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
