@@ -19,7 +19,7 @@
  */
 
 /*
- * $Id: demux_mng.c,v 1.27 2006/06/02 22:18:56 dsalt Exp $
+ * $Id: demux_mng.c,v 1.28 2006/07/10 22:08:13 dgp85 Exp $
  *
  * demux_mng.c, Demuxer plugin for Multiple-image Network Graphics format
  *
@@ -378,7 +378,7 @@ static const demuxer_info_t demux_info_mng = {
   10                       /* priority */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   { PLUGIN_DEMUX, 26, "mng", XINE_VERSION_CODE, &demux_info_mng, (void*)init_plugin},
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
