@@ -23,7 +23,7 @@
  * For more information regarding the PVA file format, refer to this PDF:
  *   http://www.technotrend.de/download/av_format_v1.pdf
  *
- * $Id: demux_pva.c,v 1.23 2006/06/02 22:18:56 dsalt Exp $
+ * $Id: demux_pva.c,v 1.24 2006/07/10 22:08:13 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -524,7 +524,7 @@ static const demuxer_info_t demux_info_pva = {
   10                       /* priority */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_DEMUX, 26, "pva", XINE_VERSION_CODE, &demux_info_pva, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
