@@ -31,7 +31,7 @@
  *   
  *   Based on FFmpeg's libav/rm.c.
  *
- * $Id: demux_real.c,v 1.103.2.2 2005/07/17 21:49:58 miguelfreitas Exp $
+ * $Id: demux_real.c,v 1.109 2006/07/10 22:08:13 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -781,8 +781,6 @@ static int demux_real_parse_references( demux_real_t *this) {
   int             len, i, j;
   int             alternative = 0;
   int             comment = 0;
-  xine_mrl_reference_data_t *data;
-  xine_event_t    uevent;
 
 
   lprintf("parsing references\n");
@@ -831,15 +829,8 @@ static int demux_real_parse_references( demux_real_t *this) {
       buf[j]='\0';
       lprintf("reference [%s] found\n", &buf[i]);
 
-      uevent.type = XINE_EVENT_MRL_REFERENCE;
-      uevent.stream = this->stream;
-      uevent.data_length = strlen(&buf[i])+sizeof(xine_mrl_reference_data_t);
-      data = malloc(uevent.data_length);
-      uevent.data = data;
-      strcpy(data->mrl, &buf[i]);
-      data->alternative = alternative;
-      xine_event_send(this->stream, &uevent);
-      free(data);
+      _x_demux_send_mrl_reference (this->stream, alternative,
+                                   &buf[i], NULL, 0, 0);
 
       i = j;
     }
@@ -1660,11 +1651,11 @@ static void *init_class (xine_t *xine, void *data) {
 /*
  * exported plugin catalog entry
  */
-demuxer_info_t demux_info_real = {
+static const demuxer_info_t demux_info_real = {
   10                       /* priority */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_DEMUX, 26, "real", XINE_VERSION_CODE, &demux_info_real, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
