@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2002 the xine project
+ * Copyright (C) 2001-2003 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -16,11 +16,13 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
+ */
+
+/*
  * RealAudio File Demuxer by Mike Melanson (melanson@pcisys.net)
+ *     improved by James Stembridge (jstembridge@users.sourceforge.net)
  *
- * $Id: demux_realaudio.c,v 1.7 2002/12/08 21:43:51 miguelfreitas Exp $
- *
+ * $Id: demux_realaudio.c,v 1.32 2004/06/13 21:28:54 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,57 +41,48 @@
 #include "demux.h"
 #include "buffer.h"
 #include "bswap.h"
+#include "group_audio.h"
 
-#define RA_FILE_HEADER_SIZE 16
-#define RA_AUDIO_HEADER_SIZE 0x39
+#define RA_FILE_HEADER_PREV_SIZE 22
 
 typedef struct {
-
   demux_plugin_t       demux_plugin;
 
   xine_stream_t       *stream;
-
-  config_values_t     *config;
-
   fifo_buffer_t       *video_fifo;
   fifo_buffer_t       *audio_fifo;
-
   input_plugin_t      *input;
-
   int                  status;
 
-  xine_waveformatex    wave;
-  unsigned int         audio_fourcc;
+  unsigned int         fourcc;
   unsigned int         audio_type;
 
   off_t                data_start;
   off_t                data_size;
+  
+  unsigned short       block_align;
+  unsigned int         bytes_per_sec;
+  
+  unsigned char       *header;
+  unsigned int         header_size;
 
   int                  seek_flag;  /* this is set when a seek just occurred */
-
-  char                 last_mrl[1024];
 } demux_ra_t;
 
 typedef struct {
-
   demux_class_t     demux_class;
-
-  /* class-wide, global variables here */
-
-  xine_t           *xine;
-  config_values_t  *config;
 } demux_ra_class_t;
 
 /* returns 1 if the RealAudio file was opened successfully, 0 otherwise */
 static int open_ra_file(demux_ra_t *this) {
-
-  unsigned char file_header[RA_FILE_HEADER_SIZE];
-  unsigned char audio_header[RA_AUDIO_HEADER_SIZE];
+  unsigned char   file_header[RA_FILE_HEADER_PREV_SIZE], len;
+  unsigned short  version;
+  off_t           offset;
+  
 
   /* check the signature */
-  this->input->seek(this->input, 0, SEEK_SET);
-  if (this->input->read(this->input, file_header, RA_FILE_HEADER_SIZE) !=
-    RA_FILE_HEADER_SIZE)
+  if (_x_demux_read_header(this->input, file_header, RA_FILE_HEADER_PREV_SIZE) !=
+      RA_FILE_HEADER_PREV_SIZE)
     return 0;
 
   if ((file_header[0] != '.') ||
@@ -97,80 +90,141 @@ static int open_ra_file(demux_ra_t *this) {
       (file_header[2] != 'a'))
     return 0;
 
-  /* load the audio header */
-  if (this->input->read(this->input, audio_header, RA_AUDIO_HEADER_SIZE) !=
-    RA_AUDIO_HEADER_SIZE)
+  /* read version */
+  version = BE_16(&file_header[0x04]);
+  
+  /* read header size according to version */
+  if (version == 3)
+    this->header_size = BE_16(&file_header[0x06]) + 8;
+  else if (version == 4)
+    this->header_size = BE_32(&file_header[0x12]) + 16;
+  else {
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_realaudio: unknown version number %d\n", version);
     return 0;
+  }
+    
+  /* allocate for and read header data */
+  this->header = xine_xmalloc(this->header_size);
+  
+  if (_x_demux_read_header(this->input, this->header, this->header_size) != this->header_size) {
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_realaudio: unable to read header\n");
+    free(this->header);
+    return 0;
+  }
+    
+  /* read header data according to version */
+  if((version == 3) && (this->header_size >= 32)) {
+    this->data_size = BE_32(&this->header[0x12]);
+    
+    this->block_align = 240;
+    
+    offset = 0x16;
+  } else if(this->header_size >= 72) {
+    this->data_size = BE_32(&this->header[0x1C]);    
+    
+    this->block_align = BE_16(&this->header[0x2A]);
+    
+    if(this->header[0x3D] == 4)
+      this->fourcc = ME_32(&this->header[0x3E]);
+    else {
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	      "demux_realaudio: invalid fourcc size %d\n", this->header[0x3D]);
+      free(this->header);
+      return 0;
+    }
+    
+    offset = 0x45;
+  } else {
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_realaudio: header too small\n");
+    free(this->header);
+    return 0;
+  }
+  
+  /* Read title */
+  len = this->header[offset];
+  if(len && ((offset+len+2) < this->header_size)) {
+    _x_meta_info_n_set(this->stream, XINE_META_INFO_TITLE,
+                        &this->header[offset+1], len);
+    offset += len+1;
+  } else
+    offset++;
+  
+  /* Author */
+  len = this->header[offset];
+  if(len && ((offset+len+1) < this->header_size)) {
+    _x_meta_info_n_set(this->stream, XINE_META_INFO_ARTIST,
+                        &this->header[offset+1], len);
+    offset += len+1;
+  } else
+    offset++;
+  
+  /* Copyright/Date */
+  len = this->header[offset];
+  if(len && ((offset+len) <= this->header_size)) {
+    _x_meta_info_n_set(this->stream, XINE_META_INFO_YEAR,
+                        &this->header[offset+1], len);
+    offset += len+1;
+  } else
+    offset++;
+  
+  /* Fourcc for version 3 comes after meta info */
+  if((version == 3) && ((offset+7) <= this->header_size)) {
+    if(this->header[offset+2] == 4)
+      this->fourcc = ME_32(&this->header[offset+3]);
+    else {
+      xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+	      "demux_realaudio: invalid fourcc size %d\n", this->header[offset+2]);
+      free(this->header);
+      return 0;
+    }
+  }
+  
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_FOURCC, this->fourcc);
+  this->audio_type = _x_formattag_to_buf_audio(this->fourcc);
 
-  /* find the important information */
-  this->data_start = RA_FILE_HEADER_SIZE + RA_AUDIO_HEADER_SIZE;
-  this->data_size = BE_32(&audio_header[0x0C]);
-  this->wave.nChannels = audio_header[0x27];
-  this->wave.nSamplesPerSec = BE_16(&audio_header[0x20]);
-  this->wave.nBlockAlign = BE_16(&audio_header[0x1A]);
-  this->wave.wBitsPerSample = audio_header[0x25];
-  this->audio_fourcc = *(unsigned int *)&audio_header[0x2E];
-  this->audio_type = formattag_to_buf_audio(this->audio_fourcc);
+  /* seek to start of data */
+  this->data_start = this->header_size;
+  if (this->input->seek(this->input, this->data_start, SEEK_SET) !=
+      this->data_start) {
+    xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "demux_realaudio: unable to seek to data start\n");
+    return 0;
+  }
+
+  if( !this->audio_type )
+    this->audio_type = BUF_AUDIO_UNKNOWN;
 
   return 1;
 }
 
 static int demux_ra_send_chunk(demux_plugin_t *this_gen) {
-
   demux_ra_t *this = (demux_ra_t *) this_gen;
-  buf_element_t *buf = NULL;
-  unsigned int remaining_sample_bytes;
-  off_t current_file_pos;
+
+  off_t current_normpos = 0;
   int64_t current_pts;
 
   /* just load data chunks from wherever the stream happens to be
    * pointing; issue a DEMUX_FINISHED status if EOF is reached */
-  remaining_sample_bytes = this->wave.nBlockAlign;
-  current_file_pos =
-    this->input->get_current_pos(this->input) - this->data_start;
-
-
+  if( this->input->get_length (this->input) )
+    current_normpos = (int)( (double) (this->input->get_current_pos (this->input) - this->data_start) * 
+                      65535 / this->data_size );
 
   current_pts = 0;  /* let the engine sort out the pts for now */
 
-
-
   if (this->seek_flag) {
-    xine_demux_control_newpts(this->stream, current_pts, 0);
+    _x_demux_control_newpts(this->stream, current_pts, BUF_FLAG_SEEK);
     this->seek_flag = 0;
   }
 
-  while (remaining_sample_bytes) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type = this->audio_type;
-    buf->input_pos = current_file_pos;
-    buf->input_length = this->data_size;
-    buf->input_time = current_pts / 90000;
-    buf->pts = current_pts;
-
-    if (remaining_sample_bytes > buf->max_size)
-      buf->size = buf->max_size;
-    else
-      buf->size = remaining_sample_bytes;
-    remaining_sample_bytes -= buf->size;
-
-    if (this->input->read(this->input, buf->content, buf->size) !=
-      buf->size) {
-      buf->free_buffer(buf);
-      this->status = DEMUX_FINISHED;
-      break;
-    }
-
-    if (!remaining_sample_bytes)
-      buf->decoder_flags |= BUF_FLAG_FRAME_END;
-
-    this->audio_fifo->put (this->audio_fifo, buf);
+  if(_x_demux_read_send_data(this->audio_fifo, this->input, this->block_align, 
+                             current_pts, this->audio_type, 0, current_normpos, 
+                             current_pts / 90, 0, 0) < 0) {
+    this->status = DEMUX_FINISHED;                           
   }
+  
   return this->status;
 }
 
 static void demux_ra_send_headers(demux_plugin_t *this_gen) {
-
   demux_ra_t *this = (demux_ra_t *) this_gen;
   buf_element_t *buf;
 
@@ -180,37 +234,45 @@ static void demux_ra_send_headers(demux_plugin_t *this_gen) {
   this->status = DEMUX_OK;
 
   /* load stream information */
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 0;
-  this->stream->stream_info[XINE_STREAM_INFO_HAS_AUDIO] = 1;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_CHANNELS] =
-    this->wave.nChannels;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_SAMPLERATE] =
-    this->wave.nSamplesPerSec;
-  this->stream->stream_info[XINE_STREAM_INFO_AUDIO_BITS] =
-    this->wave.wBitsPerSample;
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_VIDEO, 0);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_HAS_AUDIO, 1);
+  _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_FOURCC, this->fourcc);
 
   /* send start buffers */
-  xine_demux_control_start(this->stream);
+  _x_demux_control_start(this->stream);
 
   /* send init info to decoders */
   if (this->audio_fifo && this->audio_type) {
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type = this->audio_type;
-    buf->decoder_flags = BUF_FLAG_HEADER;
-    buf->decoder_info[0] = 0;
-    buf->decoder_info[1] = this->wave.nSamplesPerSec;
-    buf->decoder_info[2] = this->wave.wBitsPerSample;
-    buf->decoder_info[3] = this->wave.nChannels;
-    memcpy(buf->content, &this->wave, sizeof(this->wave));
-    buf->size = sizeof(this->wave);
+    buf->decoder_flags = BUF_FLAG_HEADER|BUF_FLAG_FRAME_END;
+    
+    if(this->header_size > buf->max_size)
+      buf->size = buf->max_size;
+    else
+      buf->size = this->header_size;
+    
+    memcpy(buf->content, this->header, buf->size);
+
     this->audio_fifo->put (this->audio_fifo, buf);
   }
 }
 
 static int demux_ra_seek (demux_plugin_t *this_gen,
-                           off_t start_pos, int start_time) {
+                           off_t start_pos, int start_time, int playing) {
 
   demux_ra_t *this = (demux_ra_t *) this_gen;
+  start_pos = (off_t) ( (double) start_pos / 65535 *
+              this->data_size );
+
+  this->seek_flag = 1;
+  this->status = DEMUX_OK;
+  _x_demux_flush_engine (this->stream);
+
+  /* if input is non-seekable, do not proceed with the rest of this
+   * seek function */
+  if (!INPUT_IS_SEEKABLE(this->input))
+    return this->status;
 
   /* check the boundary offsets */
   if (start_pos <= 0)
@@ -224,29 +286,22 @@ static int demux_ra_seek (demux_plugin_t *this_gen,
      * the block alignment integer-wise, and multiply the quotient by the
      * block alignment to get the new aligned offset. Add the data start
      * offset and seek to the new position. */
-    start_pos /= this->wave.nBlockAlign;
-    start_pos *= this->wave.nBlockAlign;
+    start_pos /= this->block_align;
+    start_pos *= this->block_align;
     start_pos += this->data_start;
 
     this->input->seek(this->input, start_pos, SEEK_SET);
   }
 
-  this->seek_flag = 1;
-  this->status = DEMUX_OK;
-  xine_demux_flush_engine (this->stream);
-  
-  /* if thread is not running, initialize demuxer */
-  if( !this->stream->demux_thread_running ) {
-
-    this->status = DEMUX_OK;
-  }
-
   return this->status;
 }
 
 
 static void demux_ra_dispose (demux_plugin_t *this_gen) {
   demux_ra_t *this = (demux_ra_t *) this_gen;
+  
+  if(this->header)
+    free(this->header);
 
   free(this);
 }
@@ -257,12 +312,14 @@ static int demux_ra_get_status (demux_plugin_t *this_gen) {
   return this->status;
 }
 
-/* return the approximate length in seconds */
+/* return the approximate length in miliseconds */
 static int demux_ra_get_stream_length (demux_plugin_t *this_gen) {
-
   demux_ra_t *this = (demux_ra_t *) this_gen;
 
-  return (int)(this->data_size / this->wave.nAvgBytesPerSec);
+  if(this->bytes_per_sec)
+    return (int)((int64_t) this->data_size * 1000 / this->bytes_per_sec);
+  else
+    return 0;
 }
 
 static uint32_t demux_ra_get_capabilities(demux_plugin_t *this_gen) {
@@ -275,16 +332,10 @@ static int demux_ra_get_optional_data(demux_plugin_t *this_gen,
 }
 
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
-                                    input_plugin_t *input_gen) {
+                                    input_plugin_t *input) {
 
-  input_plugin_t *input = (input_plugin_t *) input_gen;
   demux_ra_t     *this;
 
-  if (! (input->get_capabilities(input) & INPUT_CAP_SEEKABLE)) {
-    printf(_("demux_ra.c: input not seekable, can not handle!\n"));
-    return NULL;
-  }
-
   this         = xine_xmalloc (sizeof (demux_ra_t));
   this->stream = stream;
   this->input  = input;
@@ -295,8 +346,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.dispose           = demux_ra_dispose;
   this->demux_plugin.get_status        = demux_ra_get_status;
   this->demux_plugin.get_stream_length = demux_ra_get_stream_length;
-  this->demux_plugin.get_video_frame   = NULL;
-  this->demux_plugin.got_video_frame_cb= NULL;
   this->demux_plugin.get_capabilities  = demux_ra_get_capabilities;
   this->demux_plugin.get_optional_data = demux_ra_get_optional_data;
   this->demux_plugin.demux_class       = class_gen;
@@ -305,6 +354,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
+  case METHOD_BY_EXTENSION: {
+    char *extensions, *mrl;
+
+    mrl = input->get_mrl (input);
+    extensions = class_gen->get_extensions (class_gen);
+
+    if (!_x_demux_check_extension (mrl, extensions)) {
+      free (this);
+      return NULL;
+    }
+  }
+  /* falling through is intended */
+
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -314,51 +376,12 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
     }
 
   break;
-
-  case METHOD_BY_EXTENSION: {
-    char *ending, *mrl;
-
-    mrl = input->get_mrl (input);
-
-    ending = strrchr(mrl, '.');
-
-    if (!ending) {
-      free (this);
-      return NULL;
-    }
-
-    if (strncasecmp (ending, ".ra", 3)) {
-      free (this);
-      return NULL;
-    }
-
-    if (!open_ra_file(this)) {
-      free (this);
-      return NULL;
-    }
-
-  }
-
-  break;
-
+  
   default:
     free (this);
     return NULL;
   }
 
-  strncpy (this->last_mrl, input->get_mrl (input), 1024);
-
-  /* print vital stats */
-  xine_log (this->stream->xine, XINE_LOG_MSG,
-    _("demux_realaudio: RealAudio file, %d bits, %d Hz, %s %c%c%c%c audio\n"),
-    this->wave.wBitsPerSample,
-    this->wave.nSamplesPerSec,
-    (this->wave.nChannels == 1) ? "monaural" : "stereo",
-    *((char *)&this->audio_fourcc + 0),
-    *((char *)&this->audio_fourcc + 1),
-    *((char *)&this->audio_fourcc + 2),
-    *((char *)&this->audio_fourcc + 3));
-
   return &this->demux_plugin;
 }
 
@@ -375,23 +398,19 @@ static char *get_extensions (demux_class_t *this_gen) {
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
-  return NULL;
+  return "audio/x-realaudio: ra: RealAudio File;";
 }
 
 static void class_dispose (demux_class_t *this_gen) {
-
   demux_ra_class_t *this = (demux_ra_class_t *) this_gen;
 
   free (this);
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
-
+void *demux_realaudio_init_plugin (xine_t *xine, void *data) {
   demux_ra_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_ra_class_t));
-  this->config = xine->config;
-  this->xine   = xine;
+  this = xine_xmalloc (sizeof (demux_ra_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
@@ -402,14 +421,3 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   return this;
 }
-
-/*
- * exported plugin catalog entry
- */
-
-plugin_info_t xine_plugin_info[] = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 18, "realaudio", XINE_VERSION_CODE, NULL, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
