@@ -21,7 +21,7 @@
  */
 
 /*
- * $Id: demux_slave.c,v 1.20 2006/06/02 22:18:57 dsalt Exp $
+ * $Id: demux_slave.c,v 1.21 2006/07/10 22:08:13 dgp85 Exp $
  *
  * demuxer for slave "protocol"
  * master xine must be started with XINE_PARAM_BROADCASTER_PORT set, that is,
@@ -437,7 +437,7 @@ static const demuxer_info_t demux_info_slave = {
   10                       /* priority */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_DEMUX, 26, "slave", XINE_VERSION_CODE, &demux_info_slave, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
