@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_dvd.c,v 1.207 2006/05/03 19:46:07 dsalt Exp $
+ * $Id: input_dvd.c,v 1.211 2006/10/29 19:39:39 hadess Exp $
  *
  */
 
@@ -999,7 +999,8 @@ static void dvd_handle_events(dvd_input_plugin_t *this) {
       break;
     case XINE_EVENT_INPUT_MENU3:
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "input_dvd: MENU3 key hit.\n");
-      dvdnav_menu_call(this->dvdnav, DVD_MENU_Root);
+      if (dvdnav_menu_call(this->dvdnav, DVD_MENU_Root) == DVDNAV_STATUS_ERR)
+        dvdnav_menu_call(this->dvdnav, DVD_MENU_Title);
       break;
     case XINE_EVENT_INPUT_MENU4:
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, "input_dvd: MENU4 key hit.\n");
@@ -1547,7 +1548,7 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
 
     tt = strtol(title_part, NULL, 10);
     dvdnav_get_number_of_titles(this->dvdnav, &titles);
-    if((tt <= 0) || (tt > titles)) {
+    if((tt < 0) || (tt > titles)) {
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
 	      "input_dvd: Title %i is out of range (1 to %i).\n", tt, titles);
       dvdnav_close(this->dvdnav);
@@ -1560,7 +1561,7 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
     if(delimiter) {
       pr = strtol(delimiter+1, NULL, 10);
       dvdnav_get_number_of_parts(this->dvdnav, tt, &parts);
-      if ((pr <= 0) || (pr > parts)) {
+      if ((pr < 0) || (pr > parts)) {
 	xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
 		"input_dvd: Part %i is out of range (1 to %i).\n", pr, parts);
 	dvdnav_close(this->dvdnav);
@@ -1572,11 +1573,13 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
 #ifdef INPUT_DEBUG
     printf("input_dvd: Jumping to TT >%i<, PTT >%i<\n", tt, pr);
 #endif
-    if(pr != -1) {
-      dvdnav_part_play(this->dvdnav, tt, pr);
-    } else {
-      dvdnav_title_play(this->dvdnav, tt);
-    }
+    if (tt > 0) {
+      if (pr > 0)
+        dvdnav_part_play(this->dvdnav, tt, pr);
+      else
+        dvdnav_title_play(this->dvdnav, tt);
+    } else
+      this->mode = MODE_NAVIGATE;
   }
 #ifdef INPUT_DEBUG
   printf("input_dvd: DVD device successfully opened.\n");
@@ -1783,8 +1786,13 @@ static void *init_class (xine_t *xine, void *data) {
 					     _("The path to the device, usually a "
 					       "DVD drive, which you intend to use for playing DVDs."),
 					     10, device_change_cb, (void *)this);
-  
-  if ((dvdcss = dlopen("libdvdcss.so.2", RTLD_LAZY)) != NULL) {
+
+#ifdef HOST_OS_DARWIN
+  if ((dvdcss = dlopen("libdvdcss.2.dylib", RTLD_LAZY)) != NULL)
+#else
+  if ((dvdcss = dlopen("libdvdcss.so.2", RTLD_LAZY)) != NULL)
+#endif
+  {
     /* we have found libdvdcss, enable the specific config options */
     char *raw_device;
     static char *decrypt_modes[] = { "key", "disc", "title", NULL };
@@ -1906,7 +1914,7 @@ static void *init_class (xine_t *xine, void *data) {
 }
 
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "DVD", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
