@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2005 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_file.c,v 1.98.2.2 2005/07/17 21:49:59 miguelfreitas Exp $
+ * $Id: input_file.c,v 1.117 2006/10/01 20:14:43 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,7 +26,9 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -34,6 +36,10 @@
 #include <string.h>
 #include <errno.h>
 
+#ifdef HAVE_MMAP
+#include <sys/mman.h>
+#endif
+
 #define LOG_MODULE "input_file"
 #define LOG_VERBOSE
 /*
@@ -74,6 +80,12 @@ typedef struct {
   xine_stream_t    *stream;
 
   int               fh;
+#ifdef HAVE_MMAP
+  int               mmap_on;
+  void             *mmap_base;
+  void             *mmap_curr;
+  off_t             mmap_len;
+#endif
   char             *mrl;
 
 } file_input_plugin_t;
@@ -102,48 +114,134 @@ static uint32_t file_plugin_get_capabilities (input_plugin_t *this_gen) {
 #endif /* _MSC_VER */
 }
 
+#ifdef HAVE_MMAP
+/**
+ * @brief Check if the file can be read through mmap().
+ * @param this The instance of the input plugin to check
+ *             with
+ * @return 1 if the file can still be mmapped, 0 if the file
+ *         changed size
+ */
+static int check_mmap_file(file_input_plugin_t *this) {
+  struct stat          sbuf;
+
+  if ( ! this->mmap_on ) return 0;
+
+  if ( fstat (this->fh, &sbuf) != 0 ) {
+    return 0;
+  }
+
+  /* If the file grew, we're most likely dealing with a timeshifting recording
+   * so switch to normal access. */
+  if ( this->mmap_len != sbuf.st_size ) {
+    this->mmap_on = 0;
+
+    lseek(this->fh, this->mmap_curr - this->mmap_base, SEEK_SET);
+    return 0;
+  }
+
+  return 1;
+}
+#endif
 
 static off_t file_plugin_read (input_plugin_t *this_gen, char *buf, off_t len) {
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
+#ifdef HAVE_MMAP
+  if ( check_mmap_file(this) ) {
+    off_t l = len;
+    if ( (this->mmap_curr + len) > (this->mmap_base + this->mmap_len) )
+      l = (this->mmap_base + this->mmap_len) - this->mmap_curr;
+    
+    memcpy(buf, this->mmap_curr, l);
+    this->mmap_curr += l;
+    
+    return l;
+  }
+#endif
+
   return read (this->fh, buf, len);
 }
 
 static buf_element_t *file_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo, off_t todo) {
 
-  off_t                 num_bytes, total_bytes;
   file_input_plugin_t  *this = (file_input_plugin_t *) this_gen;
   buf_element_t        *buf = fifo->buffer_pool_alloc (fifo);
 
-  buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
-  total_bytes = 0;
 
-  while (total_bytes < todo) {
-    num_bytes = read (this->fh, buf->mem + total_bytes, todo-total_bytes);
-    if (num_bytes <= 0) {
-      if (num_bytes < 0) {
-	xine_log (this->stream->xine, XINE_LOG_MSG,
-		  _("input_file: read error (%s)\n"), strerror(errno));
-	_x_message(this->stream, XINE_MSG_READ_ERROR,
+#ifdef HAVE_MMAP
+  if ( check_mmap_file(this) ) {
+    off_t len = todo;
+
+    if ( (this->mmap_curr + len) > (this->mmap_base + this->mmap_len) )
+      len = (this->mmap_base + this->mmap_len) - this->mmap_curr;
+
+    /* We use the still-mmapped file rather than copying it */
+    buf->size = len;
+    buf->content = this->mmap_curr;
+
+    /* FIXME: it's completely illegal to free buffer->mem here
+     * - buffer->mem has not been allocated by malloc
+     * - demuxers expect buffer->mem != NULL 
+     */
+    /* free(buf->mem); buf->mem = NULL; */
+
+    this->mmap_curr += len;
+  } else
+#endif
+  {
+    off_t num_bytes, total_bytes = 0;
+    
+    buf->content = buf->mem;
+
+    while (total_bytes < todo) {
+      num_bytes = read (this->fh, buf->mem + total_bytes, todo-total_bytes);
+      if (num_bytes <= 0) {
+	if (num_bytes < 0) {
+	  xine_log (this->stream->xine, XINE_LOG_MSG,
+		    _("input_file: read error (%s)\n"), strerror(errno));
+	  _x_message(this->stream, XINE_MSG_READ_ERROR,
                      this->mrl, NULL);
+	}
+	buf->free_buffer (buf);
+	buf = NULL;
+	break;
       }
-      buf->free_buffer (buf);
-      buf = NULL;
-      break;
+      total_bytes += num_bytes;
     }
-    total_bytes += num_bytes;
+    
+    if( buf != NULL )
+      buf->size = total_bytes;
   }
-
-  if( buf != NULL )
-    buf->size = total_bytes;
-
+  
   return buf;
 }
 
 static off_t file_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
+#ifdef HAVE_MMAP /* Simulate f*() library calls */
+  if ( check_mmap_file(this) ) {
+    void *new_point = this->mmap_curr;
+    switch(origin) {
+    case SEEK_SET: new_point = this->mmap_base + offset; break;
+    case SEEK_CUR: new_point = this->mmap_curr + offset; break;
+    case SEEK_END: new_point = this->mmap_base + this->mmap_len + offset; break;
+    default:
+      errno = EINVAL;
+      return (off_t)-1;
+    }
+    if ( new_point < this->mmap_base || new_point > (this->mmap_base + this->mmap_len) ) {
+      errno = EINVAL;
+      return (off_t)-1;
+    }
+
+    this->mmap_curr = new_point;
+    return (this->mmap_curr - this->mmap_base);
+  }
+#endif
+
   return lseek (this->fh, offset, origin);
 }
 
@@ -153,6 +251,11 @@ static off_t file_plugin_get_current_pos (input_plugin_t *this_gen){
   if (this->fh <0)
     return 0;
 
+#ifdef HAVE_MMAP
+  if ( check_mmap_file(this) )
+    return (this->mmap_curr - this->mmap_base);
+#endif
+
   return lseek (this->fh, 0, SEEK_CUR);
 }
 
@@ -164,6 +267,11 @@ static off_t file_plugin_get_length (input_plugin_t *this_gen) {
   if (this->fh <0)
     return 0;
 
+#ifdef HAVE_MMAP
+  if ( check_mmap_file(this) )
+    return this->mmap_len;
+#endif
+
   if (fstat (this->fh, &buf) == 0) {
     return buf.st_size;
   } else
@@ -172,6 +280,11 @@ static off_t file_plugin_get_length (input_plugin_t *this_gen) {
 }
 
 static uint32_t file_plugin_get_blocksize (input_plugin_t *this_gen) {
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+#if 0 && defined(HAVE_MMAP)
+  if ( check_mmap_file(this) )
+    return this->mmap_len;
+#endif
   return 0;
 }
 
@@ -201,6 +314,15 @@ static int file_plugin_get_optional_data (input_plugin_t *this_gen,
 static void file_plugin_dispose (input_plugin_t *this_gen ) {
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
+#ifdef HAVE_MMAP
+  /* Check for mmap_base rather than mmap_on because the file might have
+   * started as a mmap() and now might be changed to descriptor-based
+   * access
+   */
+  if ( this->mmap_base )
+    munmap(this->mmap_base, this->mmap_len);
+#endif
+
   if (this->fh != -1)
     close(this->fh);
 
@@ -281,28 +403,44 @@ static int file_plugin_open (input_plugin_t *this_gen ) {
         _x_message(this->stream, XINE_MSG_PERMISSION_ERROR, this->mrl, NULL);
 	xine_log (this->stream->xine, XINE_LOG_MSG,
 		  _("input_file: Permission denied: >%s<\n"), this->mrl);
+	return -1;
       } else if (errno == ENOENT) {
         _x_message(this->stream, XINE_MSG_FILE_NOT_FOUND, this->mrl, NULL);
 	xine_log (this->stream->xine, XINE_LOG_MSG,
 		  _("input_file: File not found: >%s<\n"), this->mrl);
       }
 
-      return 0;
+      return -1;
     }
   }
 
-  /* don't check length of fifo */
+#ifdef HAVE_MMAP
+  this->mmap_on = 0;
+  this->mmap_base = NULL;
+  this->mmap_curr = NULL;
+  this->mmap_len = 0;
+#endif
+
+  /* don't check length of fifo or character device node */
   if (fstat (this->fh, &sbuf) == 0) {
-    if (S_ISFIFO(sbuf.st_mode))
+    if (!S_ISREG(sbuf.st_mode))
       return 1;
   }
 
+#ifdef HAVE_MMAP
+  if ( (this->mmap_base = mmap(NULL, sbuf.st_size, PROT_READ, MAP_SHARED, this->fh, 0)) != (void*)-1 ) {
+    this->mmap_on = 1;
+    this->mmap_curr = this->mmap_base;
+    this->mmap_len = sbuf.st_size;
+  }
+#endif
+
   if (file_plugin_get_length (this_gen) == 0) {
       _x_message(this->stream, XINE_MSG_FILE_EMPTY, this->mrl, NULL);
       close (this->fh);
       xine_log (this->stream->xine, XINE_LOG_MSG,
 		_("input_file: File empty: >%s<\n"), this->mrl);
-      return 0;
+      return -1;
   }
 
   return 1;
@@ -810,8 +948,12 @@ static xine_mrl_t **file_class_get_dir (input_class_t *this_gen,
     free(norm_files);
     
   }
-  else 
+  else {
+    free(hide_files);
+    free(dir_files);
+    free(norm_files);
     return NULL;
+  }
   
   /*
    * Inform caller about files found number.
@@ -849,6 +991,9 @@ static xine_mrl_t **file_class_get_dir (input_class_t *this_gen,
 
 static void file_class_dispose (input_class_t *this_gen) {
   file_input_class_t  *this = (file_input_class_t *) this_gen;
+  config_values_t     *config = this->xine->config;
+
+  config->unregister_callback(config, "media.files.origin_path");
 
   free (this->mrls);
   free (this);
@@ -904,8 +1049,8 @@ static void *init_plugin (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, "FILE", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "FILE", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
