@@ -50,10 +50,6 @@
 
 #define BUFSIZE 4096
 
-#if !defined(NDELAY) && defined(O_NDELAY)
-#define FNDELAY O_NDELAY
-#endif
-
 typedef struct {
 
   input_class_t     input_class;
@@ -105,7 +101,7 @@ static buf_element_t *pnm_plugin_read_block (input_plugin_t *this_gen,
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
   
-  total_bytes = pnm_plugin_read (this_gen, buf->content, todo);
+  total_bytes = pnm_plugin_read (this_gen, (char*)buf->content, todo);
 
   if (total_bytes != todo) {
     buf->free_buffer (buf);
@@ -121,7 +117,7 @@ static off_t pnm_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
   pnm_input_plugin_t *this = (pnm_input_plugin_t *) this_gen;
 
   xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG,
-           "input_pnm: seek %" PRIiMAX " bytes, origin %d\n", (intmax_t)offset,
+           "input_pnm: seek %" PRIdMAX " bytes, origin %d\n", (intmax_t)offset,
            origin);
 
   /* only realtive forward-seeking is implemented */
@@ -299,9 +295,9 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 15, "pnm", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 17, "pnm", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
