@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_stdin_fifo.c,v 1.60.2.2 2005/04/20 17:21:08 mroi Exp $
+ * $Id: input_stdin_fifo.c,v 1.67 2006/07/10 22:08:16 dgp85 Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,8 +27,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <fcntl.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <sys/stat.h>
 #include <errno.h>
 
@@ -44,6 +44,11 @@
 #include "net_buf_ctrl.h"
 
 #define BUFSIZE                 1024
+#if defined(WIN32) || defined(__CYGWIN__)
+#  define FILE_FLAGS (O_RDONLY | O_BINARY)
+#else
+#  define FILE_FLAGS O_RDONLY
+#endif
 
 typedef struct {
   input_plugin_t   input_plugin;
@@ -121,7 +126,7 @@ static buf_element_t *stdin_plugin_read_block (input_plugin_t *this_gen, fifo_bu
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
 
-  total_bytes = stdin_plugin_read (this_gen, buf->content, todo);
+  total_bytes = stdin_plugin_read (this_gen, (char*)buf->content, todo);
 
   if (total_bytes != todo) {
     buf->free_buffer (buf);
@@ -241,7 +246,7 @@ static int stdin_plugin_open (input_plugin_t *this_gen ) {
     char *filename;
 
     filename = (char *) &this->mrl[5];
-    this->fh = open (filename, O_RDONLY);
+    this->fh = open (filename, FILE_FLAGS);
 
     lprintf("filename '%s'\n", filename);
 
@@ -250,6 +255,9 @@ static int stdin_plugin_open (input_plugin_t *this_gen ) {
       return 0;
     }
   }
+#ifdef WIN32
+  else setmode(this->fh, FILE_FLAGS);
+#endif
 
 
   /*
@@ -375,8 +383,8 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_INPUT, 16, "stdin", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 17, "stdin", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
