@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_vcd.c,v 1.63 2003/01/07 22:51:35 f1rmb Exp $
+ * $Id: input_vcd.c,v 1.85 2006/07/10 22:08:16 dgp85 Exp $
  *
  */
 
@@ -25,6 +25,7 @@
 #include "config.h"
 #endif
 
+#include <netinet/in.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
@@ -50,6 +51,7 @@
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "input_plugin.h"
+#include "media_helper.h"
 
 #if defined(__sun)
 #define	CDROM	       "/vol/dev/aliases/cdrom0"
@@ -77,9 +79,9 @@ typedef struct {
 
 typedef struct {
 
-  input_class_t     input_class;
+  input_class_t          input_class;
 
-  xine_t           *xine;
+  xine_t                *xine;
 
   /* FIXME: add thread/mutex stuff to protect against multiple instantiation */
 
@@ -96,7 +98,7 @@ typedef struct {
 #elif defined (__FreeBSD__)
   struct ioc_toc_header  tochdr;
   struct cd_toc_entry    *tocent;
-  off_t                  cur_sector;
+  off_t                  cur_sec;
 #endif
 
   int                    total_tracks;
@@ -118,7 +120,7 @@ typedef struct {
 
   int                    cur_track;
 
-#if defined (__linux__) || defined(__sun)
+#if defined (__linux__) || defined(__sun) || defined(__FreeBSD__)
   uint8_t                cur_min, cur_sec, cur_frame;
 #endif
 
@@ -147,7 +149,8 @@ static int input_vcd_read_toc (vcd_input_class_t *this, int fd) {
 
   /* read TOC header */
   if ( ioctl(fd, CDROMREADTOCHDR, &this->tochdr) == -1 ) {
-    printf ("input_vcd : error in ioctl CDROMREADTOCHDR\n");
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, 
+	     "input_vcd : error in ioctl CDROMREADTOCHDR\n");
     return -1;
   }
 
@@ -156,7 +159,8 @@ static int input_vcd_read_toc (vcd_input_class_t *this, int fd) {
     this->tocent[i-1].cdte_track = i;
     this->tocent[i-1].cdte_format = CDROM_MSF;
     if ( ioctl(fd, CDROMREADTOCENTRY, &this->tocent[i-1]) == -1 ) {
-      printf ("input_vcd: error in ioctl CDROMREADTOCENTRY for track %d\n", i);
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, 
+	       "input_vcd: error in ioctl CDROMREADTOCENTRY for track %d\n", i);
       return -1;
     }
   }
@@ -167,7 +171,8 @@ static int input_vcd_read_toc (vcd_input_class_t *this, int fd) {
 
   if (ioctl(fd, CDROMREADTOCENTRY, 
 	    &this->tocent[this->tochdr.cdth_trk1]) == -1 ) {
-    printf ("input_vcd: error in ioctl CDROMREADTOCENTRY for lead-out\n");
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, 
+	     "input_vcd: error in ioctl CDROMREADTOCENTRY for lead-out\n");
     return -1;
   }
 
@@ -183,7 +188,8 @@ static int input_vcd_read_toc (vcd_input_class_t *this, int fd) {
 
   /* read TOC header */
   if ( ioctl(fd, CDIOREADTOCHEADER, &this->tochdr) == -1 ) {
-    printf ("input_vcd : error in ioctl CDROMREADTOCHDR\n");
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, 
+	     "input_vcd : error in ioctl CDROMREADTOCHDR\n");
     return -1;
   }
 
@@ -198,7 +204,8 @@ static int input_vcd_read_toc (vcd_input_class_t *this, int fd) {
   te.data = this->tocent;
   
   if ( ioctl(fd, CDIOREADTOCENTRYS, &te) == -1 ){
-    printf ("input_vcd: error in ioctl CDROMREADTOCENTRY\n");
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, 
+	     "input_vcd: error in ioctl CDROMREADTOCENTRY\n");
     return -1;
   }
 
@@ -245,8 +252,8 @@ static int sun_vcd_read(vcd_input_plugin_t *this, long lba, cdsector_t *data)
       cdxa.cdxa_format = CDROM_XA_SECTOR_DATA;
   
       if(ioctl(this->fd,CDROMCDXA,&cdxa)==-1) {
-	  perror("CDROMCDXA");
-	  return -1;
+	xprintf(this->cls->xine, XINE_VERBOSITY_DEBUG, "CDROMCDXA: %s\n", strerror(errno));
+	return -1;
       }
     }
 #else
@@ -257,8 +264,8 @@ static int sun_vcd_read(vcd_input_plugin_t *this, long lba, cdsector_t *data)
       cdread.cdread_buflen = 2336;
 
       if(ioctl(this->fd,CDROMREADMODE2,&cdread)==-1) {
-	  perror("CDROMREADMODE2");
-	  return -1;
+	xprintf(this->cls->xine, XINE_VERBOSITY_DEBUG, "CDROMREADMODE2: %s\n", strerror(errno));
+	return -1;
       }
     }
 #endif
@@ -313,11 +320,12 @@ static int sun_vcd_read(vcd_input_plugin_t *this, long lba, cdsector_t *data)
       sc.uscsi_flags = USCSI_ISOLATE | USCSI_READ;
       sc.uscsi_timeout = 20;
       if (ioctl(this->fd, USCSICMD, &sc)) {
-	perror("USCSICMD: READ CD");
+	xprintf(this->cls->xine, XINE_VERBOSITY_DEBUG, "USCSICMD: READ CD: %s\n", strerror(errno));
 	return -1;
       }
       if (sc.uscsi_status) {
-	printf ("scsi command failed with status %d\n", sc.uscsi_status);
+	xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, 
+		 "scsi command failed with status %d\n", sc.uscsi_status);
 	return -1;
       }
     }
@@ -364,7 +372,8 @@ static off_t vcd_plugin_read (input_plugin_t *this_gen,
     memcpy (&data, &msf, sizeof (msf));
 
     if (ioctl (this->fd, CDROMREADRAW, &data) == -1) {
-      printf ("input_vcd: error in CDROMREADRAW\n");
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, 
+	       "input_vcd: error in CDROMREADRAW\n");
       return 0;
     }
 
@@ -398,15 +407,15 @@ static off_t vcd_plugin_read (input_plugin_t *this_gen,
     return 0;
 
   do {
-    if (lseek (this->fd, this->cur_sector * bsize, SEEK_SET) == -1) {
-      printf ("input_vcd: seek error %d\n", errno);
+    if (lseek (this->fd, this->cur_sec * bsize, SEEK_SET) == -1) {
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, "input_vcd: seek error %d\n", errno);
       return 0;
     }
     if (read (this->fd, &data, bsize) == -1) {
-      printf ("input_vcd: read error %d\n", errno);
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, "input_vcd: read error %d\n", errno);
       return 0;
     }
-    this->cur_sector++;
+    this->cur_sec++;
   } while ((data.subheader[2]&~0x01)==0x60);
   memcpy (buf, data.data, VCDSECTORSIZE);
   return VCDSECTORSIZE;
@@ -441,7 +450,7 @@ static off_t vcd_plugin_read (input_plugin_t *this_gen,
     lba = (this->cur_min * 60 + this->cur_sec) * 75L + this->cur_frame;
 
     if (sun_vcd_read(this, lba, &data) < 0) {
-      printf ("input_vcd: read data failed\n");
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, "input_vcd: read data failed\n");
       return 0;
     }
 
@@ -499,7 +508,7 @@ static buf_element_t *vcd_plugin_read_block (input_plugin_t *this_gen,
     memcpy (&data, &msf, sizeof (msf));
 
     if (ioctl (this->fd, CDROMREADRAW, &data) == -1) {
-      printf ("input_vcd: error in CDROMREADRAW\n");
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, "input_vcd: error in CDROMREADRAW\n");
       return NULL;
     }
 
@@ -538,15 +547,15 @@ static buf_element_t *vcd_plugin_read_block (input_plugin_t *this_gen,
     return NULL;
 
   do {
-    if (lseek (this->fd, this->cur_sector * bsize, SEEK_SET) == -1) {
-      printf ("input_vcd: seek error %d\n", errno);
+    if (lseek (this->fd, this->cur_sec * bsize, SEEK_SET) == -1) {
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, "input_vcd: seek error %d\n", errno);
       return NULL;
     }
     if (read (this->fd, &data, bsize) == -1) {
-      printf ("input_vcd: read error %d\n", errno);
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, "input_vcd: read error %d\n", errno);
       return NULL;
     }
-    this->cur_sector++;
+    this->cur_sec++;
   } while ((data.subheader[2]&~0x01)==0x60);
 
   buf = fifo->buffer_pool_alloc (fifo);
@@ -586,7 +595,7 @@ static buf_element_t *vcd_plugin_read_block (input_plugin_t *this_gen,
     lba = (this->cur_min * 60 + this->cur_sec) * 75L + this->cur_frame;
 
     if (sun_vcd_read (this, lba, &data) < 0) {
-      printf ("input_vcd: read data failed\n");
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, "input_vcd: read data failed\n");
       return NULL;
     }
 
@@ -614,6 +623,7 @@ static buf_element_t *vcd_plugin_read_block (input_plugin_t *this_gen,
 }
 #endif
 
+#if defined (__linux__) || defined(__sun) || defined(HOST_OS_DARWIN)
 static off_t vcd_time_to_offset (int min, int sec, int frame) {
   return min * 60 * 75 + sec * 75 + frame;
 }
@@ -628,7 +638,6 @@ static void vcd_offset_to_time (off_t offset, uint8_t *min, uint8_t *sec,
 
 }
 
-#if defined (__linux__) || defined(__sun)
 static off_t vcd_plugin_seek (input_plugin_t *this_gen, 
 			      off_t offset, int origin) {
 
@@ -658,7 +667,8 @@ static off_t vcd_plugin_seek (input_plugin_t *this_gen,
     break;
   case SEEK_CUR:
     if (offset) 
-      printf ("input_vcd: SEEK_CUR not implemented for offset != 0\n");
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, 
+	       "input_vcd: SEEK_CUR not implemented for offset != 0\n");
 
     /*
     printf ("input_vcd: current pos: %02d:%02d:%02d\n",
@@ -676,8 +686,8 @@ static off_t vcd_plugin_seek (input_plugin_t *this_gen,
 
     break;
   default:
-    printf ("input_vcd: error seek to origin %d not implemented!\n",
-	    origin);
+    xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, 
+	     "input_vcd: error seek to origin %d not implemented!\n", origin);
     return 0;
   }
 
@@ -695,7 +705,7 @@ static off_t vcd_plugin_seek (input_plugin_t *this_gen,
 
   start = 
     ntohl(this->cls->tocent
-	  [this->cur_track+1 - this->tochdr.starting_track].addr.lba);
+	  [this->cur_track+1 - this->cls->tochdr.starting_track].addr.lba);
 
   /*  printf("seek: start sector:%lu, origin: %d, offset:%qu\n", 
 	 start, origin, offset);
@@ -704,21 +714,21 @@ static off_t vcd_plugin_seek (input_plugin_t *this_gen,
   switch (origin) {
   case SEEK_SET:
     dist = offset / VCDSECTORSIZE;
-    this->cur_sector = start + dist;
+    this->cur_sec = start + dist;
     break;
   case SEEK_CUR:
 
     if (offset) 
-      printf ("input_vcd: SEEK_CUR not implemented for offset != 0\n");
+      xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, "input_vcd: SEEK_CUR not implemented for offset != 0\n");
 
-    sector_pos = this->cur_sector;
+    sector_pos = this->cur_sec;
 
     return sector_pos * VCDSECTORSIZE;
 
     break;
   default:
-    printf ("input_vcd: error seek to origin %d not implemented!\n",
-	    origin);
+    xprintf (this->cls->xine, XINE_VERBOSITY_DEBUG, 
+	     "input_vcd: error seek to origin %d not implemented!\n", origin);
     return 0;
   }
 
@@ -766,10 +776,10 @@ static off_t vcd_plugin_get_length (input_plugin_t *this_gen) {
   len = 
     ntohl(this->cls->tocent
 	  [this->cur_track+2 
-	  - this->tochdr.starting_track].addr.lba)
+	  - this->cls->tochdr.starting_track].addr.lba)
     - ntohl(this->cls->tocent
 	    [this->cur_track+1 
-	    - this->tochdr.starting_track].addr.lba);
+	    - this->cls->tochdr.starting_track].addr.lba);
   
   return len * 2352; /*VCDSECTORSIZE;*/
 
@@ -796,7 +806,8 @@ static void vcd_plugin_dispose (input_plugin_t *this_gen ) {
 
   vcd_input_plugin_t *this = (vcd_input_plugin_t *) this_gen;
 
-  close(this->fd);
+  if (this->fd != -1)
+    close(this->fd);
 
   free (this->mrl);
   free (this);
@@ -814,33 +825,80 @@ static int vcd_plugin_get_optional_data (input_plugin_t *this_gen,
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
-static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *stream, 
+static int vcd_plugin_open (input_plugin_t *this_gen) {
+  vcd_input_plugin_t *this = (vcd_input_plugin_t *) this_gen;
+  vcd_input_class_t  *cls = this->cls;
+  char               *filename;
+  int                 fd;
+
+  fd = open (cls->device, O_RDONLY|O_EXCL);
+  if (fd == -1) {
+    return 0;
+  }
+
+  this->fd     = fd;
+
+  if (input_vcd_read_toc (this->cls, this->fd)) {
+    return 0;
+  }
+
+  filename = (char *) &this->mrl[5];
+  while (*filename == '/') filename++;
+
+  if (sscanf (filename, "%d", &this->cur_track) != 1) {
+    xprintf (cls->xine, XINE_VERBOSITY_LOG, 
+	     _("input_vcd: malformed MRL. Use vcdo:/<track #>\n"));
+    return 0;
+  }
+
+  if (this->cur_track>=this->cls->total_tracks) {
+    xprintf (cls->xine, XINE_VERBOSITY_LOG, 
+	     _("input_vcd: invalid track %d (valid range: 0 .. %d)\n"),
+	     this->cur_track, this->cls->total_tracks-1);
+    return 0;
+  }
+
+#if defined (__linux__) || defined(__sun)
+  this->cur_min   = this->cls->tocent[this->cur_track].cdte_addr.msf.minute;
+  this->cur_sec   = this->cls->tocent[this->cur_track].cdte_addr.msf.second;
+  this->cur_frame = this->cls->tocent[this->cur_track].cdte_addr.msf.frame;
+#elif defined (__FreeBSD__)
+  {
+    int bsize = 2352;
+    if (ioctl (this->fd, CDRIOCSETBLOCKSIZE, &bsize) == -1) {
+      xprintf (cls->xine, XINE_VERBOSITY_DEBUG, "input_vcd: error in CDRIOCSETBLOCKSIZE %d\n", errno);
+      return 0;
+    }
+  
+    this->cur_sec = 
+      ntohl(this->cls->tocent
+	    [this->cur_track+1 - this->cls->tochdr.starting_track].addr.lba);
+    
+  }
+#endif
+  
+  return 1;
+}
+
+static input_plugin_t *vcd_class_get_instance (input_class_t *cls_gen, xine_stream_t *stream, 
 				    const char *data) {
 
   vcd_input_class_t  *cls = (vcd_input_class_t *) cls_gen; 
   vcd_input_plugin_t *this;
   char               *mrl = strdup(data);
-  char               *filename;
-  int                 fd;
 
-  if (strncasecmp (mrl, "vcd:/",5)) {
+  if (strncasecmp (mrl, "vcdo:/",6)) {
     free (mrl);
     return 0;
   }
     
-  fd = open (cls->device, O_RDONLY);
-
-  if (fd == -1) {
-    free (mrl);
-    return 0;
-  }
-
   this = (vcd_input_plugin_t *) xine_xmalloc(sizeof(vcd_input_plugin_t));
 
   this->stream = stream;
   this->mrl    = mrl;
-  this->fd     = fd;
+  this->fd     = -1;
 
+  this->input_plugin.open              = vcd_plugin_open;
   this->input_plugin.get_capabilities  = vcd_plugin_get_capabilities;
   this->input_plugin.read              = vcd_plugin_read;
   this->input_plugin.read_block        = vcd_plugin_read_block;
@@ -854,53 +912,6 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
   this->input_plugin.input_class       = cls_gen;
   this->cls                            = cls;
 
-  if (input_vcd_read_toc (this->cls, this->fd)) {
-    close (this->fd);
-    free (this);
-    free (mrl);
-    return NULL;
-  }
-
-  filename = (char *) &this->mrl[5];
-  while (*filename == '/') filename++;
-
-  if (sscanf (filename, "%d", &this->cur_track) != 1) {
-    printf ("input_vcd: malformed MRL. Use vcd:/<track #>\n");
-    close (this->fd);
-    free (this);
-    free (mrl);
-    return NULL;
-  }
-
-  if (this->cur_track>=this->cls->total_tracks) {
-    printf ("input_vcd: invalid track %d (valid range: 0 .. %d)\n",
-	    this->cur_track, this->cls->total_tracks-1);
-    close (this->fd);
-    free (this);
-    free (mrl);
-    return NULL;
-  }
-
-#if defined (__linux__) || defined(__sun)
-  this->cur_min   = this->cls->tocent[this->cur_track].cdte_addr.msf.minute;
-  this->cur_sec   = this->cls->tocent[this->cur_track].cdte_addr.msf.second;
-  this->cur_frame = this->cls->tocent[this->cur_track].cdte_addr.msf.frame;
-#elif defined (__FreeBSD__)
-  {
-    int bsize = 2352;
-    if (ioctl (this->fd, CDRIOCSETBLOCKSIZE, &bsize) == -1) {
-      printf ("input_vcd: error in CDRIOCSETBLOCKSIZE %d\n", errno);
-      return 0;
-    }
-  
-    this->cur_sector = 
-      ntohl(this->cls->tocent
-	    [this->cur_track+1 - this->tochdr.starting_track].addr.lba);
-    
-  }
-#endif
-  
-
   return &this->input_plugin;
 }
 
@@ -912,14 +923,17 @@ static char *vcd_class_get_description (input_class_t *this_gen) {
   return _("Video CD input plugin");
 }
 
-static char *vcd_class_get_identifier (input_class_t *this_gen) {
-  return "vcd";
+static const char *vcd_class_get_identifier (input_class_t *this_gen) {
+  return "vcdo";
 }
 
 static void vcd_class_dispose (input_class_t *this_gen) {
 
   vcd_input_class_t  *this = (vcd_input_class_t *) this_gen;
   int                 i;
+  config_values_t *config = this->xine->config;
+
+  config->unregister_callback(config, "media.vcd.device");
 
   for (i = 0; i < 100; i++)
     free (this->filelist[i]);
@@ -928,71 +942,12 @@ static void vcd_class_dispose (input_class_t *this_gen) {
   free (this);
 }
 
-#if defined (__linux__)
 static int vcd_class_eject_media (input_class_t *this_gen) {
-  vcd_input_class_t *this = (vcd_input_class_t *) this_gen;
-  int ret, status, fd;
 
-  if ((fd = open (this->device, O_RDONLY|O_NONBLOCK)) > -1) {
-    if((status = ioctl(fd, CDROM_DRIVE_STATUS, CDSL_CURRENT)) > 0) {
-      switch(status) {
-      case CDS_TRAY_OPEN:
-	if((ret = ioctl(fd, CDROMCLOSETRAY)) != 0) {
-	  printf ("input_vcd: CDROMCLOSETRAY failed: %s\n", strerror(errno));  
-	}
-	break;
-      case CDS_DISC_OK:
-	if((ret = ioctl(fd, CDROMEJECT)) != 0) {
-	  printf ("input_vcd: CDROMEJECT failed: %s\n", strerror(errno));  
-	}
-	break;
-      }
-    }
-    else {
-      printf ("input_vcd: CDROM_DRIVE_STATUS failed: %s\n", 
-	      strerror(errno));
-      close(fd);
-      return 0;
-    }
-  }
+  vcd_input_class_t  *this = (vcd_input_class_t *) this_gen;
 
-  close(fd);
-  
-  return 1;
+  return media_eject_media (this->xine, this->device);
 }
-#elif defined (__FreeBSD__)
-static int vcd_class_eject_media (input_class_t *this_gen) {
-  vcd_input_class_t *this = (vcd_input_class_t *) this_gen;
-  int          fd;
-
-  if ((fd = open(this->device, O_RDONLY|O_NONBLOCK)) > -1) {
-    if (ioctl(fd, CDIOCALLOW) == -1) {
-      perror("ioctl(cdromallow)");
-    } else {
-      if (ioctl(fd, CDIOCEJECT) == -1) {
-	perror("ioctl(cdromeject)");
-      }
-    }
-    close(fd);
-  }
-  
-  return 1;
-}
-#elif defined (__sun)
-static int vcd_class_eject_media (input_class_t *this_gen) {
-  vcd_input_class_t *this = (vcd_input_class_t *) this_gen;
-  int          fd, ret;
-
-  if ((fd = open(this->device, O_RDONLY|O_NONBLOCK)) > -1) {
-    if ((ret = ioctl(fd, CDROMEJECT)) != 0) {
-      printf ("input_vcd: CDROMEJECT failed: %s\n", strerror(errno));  
-    }
-    close(fd);
-  }
-
-  return 1;
-}
-#endif
 
 static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *filename, 
 				       int *num_files) {
@@ -1006,10 +961,11 @@ static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *file
     return NULL;
 
   
-  fd = open (this->device, O_RDONLY);
+  fd = open (this->device, O_RDONLY|O_EXCL);
 
   if (fd == -1) {
-    printf ("unable to open %s: %s.\n", this->device, strerror(errno));
+    xprintf (this->xine, XINE_VERBOSITY_LOG,
+	     _("unable to open %s: %s.\n"), this->device, strerror(errno));
     return NULL;
   }
 
@@ -1017,7 +973,7 @@ static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *file
     close (fd);
     fd = -1;
 
-    printf ("vcd_read_toc failed\n");
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, "vcd_read_toc failed\n");
 
     return NULL;
   }
@@ -1032,7 +988,7 @@ static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *file
     char mrl[1024];
     
     memset(&mrl, 0, sizeof (mrl));
-    sprintf(mrl, "vcd:/%d",i);
+    sprintf(mrl, "vcdo:/%d",i);
     
     if((i-1) >= this->mrls_allocated_entries) {
       ++this->mrls_allocated_entries;
@@ -1081,10 +1037,11 @@ static char ** vcd_class_get_autoplay_list (input_class_t *this_gen, int *num_fi
   int i, fd;
 
 
-  fd = open (this->device, O_RDONLY);
+  fd = open (this->device, O_RDONLY|O_EXCL);
 
   if (fd == -1) {
-    printf ("input_vcd: unable to open %s: %s.\n", this->device, strerror(errno));
+    xprintf (this->xine, XINE_VERBOSITY_LOG,
+	     _("input_vcd: unable to open %s: %s.\n"), this->device, strerror(errno));
     return NULL;
   }
 
@@ -1092,7 +1049,7 @@ static char ** vcd_class_get_autoplay_list (input_class_t *this_gen, int *num_fi
     close (fd);
     fd = -1;
 
-    printf ("input_vcd: vcd_read_toc failed\n");
+    xprintf (this->xine, XINE_VERBOSITY_DEBUG, "input_vcd: vcd_read_toc failed\n");
 
     return NULL;
   }
@@ -1109,7 +1066,7 @@ static char ** vcd_class_get_autoplay_list (input_class_t *this_gen, int *num_fi
     if(this->filelist[i - 1] == NULL)
       this->filelist[i - 1] = (char *) realloc(this->filelist[i - 1], sizeof(char *) * 256);
 
-    sprintf (this->filelist[i - 1], "vcd:/%d",i);
+    sprintf (this->filelist[i - 1], "vcdo:/%d",i);
     /* printf ("list[%d] : %d %s\n", i, this->filelist[i-1], this->filelist[i-1]);   */
   }
 
@@ -1129,7 +1086,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   this->xine   = xine;
 
-  this->input_class.open_plugin        = open_plugin;
+  this->input_class.get_instance       = vcd_class_get_instance;
   this->input_class.get_identifier     = vcd_class_get_identifier;
   this->input_class.get_description    = vcd_class_get_description;
   this->input_class.get_dir            = vcd_class_get_dir;
@@ -1137,9 +1094,11 @@ static void *init_class (xine_t *xine, void *data) {
   this->input_class.dispose            = vcd_class_dispose;
   this->input_class.eject_media        = vcd_class_eject_media;
 
-  this->device = config->register_string (config, "input.vcd_device", CDROM,
-					  _("path to your local vcd device file"),
-					  NULL, 10, device_change_cb, (void *)this);
+  this->device = config->register_string (config, "media.vcd.device", CDROM,
+					  _("device used for VCD playback"),
+					  _("The path to the device, usually a CD or DVD drive, "
+					    "you intend to play your VideoCDs with."),
+					  10, device_change_cb, (void *)this);
 
   this->mrls = (xine_mrl_t **) xine_xmalloc(sizeof(xine_mrl_t*));
   this->mrls_allocated_entries = 0;
@@ -1155,8 +1114,8 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 11, "VCD", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, "VCDO", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
