@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 the xine project
+ * Copyright (C) 2002-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mmsh.h,v 1.1 2003/01/13 01:11:57 tmattern Exp $
+ * $Id: mmsh.h,v 1.5 2006/06/20 01:46:41 dgp85 Exp $
  *
  * libmmsh public header
  */
@@ -37,6 +37,8 @@ int      mmsh_read (mmsh_t *this, char *data, int len);
 uint32_t mmsh_get_length (mmsh_t *this);
 void     mmsh_close (mmsh_t *this);
 
-int      mmsh_peek_header (mmsh_t *this, char *data);
+size_t   mmsh_peek_header (mmsh_t *this, char *data, size_t maxsize);
+
+off_t    mmsh_get_current_pos (mmsh_t *this);
 
 #endif
