@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: pnm.c,v 1.10 2003/01/22 16:33:24 holstsn Exp $
+ * $Id: pnm.c,v 1.22 2006/06/20 01:46:41 dgp85 Exp $
  *
  * pnm protocol implementation 
  * based upon code from joschka
@@ -25,7 +25,6 @@
 
 #include <unistd.h>
 #include <stdio.h>
-#include <assert.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
@@ -37,20 +36,26 @@
 #include <sys/time.h>
 #include <sys/types.h>
 
-#include "pnm.h"
-#include "libreal/rmff.h"
-#include "bswap.h"
-#include "xineutils.h"
-
+#define LOG_MODULE "pnm"
+#define LOG_VERBOSE
 /*
 #define LOG
 */
 
+#include "pnm.h"
+#include "libreal/rmff.h"
+#include "bswap.h"
+#include "io_helper.h"
+#include "xineutils.h"
+#include "xine_internal.h"
+
 #define BUF_SIZE 4096
 #define HEADER_SIZE 4096
 
 struct pnm_s {
 
+  xine_stream_t *stream;
+
   int           s;
 
   char         *host;
@@ -121,14 +126,14 @@ const unsigned char pnm_data_header[]={
 #define PNA_CLIENT_STRING    0x63
 #define PNA_PATH_REQUEST     0x52
 
-const unsigned char pnm_challenge[] = "0990f6b4508b51e801bd6da011ad7b56";
-const unsigned char pnm_timestamp[] = "[15/06/1999:22:22:49 00:00]";
-const unsigned char pnm_guid[]      = "3eac2411-83d5-11d2-f3ea-d7c3a51aa8b0";
-const unsigned char pnm_response[]  = "97715a899cbe41cee00dd434851535bf";
-const unsigned char client_string[] = "WinNT_4.0_6.0.6.45_plus32_MP60_en-US_686l";
+static const char pnm_challenge[] = "0990f6b4508b51e801bd6da011ad7b56";
+static const char pnm_timestamp[] = "[15/06/1999:22:22:49 00:00]";
+static const char pnm_guid[]      = "3eac2411-83d5-11d2-f3ea-d7c3a51aa8b0";
+static const char pnm_response[]  = "97715a899cbe41cee00dd434851535bf";
+static const char client_string[] = "WinNT_9.0_6.0.6.45_plus32_MP60_en-US_686l";
 
 #define PNM_HEADER_SIZE 11
-const unsigned char pnm_header[] = {
+static const unsigned char pnm_header[] = {
         'P','N','A',
         0x00, 0x0a,
         0x00, 0x14,
@@ -136,7 +141,7 @@ const unsigned char pnm_header[] = {
         0x00, 0x01 };
 
 #define PNM_CLIENT_CAPS_SIZE 126
-const unsigned char pnm_client_caps[] = {
+static const unsigned char pnm_client_caps[] = {
     0x07, 0x8a, 'p','n','r','v', 
        0, 0x90, 'p','n','r','v', 
        0, 0x64, 'd','n','e','t', 
@@ -159,12 +164,12 @@ const unsigned char pnm_client_caps[] = {
        0, 0x12, 'l','p','c','J', 
        0, 0x07, '0','5','_','6' };
 
-const uint32_t pnm_default_bandwidth=10485800;
-const uint32_t pnm_available_bandwidths[]={14400,19200,28800,33600,34430,57600,
+static const uint32_t pnm_default_bandwidth=10485800;
+static const uint32_t pnm_available_bandwidths[]={14400,19200,28800,33600,34430,57600,
                                   115200,262200,393216,524300,1544000,10485800};
 
 #define PNM_TWENTYFOUR_SIZE 16
-unsigned char pnm_twentyfour[]={
+static unsigned char pnm_twentyfour[]={
     0xd5, 0x42, 0xa3, 0x1b, 0xef, 0x1f, 0x70, 0x24,
     0x85, 0x29, 0xb3, 0x8d, 0xba, 0x11, 0xf3, 0xd6 };
 
@@ -179,152 +184,6 @@ unsigned char after_chunks[]={
 
 
 
-static void hexdump (char *buf, int length);
-
-/*
- * network utilities
- * nothing really sophisticated here, only tools
- * to connect to a host, send and receive data over this connection
- */
- 
-static int host_connect_attempt(struct in_addr ia, int port) {
-
-  int                s;
-  struct sockaddr_in sin;
-
-  s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);  
-  if (s == -1) {
-    printf ("input_pnm: socket(): %s\n", strerror(errno));
-    return -1;
-  }
-
-  sin.sin_family = AF_INET;
-  sin.sin_addr   = ia;
-  sin.sin_port   = htons(port);
-  
-  if (connect(s, (struct sockaddr *)&sin, sizeof(sin))==-1 
-      && errno != EINPROGRESS) {
-    printf ("input_pnm: connect(): %s\n", strerror(errno));
-    close(s);
-    return -1;
-  }
-
-  return s;
-}
-
-static int host_connect(const char *host, int port) {
-
-  struct hostent *h;
-  int             i, s;
-  
-  h = gethostbyname(host);
-  if (h == NULL) {
-    printf ("input_pnm: unable to resolve '%s'.\n", host);
-    return -1;
-  }
-
-  for (i = 0; h->h_addr_list[i]; i++) {
-    struct in_addr ia;
-
-    memcpy (&ia, h->h_addr_list[i], 4);
-    s = host_connect_attempt(ia, port);
-    if(s != -1)
-      return s;
-  }
-  printf ("input_pnm: unable to connect to '%s'.\n", host);
-  return -1;
-}
-
-static int rm_write(int s, const char *buf, int len) {
-  int total, timeout;
-
-  total = 0; timeout = 30;
-  while (total < len){ 
-    int n;
-
-    n = write (s, &buf[total], len - total);
-
-    if (n > 0)
-      total += n;
-    else if (n < 0) {
-      if ((timeout>0) && ((errno == EAGAIN) || (errno == EINPROGRESS))) {
-        sleep (1); timeout--;
-      } else
-        return -1;
-    }
-  }
-
-  return total;
-}
-
-static ssize_t rm_read(int fd, void *buf, size_t count) {
-  
-  ssize_t ret, total;
-
-  total = 0;
-
-  while (total < count) {
-  
-    fd_set rset;
-    struct timeval timeout;
-
-    FD_ZERO (&rset);
-    FD_SET  (fd, &rset);
-    
-    timeout.tv_sec  = 3;
-    timeout.tv_usec = 0;
-    
-    if (select (fd+1, &rset, NULL, NULL, &timeout) <= 0) {
-      return -1;
-    }
-    
-    ret=read (fd, ((uint8_t*)buf)+total, count-total);
-
-    if (ret<=0) {
-      printf ("input_pnm: read error.\n");
-      return ret;
-    } else
-      total += ret;
-  }
-
-  return total;
-}
-
-/*
- * a simple hexdump tool for debugging purposes
- */
- 
-static void hexdump (char *buf, int length) {
-
-  int i;
-
-  printf ("input_pnm: ascii>");
-  for (i = 0; i < length; i++) {
-    unsigned char c = buf[i];
-
-    if ((c >= 32) && (c <= 128))
-      printf ("%c", c);
-    else
-      printf (".");
-  }
-  printf ("\n");
-
-  printf ("input_pnm: hexdump> ");
-  for (i = 0; i < length; i++) {
-    unsigned char c = buf[i];
-
-    printf ("%02x", c);
-
-    if ((i % 16) == 15)
-      printf ("\npnm:         ");
-
-    if ((i % 2) == 1)
-      printf (" ");
-
-  }
-  printf ("\n");
-}
-
 /*
  * pnm_get_chunk gets a chunk from stream
  * and returns number of bytes read, chunk_type, data and
@@ -346,16 +205,21 @@ static unsigned int pnm_get_chunk(pnm_t *p,
                          char *data, int *need_response) {
 
   unsigned int chunk_size;
-  int n;
+  unsigned int n;
   char *ptr;
- 
+
+  if( max < PREAMBLE_SIZE )
+    return -1;
+    
   /* get first PREAMBLE_SIZE bytes and ignore checksum */
-  rm_read (p->s, data, CHECKSUM_SIZE);
+  _x_io_tcp_read (p->stream, p->s, data, CHECKSUM_SIZE);
   if (data[0] == 0x72)
-    rm_read (p->s, data, PREAMBLE_SIZE);
+    _x_io_tcp_read (p->stream, p->s, data, PREAMBLE_SIZE);
   else
-    rm_read (p->s, data+CHECKSUM_SIZE, PREAMBLE_SIZE-CHECKSUM_SIZE);
-  
+    _x_io_tcp_read (p->stream, p->s, data+CHECKSUM_SIZE, PREAMBLE_SIZE-CHECKSUM_SIZE);
+
+  max -= PREAMBLE_SIZE;
+    
   *chunk_type = be2me_32(*((uint32_t *)data));
   chunk_size = be2me_32(*((uint32_t *)(data+4)));
 
@@ -363,7 +227,11 @@ static unsigned int pnm_get_chunk(pnm_t *p,
     case PNA_TAG:
       *need_response=0;
       ptr=data+PREAMBLE_SIZE;
-      rm_read (p->s, ptr++, 1);
+
+      if( max < 1 )
+        return -1;
+      _x_io_tcp_read (p->stream, p->s, ptr++, 1);
+      max -= 1;
 
       while(1) {
 	/* The pna chunk is devided into subchunks.
@@ -376,26 +244,38 @@ static unsigned int pnm_get_chunk(pnm_t *p,
 	 * if first byte is 'F', we got an error
 	 */
 
-        rm_read (p->s, ptr, 2);
+        if( max < 2 )
+          return -1;
+        _x_io_tcp_read (p->stream, p->s, ptr, 2);
+        max -= 2;
+        
 	if (*ptr == 'X') /* checking for server message */
 	{
-	  printf("input_pnm: got a message from server:\n");
-	  rm_read (p->s, ptr+2, 1);
+	  xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, "input_pnm: got a message from server:\n");
+          if( max < 1 )
+            return -1;
+	  _x_io_tcp_read (p->stream, p->s, ptr+2, 1);
+          max -= 1;
 
 	  /* two bytes of message length*/
 	  n=be2me_16(*(uint16_t*)(ptr+1));
 
 	  /* message itself */
-	  rm_read (p->s, ptr+3, n);
+          if( max < n )
+            return -1;
+	  _x_io_tcp_read (p->stream, p->s, ptr+3, n);
+          max -= n;
+          if( max < 1 )
+            return -1;
 	  ptr[3+n]=0;
-	  printf("%s\n",ptr+3);
+	  xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, "%s\n", ptr+3);
 	  return -1;
 	}
 	
 	if (*ptr == 'F') /* checking for server error */
 	{
 	  /* some error codes after 'F' were ignored */
-	  printf("input_pnm: server error.\n");
+	  xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, "input_pnm: server error.\n");
 	  return -1;
 	}
 	if (*ptr == 'i') /* the server want a response from us. it will be sent after these headers */
@@ -406,11 +286,16 @@ static unsigned int pnm_get_chunk(pnm_t *p,
 	}
 	if (*ptr != 0x4f) break;
 	n=ptr[1];
-	rm_read (p->s, ptr+2, n);
+        if( max < n )
+          return -1;
+        _x_io_tcp_read (p->stream, p->s, ptr+2, n);
 	ptr+=(n+2);
+        max-=n;
       }
       /* the checksum of the next chunk is ignored here */
-      rm_read (p->s, ptr+2, 1);
+      if( max < 1 )
+        return -1;
+      _x_io_tcp_read (p->stream, p->s, ptr+2, 1);
       ptr+=3;
       chunk_size=ptr-data;
       break;
@@ -419,14 +304,16 @@ static unsigned int pnm_get_chunk(pnm_t *p,
     case PROP_TAG:
     case MDPR_TAG:
     case CONT_TAG:
-      if (chunk_size > max) {
-        printf("error: max chunk size exeeded (max was 0x%04x)\n", max);
+      if (chunk_size > max || chunk_size < PREAMBLE_SIZE) {
+        xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, "error: max chunk size exeeded (max was 0x%04x)\n", max);
+#ifdef LOG
 	/* reading some bytes for debugging */
-        n=rm_read (p->s, &data[PREAMBLE_SIZE], 0x100 - PREAMBLE_SIZE);
-        hexdump(data,n+PREAMBLE_SIZE);
+        n=_x_io_tcp_read (p->stream, p->s, &data[PREAMBLE_SIZE], 0x100 - PREAMBLE_SIZE);
+        xine_hexdump(data,n+PREAMBLE_SIZE);
+#endif
         return -1;
       }
-      rm_read (p->s, &data[PREAMBLE_SIZE], chunk_size-PREAMBLE_SIZE);
+      _x_io_tcp_read (p->stream, p->s, &data[PREAMBLE_SIZE], chunk_size-PREAMBLE_SIZE);
       break;
     default:
       *chunk_type = 0;
@@ -474,7 +361,7 @@ static void pnm_send_request(pnm_t *p, uint32_t bandwidth) {
   c+=pnm_write_chunk(PNA_CLIENT_CHALLANGE,strlen(pnm_challenge),
           pnm_challenge,&p->buffer[c]);
   c+=pnm_write_chunk(PNA_CLIENT_CAPS,PNM_CLIENT_CAPS_SIZE,
-          pnm_client_caps,&p->buffer[c]);
+          (char*)pnm_client_caps,&p->buffer[c]);
   c+=pnm_write_chunk(0x0a,0,NULL,&p->buffer[c]);
   c+=pnm_write_chunk(0x0c,0,NULL,&p->buffer[c]);
   c+=pnm_write_chunk(0x0d,0,NULL,&p->buffer[c]);
@@ -493,7 +380,7 @@ static void pnm_send_request(pnm_t *p, uint32_t bandwidth) {
   c+=pnm_write_chunk(PNA_GUID,strlen(pnm_guid),
           pnm_guid,&p->buffer[c]);
   c+=pnm_write_chunk(PNA_TWENTYFOUR,PNM_TWENTYFOUR_SIZE,
-          pnm_twentyfour,&p->buffer[c]);
+          (char*)pnm_twentyfour,&p->buffer[c]);
   
   /* data after chunks */
   memcpy(&p->buffer[c],after_chunks,after_chunks_length);
@@ -518,7 +405,7 @@ static void pnm_send_request(pnm_t *p, uint32_t bandwidth) {
   p->buffer[c]='y';
   p->buffer[c+1]='B';
   
-  rm_write(p->s,p->buffer,c+2);
+  _x_io_tcp_write(p->stream,p->s,p->buffer,c+2);
 }
 
 /*
@@ -535,7 +422,7 @@ static void pnm_send_response(pnm_t *p, const char *response) {
 
   memcpy(&p->buffer[3], response, size);
 
-  rm_write (p->s, p->buffer, size+3);
+  _x_io_tcp_write(p->stream, p->s, p->buffer, size+3);
 
 }
 
@@ -550,10 +437,10 @@ static void pnm_send_response(pnm_t *p, const char *response) {
 static int pnm_get_headers(pnm_t *p, int *need_response) {
 
   uint32_t chunk_type;
-  uint8_t  *ptr=p->header;
-  uint8_t  *prop_hdr=NULL;
+  char     *ptr=(char*)p->header;
+  char     *prop_hdr=NULL;
   int      chunk_size,size=0;
-  int      nr;
+  int      nr =0;
 /*  rmff_header_t *h; */
 
   *need_response=0;
@@ -561,7 +448,7 @@ static int pnm_get_headers(pnm_t *p, int *need_response) {
   while(1) {
     if (HEADER_SIZE-size<=0)
     {
-      printf("input_pnm: header buffer overflow. exiting\n");
+      xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, "input_pnm: header buffer overflow. exiting\n");
       return 0;
     }
     chunk_size=pnm_get_chunk(p,HEADER_SIZE-size,&chunk_type,ptr,&nr);
@@ -584,7 +471,7 @@ static int pnm_get_headers(pnm_t *p, int *need_response) {
   }
 
   if (!prop_hdr) {
-    printf("input_pnm: error while parsing headers.\n");
+    xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, "input_pnm: error while parsing headers.\n");
     return 0;
   }
   
@@ -598,7 +485,7 @@ static int pnm_get_headers(pnm_t *p, int *need_response) {
   
   /* read challenge */
   memcpy (p->buffer, ptr, PREAMBLE_SIZE);
-  rm_read (p->s, &p->buffer[PREAMBLE_SIZE], 64);
+  _x_io_tcp_read (p->stream, p->s, &p->buffer[PREAMBLE_SIZE], 64);
 
   /* now write a data header */
   memcpy(ptr, pnm_data_header, PNM_DATA_HEADER_SIZE);
@@ -673,15 +560,15 @@ static int pnm_calc_stream(pnm_t *p) {
       if (p->ts_current < p->ts_last[0])
         return 1;
       /* does not help, we guess type 0     */
-#ifdef LOG
-      printf("guessing stream# 0\n");
-#endif
+      lprintf("guessing stream# 0\n");
+
       p->seq_num[0]=p->seq_current[0]+1;
       p->seq_num[1]=p->seq_current[1]+1;
       return 0;
       break;
   }
-  printf("input_pnm: wow, something very nasty happened in pnm_calc_stream\n");
+  xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, 
+	  "input_pnm: wow, something very nasty happened in pnm_calc_stream\n");
   return 2;
 }
 
@@ -691,7 +578,7 @@ static int pnm_calc_stream(pnm_t *p) {
 
 static int pnm_get_stream_chunk(pnm_t *p) {
 
-  int  n;
+  unsigned int n;
   char keepalive='!';
   unsigned int fof1, fof2, stream;
 
@@ -699,7 +586,7 @@ static int pnm_get_stream_chunk(pnm_t *p) {
   /* realplayer seems to do that every 43th package */
   if ((p->packet%43) == 42)  
   {
-    rm_write(p->s,&keepalive,1);
+    _x_io_tcp_write(p->stream,p->s,&keepalive,1);
   }
 
   /* data chunks begin with: 'Z' <o> <o> <i1> 'Z' <i2>
@@ -708,17 +595,15 @@ static int pnm_get_stream_chunk(pnm_t *p) {
    * <i2> is a 8 bit index which counts from 0x10 to somewhere
    */
   
-  n = rm_read (p->s, p->buffer, 8);
+  n = _x_io_tcp_read (p->stream, p->s, p->buffer, 8);
   if (n<8) return 0;
   
   /* skip 8 bytes if 0x62 is read */
   if (p->buffer[0] == 0x62)
   {
-    n = rm_read (p->s, p->buffer, 8);
+    n = _x_io_tcp_read (p->stream, p->s, p->buffer, 8);
     if (n<8) return 0;
-#ifdef LOG
-    printf("input_pnm: had to seek 8 bytes on 0x62\n");
-#endif
+    lprintf("had to seek 8 bytes on 0x62\n");
   }
   
   /* a server message */
@@ -726,14 +611,15 @@ static int pnm_get_stream_chunk(pnm_t *p) {
   {
     int size=be2me_16(*(uint16_t*)(&p->buffer[1]));
 
-    rm_read (p->s, &p->buffer[8], size-5);
+    _x_io_tcp_read (p->stream, p->s, &p->buffer[8], size-5);
     p->buffer[size+3]=0;
-    printf("input_pnm: got message from server while reading stream:\n%s\n", &p->buffer[3]);
+    xprintf(p->stream->xine, XINE_VERBOSITY_LOG, 
+	    _("input_pnm: got message from server while reading stream:\n%s\n"), &p->buffer[3]);
     return 0;
   }
   if (p->buffer[0] == 'F')
   {
-    printf("input_pnm: server error.\n");
+    xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, "input_pnm: server error.\n");
     return 0;
   }
 
@@ -747,7 +633,7 @@ static int pnm_get_stream_chunk(pnm_t *p) {
     for (i=1; i<8; i++) {
       p->buffer[i-1]=p->buffer[i];
     }
-    rm_read (p->s, &p->buffer[7], 1);
+    _x_io_tcp_read (p->stream, p->s, &p->buffer[7], 1);
     n++;
   }
 
@@ -758,8 +644,10 @@ static int pnm_get_stream_chunk(pnm_t *p) {
   /* check for 'Z's */
   if ((p->buffer[0] != 0x5a)||(p->buffer[7] != 0x5a))
   {
-    printf("input_pnm: bad boundaries\n");
-    hexdump(p->buffer, 8);
+    xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, "input_pnm: bad boundaries\n");
+#ifdef LOG
+    xine_hexdump(p->buffer, 8);
+#endif
     return 0;
   }
 
@@ -768,7 +656,8 @@ static int pnm_get_stream_chunk(pnm_t *p) {
   fof2=be2me_16(*(uint16_t*)(&p->buffer[3]));
   if (fof1 != fof2)
   {
-    printf("input_pnm: frame offsets are different: 0x%04x 0x%04x\n",fof1,fof2);
+    xprintf(p->stream->xine, XINE_VERBOSITY_DEBUG, 
+	    "input_pnm: frame offsets are different: 0x%04x 0x%04x\n", fof1, fof2);
     return 0;
   }
 
@@ -776,7 +665,7 @@ static int pnm_get_stream_chunk(pnm_t *p) {
   p->seq_current[0]=be2me_16(*(uint16_t*)(&p->buffer[5]));
   
   /* now read the rest of stream chunk */
-  n = rm_read (p->s, &p->recv[5], fof1-5);
+  n = _x_io_tcp_read (p->stream, p->s, (char*)&p->recv[5], fof1-5);
   if (n<(fof1-5)) return 0;
 
   /* get second index */
@@ -813,12 +702,12 @@ static int pnm_get_stream_chunk(pnm_t *p) {
   return fof1;
 }
 
-pnm_t *pnm_connect(const char *mrl) {
+pnm_t *pnm_connect(xine_stream_t *stream, const char *mrl) {
   
   char *mrl_ptr=strdup(mrl);
   char *slash, *colon;
-  int pathbegin, hostend;
-  pnm_t *p=xine_xmalloc(sizeof(pnm_t));
+  size_t pathbegin, hostend;
+  pnm_t *p;
   int fd;
   int need_response=0;
   
@@ -829,6 +718,8 @@ pnm_t *pnm_connect(const char *mrl) {
   
   mrl_ptr+=6;
 
+  p = xine_xmalloc(sizeof(pnm_t));
+  p->stream = stream;
   p->port=7070;
   p->url=strdup(mrl);
   p->packet=0;
@@ -843,7 +734,7 @@ pnm_t *pnm_connect(const char *mrl) {
   pathbegin=slash-mrl_ptr;
   hostend=colon-mrl_ptr;
 
-  p->host=malloc(sizeof(char)*hostend+1);
+  p->host = malloc(sizeof(char)*hostend+1);
   strncpy(p->host, mrl_ptr, hostend);
   p->host[hostend]=0;
 
@@ -856,14 +747,12 @@ pnm_t *pnm_connect(const char *mrl) {
 
   free(mrl_ptr-6);
 
-#ifdef LOG
-  printf("input_pnm: got mrl: %s %i %s\n",p->host,p->port,p->path);
-#endif
+  lprintf("got mrl: %s %i %s\n",p->host,p->port,p->path);
   
-  fd = host_connect (p->host, p->port);
+  fd = _x_io_tcp_connect (stream, p->host, p->port);
 
   if (fd == -1) {
-    printf ("input_pnm: failed to connect '%s'\n", p->host);
+    xprintf (p->stream->xine, XINE_VERBOSITY_LOG, _("input_pnm: failed to connect '%s'\n"), p->host);
     free(p->path);
     free(p->host);
     free(p->url);
@@ -874,7 +763,7 @@ pnm_t *pnm_connect(const char *mrl) {
 
   pnm_send_request(p,pnm_available_bandwidths[10]);
   if (!pnm_get_headers(p, &need_response)) {
-    printf ("input_pnm: failed to set up stream\n");
+    xprintf (p->stream->xine, XINE_VERBOSITY_LOG, _("input_pnm: failed to set up stream\n"));
     free(p->path);
     free(p->host);
     free(p->url);
@@ -899,7 +788,7 @@ int pnm_read (pnm_t *this, char *data, int len) {
   
   int to_copy=len;
   char *dest=data;
-  char *source=this->recv + this->recv_read;
+  char *source=(char*)(this->recv + this->recv_read);
   int fill=this->recv_size - this->recv_read;
   
   if (len < 0) return 0;
@@ -911,29 +800,30 @@ int pnm_read (pnm_t *this, char *data, int len) {
     this->recv_read=0;
 
     if (!pnm_get_stream_chunk (this)) {
-#ifdef LOG
-      printf ("input_pnm: %d of %d bytes provided\n", len-to_copy, len);
-#endif
+      lprintf ("%d of %d bytes provided\n", len-to_copy, len);
+
       return len-to_copy;
     }
-    source = this->recv;
+    source = (char*)this->recv;
     fill = this->recv_size - this->recv_read;
   }
 
   memcpy(dest, source, to_copy);
   this->recv_read += to_copy;
 
-#ifdef LOG
-  printf ("input_pnm: %d bytes provided\n", len);
-#endif
+  lprintf ("%d bytes provided\n", len);
 
   return len;
 }
 
-int pnm_peek_header (pnm_t *this, char *data) {
+int pnm_peek_header (pnm_t *this, char *data, int maxsize) {
 
-  memcpy (data, this->header, this->header_len);
-  return this->header_len;
+  int len;
+
+  len = (this->header_len < maxsize) ? this->header_len : maxsize;
+
+  memcpy(data, this->header, len);
+  return len;
 }
 
 void pnm_close(pnm_t *p) {
