@@ -1,5 +1,5 @@
 /*
-  $Id: xineplug_inp_vcd.c,v 1.26.2.4 2005/07/17 21:50:00 miguelfreitas Exp $
+  $Id: xineplug_inp_vcd.c,v 1.50 2006/07/10 22:08:29 dgp85 Exp $
  
   Copyright (C) 2002, 2003, 2004, 2005 Rocky Bernstein <rocky@panix.com>
   
@@ -167,8 +167,10 @@ vcd_input_plugin_t  my_vcd;
 /* Prototype definitions */
 static bool vcd_handle_events (void);
 static void vcd_close(vcd_input_class_t *class);
+#if LIBVCD_VERSION_NUM >= 23
 static void send_mouse_enter_leave_event(vcd_input_plugin_t *p_this, 
                                          bool b_mouse_in);
+#endif
 
 /* 
    If class->vcd_device is NULL or the empty string, 
@@ -261,7 +263,7 @@ vcd_add_mrl_slot(vcd_input_class_t *this, const char *mrl, off_t size,
   
   this->mrls[*i] = malloc(sizeof(xine_mrl_t));
   if (NULL==this->mrls[*i]) {
-    LOG_ERR("Can't malloc %d bytes for MRL slot %d (%s)", 
+    LOG_ERR("Can't malloc %zu bytes for MRL slot %u (%s)", 
             sizeof(xine_mrl_t), *i, mrl);
     return;
   }
@@ -272,7 +274,7 @@ vcd_add_mrl_slot(vcd_input_class_t *this, const char *mrl, off_t size,
   
   this->mrls[*i]->mrl = (char *) malloc(strlen(mrl) + 1);
   if (NULL==this->mrls[*i]->mrl) {
-    LOG_ERR("Can't malloc %d bytes for MRL name %s", sizeof(xine_mrl_t), mrl);
+    LOG_ERR("Can't malloc %zu bytes for MRL name %s", sizeof(xine_mrl_t), mrl);
   } else {
     sprintf(this->mrls[*i]->mrl, "%s", mrl);
   }
@@ -370,7 +372,7 @@ vcd_build_mrl_list(vcd_input_class_t *class, char *vcd_device)
 
   class->mrls = calloc(class->num_mrls, sizeof(xine_mrl_t *));
   if (NULL == class->mrls) {
-    LOG_ERR("Can't calloc %d MRL entries", class->mrls);
+    LOG_ERR("Can't calloc %d MRL entries", class->num_mrls);
     class->num_mrls = 0;
     return false;
   }
@@ -948,7 +950,7 @@ vcd_plugin_get_mrl (input_plugin_t *this_gen)
 {
   vcd_input_plugin_t *t         = (vcd_input_plugin_t *) this_gen;
   vcdplayer_t        *vcdplayer = &my_vcd.player;
-  unsigned int n;
+  int n;
   int size; /* need something to feed get_mrl_type_offset */
   int offset;
 
@@ -1002,9 +1004,6 @@ vcd_class_get_identifier (input_class_t *this_gen) {
   return SHORT_PLUGIN_NAME;
 }
 
-/* HACK until we put modify and put in vcdplayer. */
-#define dvdnav_mouse_select(a, b, c) 1
-
 /* 
    Handle all queued keyboard/mouse events. Return TRUE if this causes
    a change in the play item.
@@ -1175,7 +1174,9 @@ vcd_handle_events (void)
         xine_input_data_t *p_input = p_event->data;
         if (p_input->button == 1) 
         {
+#if LIBVCD_VERSION_NUM >= 23
           int i_selection;
+#endif
 
           dbg_print(INPUT_DBG_EVENT, 
                     "Button to x: %d, y: %d, scaled x: %d, scaled y %d\n", 
@@ -1284,11 +1285,11 @@ vcd_get_optional_data (input_plugin_t *this_gen,
     
   case INPUT_OPTIONAL_DATA_AUDIOLANG: 
     {
-      int8_t   channel;
-      channel = (int8_t) _x_get_audio_channel(my_vcd.stream);
+      uint8_t   channel;
+      channel = _x_get_audio_channel(my_vcd.stream);
 
       dbg_print(INPUT_DBG_EXT, "AUDIO CHANNEL = %d\n", channel);
-      if (-1 == channel) {
+      if (channel == (uint8_t)-1) {
         sprintf(data, " %s", "auto");
       } else {
         const vcdinfo_obj_t *p_vcdinfo= my_vcd.player.vcd;
@@ -1352,7 +1353,7 @@ vcd_class_get_autoplay_list (input_class_t *this_gen, int *num_files)
     *num_files = 0;
     return NULL;
   } else {
-    unsigned int i;
+    int i;
     int size = 0;
     vcdinfo_item_enum_t itemtype = 
       autoplay2itemtype[my_vcd.player.default_autoplay];
@@ -1498,6 +1499,8 @@ static void
 vcd_class_dispose (input_class_t *this_gen) {
   vcd_input_class_t  *class = (vcd_input_class_t *) this_gen;
 
+  class->xine->config->unregister_callback(class->xine->config,
+					   "media.vcd.device");
   gl_default_vcd_log_handler  = vcd_log_set_handler (uninit_log_handler);
   gl_default_cdio_log_handler = 
     cdio_log_set_handler ((cdio_log_handler_t) uninit_log_handler);
@@ -1540,6 +1543,7 @@ vcd_update_title_display(void)
   xine_event_send(my_vcd.stream, &uevent);
 }
 
+#if LIBVCD_VERSION_NUM >= 23
 static void 
 send_mouse_enter_leave_event(vcd_input_plugin_t *p_this, bool b_mouse_in) 
 {
@@ -1567,6 +1571,7 @@ send_mouse_enter_leave_event(vcd_input_plugin_t *p_this, bool b_mouse_in)
   if (!b_mouse_in)
     p_this->i_mouse_button = -1;
 }
+#endif
  
 /* 
    Not much special initialization needed here. All of the initialization 
@@ -1985,9 +1990,9 @@ _("For tracking down bugs in the VCD plugin. Mask values are:\n"
    the next API number since in both cases the plugin is broken.
  */
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 16, (char *) SHORT_PLUGIN_NAME,
+  { PLUGIN_INPUT | PLUGIN_MUST_PRELOAD, 17, (char *) SHORT_PLUGIN_NAME,
     XINE_VERSION_CODE, NULL, vcd_init },
   { PLUGIN_NONE, 0, (char *) "", 0, NULL, NULL }
 };
