@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2005 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.37 2004/12/16 13:59:10 mroi Exp $
+ * $Id: xine_decoder.c,v 1.48 2006/07/10 22:08:29 dgp85 Exp $
  *
  */
 
@@ -38,7 +38,12 @@
 #include "audio_out.h"
 #include "buffer.h"
 #include "xineutils.h"
-#include "faad.h"
+#include "common.h"
+#include "structs.h"
+#include "decoder.h"
+#include "syntax.h"
+
+#define FAAD_MIN_STREAMSIZE 768 /* 6144 bits/channel */
 
 typedef struct {
   audio_decoder_class_t   decoder_class;
@@ -50,9 +55,9 @@ typedef struct faad_decoder_s {
   xine_stream_t    *stream;
   
   /* faad2 stuff */
-  faacDecHandle           faac_dec;
-  faacDecConfigurationPtr faac_cfg;
-  faacDecFrameInfo        faac_finfo;
+  NeAACDecHandle           faac_dec;
+  NeAACDecConfigurationPtr faac_cfg;
+  NeAACDecFrameInfo        faac_finfo;
   int                     faac_failed;
  
   int              raw_mode;
@@ -66,12 +71,16 @@ typedef struct faad_decoder_s {
   unsigned char   *dec_config;
   int              dec_config_size;
   
-  unsigned long    rate;
+  uint32_t         rate;
   int              bits_per_sample; 
   unsigned char    num_channels; 
+  int              sbr;
   uint32_t         ao_cap_mode; 
    
   int              output_open;
+
+  unsigned long    total_time;
+  unsigned long    total_data;
 } faad_decoder_t;
 
 
@@ -81,49 +90,68 @@ static void faad_reset (audio_decoder_t *this_gen) {
   this->size = 0;
 }
 
+static void faad_meta_info_set ( faad_decoder_t *this ) {
+  switch (this->num_channels) {
+    case 1:
+      if (this->faac_finfo.sbr == SBR_UPSAMPLED)
+        _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC,
+                              "HE-AAC 1.0 (libfaad)");
+      else
+        _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC,
+                              "AAC 1.0 (libfaad)");
+      break;
+    case 2:
+      /* check if this is downmixed 5.1 */
+      if (!this->faac_cfg || !this->faac_cfg->downMatrix) {
+        if (this->faac_finfo.sbr == SBR_UPSAMPLED)
+          _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC,
+                                "HE-AAC 2.0 (libfaad)");
+        else
+          _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC,
+                                "AAC 2.0 (libfaad)");
+        break;
+      }
+    case 6:
+      if (this->faac_finfo.sbr == SBR_UPSAMPLED)
+        _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC,
+                              "HE-AAC 5.1 (libfaad)");
+      else
+        _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC,
+                              "AAC 5.1 (libfaad)");
+      break;
+  }
+}
+
 static int faad_open_dec( faad_decoder_t *this ) {
   int used;
 
-  this->faac_dec = faacDecOpen();
+  this->faac_dec = NeAACDecOpen();
   if( !this->faac_dec ) {
     xprintf( this->stream->xine, XINE_VERBOSITY_LOG,
-             _("libfaad: libfaad faacDecOpen() failed.\n"));
+             _("libfaad: libfaad NeAACDecOpen() failed.\n"));
     this->faac_failed++;
   } else {
     if( this->dec_config ) {
-      used = faacDecInit2(this->faac_dec, this->dec_config, this->dec_config_size,
+      used = NeAACDecInit2(this->faac_dec, this->dec_config, this->dec_config_size,
                           &this->rate, &this->num_channels);
       
       if( used < 0 ) {
         xprintf( this->stream->xine, XINE_VERBOSITY_LOG,
-                _("libfaad: libfaad faacDecInit2 failed.\n"));
+                _("libfaad: libfaad NeAACDecInit2 failed.\n"));
         this->faac_failed++;
       } else
-        lprintf( "faacDecInit2 returned rate=%ld channels=%d\n",
+        lprintf( "NeAACDecInit2 returned rate=%ld channels=%d\n",
                  this->rate, this->num_channels );
     } else {
-      /* Set the default object type and samplerate */
-      /* This is useful for RAW AAC files */
-      this->faac_cfg = faacDecGetCurrentConfiguration(this->faac_dec);
-      if( this->faac_cfg ) {
-        this->faac_cfg->defSampleRate = this->rate;
-        this->faac_cfg->outputFormat = FAAD_FMT_16BIT;
-        this->bits_per_sample = 16;
-        this->faac_cfg->useOldADTSFormat = 0;
-        this->faac_cfg->dontUpSampleImplicitSBR = 1;
-        faacDecSetConfiguration(this->faac_dec, this->faac_cfg);
-      }
-  
-
-      used = faacDecInit(this->faac_dec, this->buf, this->size,
+      used = NeAACDecInit(this->faac_dec, this->buf, this->size,
                         &this->rate, &this->num_channels);
         
       if( used < 0 ) {
         xprintf ( this->stream->xine, XINE_VERBOSITY_LOG,
-                  _("libfaad: libfaad faacDecInit failed.\n"));
+                  _("libfaad: libfaad NeAACDecInit failed.\n"));
         this->faac_failed++;
       } else {
-        lprintf( "faacDecInit() returned rate=%ld channels=%d (used=%d)\n",
+        lprintf( "NeAACDecInit() returned rate=%ld channels=%d (used=%d)\n",
                  this->rate, this->num_channels, used);
                       
         this->size -= used;
@@ -132,12 +160,17 @@ static int faad_open_dec( faad_decoder_t *this ) {
     }
   }
   
+  if( !this->bits_per_sample )
+    this->bits_per_sample = 16;
+  
   if( this->faac_failed ) {
     if( this->faac_dec ) {
-      faacDecClose( this->faac_dec );
+      NeAACDecClose( this->faac_dec );
       this->faac_dec = NULL;
     }
     _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_HANDLED, 0);
+  } else {
+    faad_meta_info_set(this);
   }
   
   return this->faac_failed;
@@ -156,9 +189,9 @@ static int faad_open_output( faad_decoder_t *this ) {
         this->ao_cap_mode = AO_CAP_MODE_5_1CHANNEL;
         break;
       } else {
-        this->faac_cfg = faacDecGetCurrentConfiguration(this->faac_dec);
+        this->faac_cfg = NeAACDecGetCurrentConfiguration(this->faac_dec);
         this->faac_cfg->downMatrix = 1;
-        faacDecSetConfiguration(this->faac_dec, this->faac_cfg);
+        NeAACDecSetConfiguration(this->faac_dec, this->faac_cfg);
         this->num_channels = 2;
       }
     case 2:
@@ -188,17 +221,17 @@ static void faad_decode_audio ( faad_decoder_t *this, int end_frame ) {
   while( (!this->raw_mode && end_frame && this->size >= 10) ||
          (this->raw_mode && this->size >= this->rec_audio_src_size) ) {
       
-    sample_buffer = faacDecDecode(this->faac_dec, 
+    sample_buffer = NeAACDecDecode(this->faac_dec, 
                                   &this->faac_finfo, inbuf, sample_size);
  
     if( !sample_buffer ) {
       xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
-              "libfaad: %s\n", faacDecGetErrorMessage(this->faac_finfo.error));
+              "libfaad: %s\n", NeAACDecGetErrorMessage(this->faac_finfo.error));
       used = 1;    
     } else {
       used = this->faac_finfo.bytesconsumed;
           
-      /* raw AAC parameters is only known after decoding the first frame */
+      /* raw AAC parameters might only be known after decoding the first frame */
       if( !this->dec_config &&
           (this->num_channels != this->faac_finfo.channels ||
            this->rate != this->faac_finfo.samplerate) ) {
@@ -212,8 +245,29 @@ static void faad_decode_audio ( faad_decoder_t *this, int end_frame ) {
         this->stream->audio_out->close (this->stream->audio_out, this->stream);
         this->output_open = 0;
         faad_open_output( this );
+
+        faad_meta_info_set( this );
+      }
+      
+      /* faad doesn't tell us about sbr until after the first frame */
+      if (this->sbr != this->faac_finfo.sbr) {
+        this->sbr = this->faac_finfo.sbr;
+        faad_meta_info_set( this );
       }
     
+      /* estimate bitrate */
+      this->total_time += (1000*this->faac_finfo.samples/(this->rate*this->num_channels));
+      this->total_data += 8*used;
+
+      if ((this->total_time > LONG_MAX) || (this->total_data > LONG_MAX)) {
+        this->total_time >>= 2;
+        this->total_data >>= 2;
+      }
+
+      if (this->total_time)
+        _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITRATE,
+                           1000*(this->total_data/this->total_time));
+
       decoded = this->faac_finfo.samples * 2; /* 1 sample = 2 bytes */
 
       lprintf("decoded %d/%d output %ld\n",
@@ -284,9 +338,6 @@ static void faad_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
     this->bits_per_sample=buf->decoder_info[2] ; 
     this->num_channels=buf->decoder_info[3] ; 
  
-    if( !this->bits_per_sample )
-      this->bits_per_sample = 16;
-    
     if( buf->size > sizeof(xine_waveformatex) ) {
       xine_waveformatex *wavex = (xine_waveformatex *) buf->content;
 
@@ -298,13 +349,10 @@ static void faad_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
                
         if( faad_open_dec(this) )
           return;
+
+        this->raw_mode = 0;
       }
     }
-                                               
-    /* stream/meta info */
-    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC,
-      "AAC (libfaad)");
-
   } else {
 
     lprintf ("decoding %d data bytes...\n", buf->size);
@@ -358,7 +406,7 @@ static void faad_dispose (audio_decoder_t *this_gen) {
   this->dec_config_size = 0;
   
   if( this->faac_dec )
-    faacDecClose(this->faac_dec);
+    NeAACDecClose(this->faac_dec);
   this->faac_dec = NULL;
   this->faac_failed = 0;
 
@@ -387,8 +435,10 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   this->max_audio_src_size = 0;
   this->dec_config         = NULL;
   this->dec_config_size    = 0;
+  this->total_time         = 0;
+  this->total_data         = 0;
 
-  this->rate               = 44100;
+  this->rate               = 0;
 
   return &this->audio_decoder;
 }
@@ -423,12 +473,12 @@ static uint32_t audio_types[] = {
   BUF_AUDIO_AAC, 0
  };
 
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   1                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_AUDIO_DECODER, 15, "faad", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
