@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_decoder.c,v 1.39.2.3 2005/07/25 21:22:46 tmattern Exp $
+ * $Id: video_decoder.c,v 1.64 2006/12/02 21:06:18 miguelfreitas Exp $
  *
  * xine video decoder plugin using ffmpeg
  *
@@ -113,6 +113,8 @@ struct ff_video_decoder_s {
   int               yuv_init;
 
   int               cs_convert_init;
+  int               is_direct_rendering_disabled;
+
   AVPaletteControl  palette_control;
 };
 
@@ -146,11 +148,16 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
   avcodec_align_dimensions(context, &width, &height);
 
   if( this->context->pix_fmt != PIX_FMT_YUV420P ) {
-    xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
-            _("ffmpeg_video_dec: unsupported frame format, DR1 disabled.\n"));
+    if (!this->is_direct_rendering_disabled) {
+      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+              _("ffmpeg_video_dec: unsupported frame format, DR1 disabled.\n"));
+      this->is_direct_rendering_disabled = 1;
+    }
 
-    this->context->get_buffer = avcodec_default_get_buffer;
-    this->context->release_buffer = avcodec_default_release_buffer;
+    /* FIXME: why should i have to do that ? */
+    av_frame->data[0]= NULL;
+    av_frame->data[1]= NULL;
+    av_frame->data[2]= NULL;
     return avcodec_default_get_buffer(context, av_frame);
   }
   
@@ -159,11 +166,15 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
       this->crop_right = width - this->bih.biWidth;
       this->crop_bottom = height - this->bih.biHeight;
     } else {
-      xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
-              _("ffmpeg_video_dec: unsupported frame dimensions, DR1 disabled.\n"));
-
-      this->context->get_buffer = avcodec_default_get_buffer;
-      this->context->release_buffer = avcodec_default_release_buffer;
+      if (!this->is_direct_rendering_disabled) {
+        xprintf(this->stream->xine, XINE_VERBOSITY_LOG, 
+                _("ffmpeg_video_dec: unsupported frame dimensions, DR1 disabled.\n"));
+        this->is_direct_rendering_disabled = 1;
+      }
+      /* FIXME: why should i have to do that ? */
+      av_frame->data[0]= NULL;
+      av_frame->data[1]= NULL;
+      av_frame->data[2]= NULL;
       return avcodec_default_get_buffer(context, av_frame);
     }
   }
@@ -196,18 +207,20 @@ static int get_buffer(AVCodecContext *context, AVFrame *av_frame){
 }
 
 static void release_buffer(struct AVCodecContext *context, AVFrame *av_frame){
-  vo_frame_t *img = (vo_frame_t *)av_frame->opaque;
+
+  if (av_frame->type == FF_BUFFER_TYPE_USER) {
+    vo_frame_t *img = (vo_frame_t *)av_frame->opaque;
     
-  assert(av_frame->type == FF_BUFFER_TYPE_USER);
-  assert(av_frame->opaque);  
-  
+    assert(av_frame->opaque);  
+    img->free(img);
+  } else {
+    avcodec_default_release_buffer(context, av_frame);
+  }
+
+  av_frame->opaque = NULL;
   av_frame->data[0]= NULL;
   av_frame->data[1]= NULL;
   av_frame->data[2]= NULL;
-
-  img->free(img);
-
-  av_frame->opaque = NULL;
 }
 #endif
 
@@ -217,6 +230,7 @@ static const ff_codec_t ff_video_lookup[] = {
   {BUF_VIDEO_MSMPEG4_V3,  CODEC_ID_MSMPEG4V3, "Microsoft MPEG-4 v3 (ffmpeg)"},
   {BUF_VIDEO_WMV7,        CODEC_ID_WMV1,      "MS Windows Media Video 7 (ffmpeg)"},
   {BUF_VIDEO_WMV8,        CODEC_ID_WMV2,      "MS Windows Media Video 8 (ffmpeg)"},
+  {BUF_VIDEO_WMV9,        CODEC_ID_WMV3,      "MS Windows Media Video 9 (ffmpeg)"},
   {BUF_VIDEO_MPEG4,       CODEC_ID_MPEG4,     "ISO MPEG-4 (ffmpeg)"},
   {BUF_VIDEO_XVID,        CODEC_ID_MPEG4,     "ISO MPEG-4 (XviD, ffmpeg)"},
   {BUF_VIDEO_DIVX5,       CODEC_ID_MPEG4,     "ISO MPEG-4 (DivX5, ffmpeg)"},
@@ -250,6 +264,7 @@ static const ff_codec_t ff_video_lookup[] = {
   {BUF_VIDEO_8BPS,        CODEC_ID_8BPS,      "Planar RGB (ffmpeg)"},
   {BUF_VIDEO_SMC,         CODEC_ID_SMC,       "Apple Quicktime Graphics/SMC (ffmpeg)"},
   {BUF_VIDEO_DUCKTM1,     CODEC_ID_TRUEMOTION1,"Duck TrueMotion v1 (ffmpeg)"},
+  {BUF_VIDEO_DUCKTM2,     CODEC_ID_TRUEMOTION2,"Duck TrueMotion v2 (ffmpeg)"},
   {BUF_VIDEO_VMD,         CODEC_ID_VMDVIDEO,   "Sierra VMD Video (ffmpeg)"},
   {BUF_VIDEO_ZLIB,        CODEC_ID_ZLIB,       "ZLIB Video (ffmpeg)"},
   {BUF_VIDEO_MSZH,        CODEC_ID_MSZH,       "MSZH Video (ffmpeg)"},
@@ -259,18 +274,41 @@ static const ff_codec_t ff_video_lookup[] = {
   {BUF_VIDEO_FLV1,        CODEC_ID_FLV1,       "Flash Video (ffmpeg)"},
   {BUF_VIDEO_QTRLE,       CODEC_ID_QTRLE,      "Apple Quicktime Animation/RLE (ffmpeg)"},
   {BUF_VIDEO_H264,        CODEC_ID_H264,       "H.264/AVC (ffmpeg)"},
-  {BUF_VIDEO_MPEG,        CODEC_ID_MPEG1VIDEO, "MPEG 1/2 (ffmpeg)"} };
+  {BUF_VIDEO_H261,        CODEC_ID_H261,       "H.261 (ffmpeg)"},
+  {BUF_VIDEO_AASC,        CODEC_ID_AASC,       "Autodesk Video (ffmpeg)"},
+  {BUF_VIDEO_LOCO,        CODEC_ID_LOCO,       "LOCO (ffmpeg)"},
+  {BUF_VIDEO_QDRW,        CODEC_ID_QDRAW,      "QuickDraw (ffmpeg)"},
+  {BUF_VIDEO_QPEG,        CODEC_ID_QPEG,       "Q-Team QPEG (ffmpeg)"},
+  {BUF_VIDEO_TSCC,        CODEC_ID_TSCC,       "TechSmith Video (ffmpeg)"},
+  {BUF_VIDEO_ULTI,        CODEC_ID_ULTI,       "IBM UltiMotion (ffmpeg)"},
+  {BUF_VIDEO_WNV1,        CODEC_ID_WNV1,       "Winnow Video (ffmpeg)"},
+  {BUF_VIDEO_XL,          CODEC_ID_VIXL,       "Miro/Pinnacle VideoXL (ffmpeg)"},
+  {BUF_VIDEO_RT21,        CODEC_ID_INDEO2,     "Indeo/RealTime 2 (ffmpeg)"},
+  {BUF_VIDEO_FPS1,        CODEC_ID_FRAPS,      "Fraps (ffmpeg)"},
+  {BUF_VIDEO_MPEG,        CODEC_ID_MPEG1VIDEO, "MPEG 1/2 (ffmpeg)"},
+  {BUF_VIDEO_CSCD,        CODEC_ID_CSCD,       "CamStudio (ffmpeg)"},
+  {BUF_VIDEO_AVS,         CODEC_ID_AVS,        "AVS (ffmpeg)"},
+  {BUF_VIDEO_ALGMM,       CODEC_ID_MMVIDEO,    "American Laser Games MM (ffmpeg)"},
+  {BUF_VIDEO_ZMBV,        CODEC_ID_ZMBV,       "Zip Motion Blocks Video (ffmpeg)"},
+  {BUF_VIDEO_SMACKER,     CODEC_ID_SMACKVIDEO, "Smacker (ffmpeg)"},
+  {BUF_VIDEO_NUV,         CODEC_ID_NUV,        "NuppelVideo (ffmpeg)"},
+  {BUF_VIDEO_KMVC,        CODEC_ID_KMVC,       "Karl Morton's Video Codec (ffmpeg)"},
+  {BUF_VIDEO_FLASHSV,     CODEC_ID_FLASHSV,    "Flash Screen Video (ffmpeg)"},
+  {BUF_VIDEO_CAVS,        CODEC_ID_CAVS,       "Chinese AVS (ffmpeg)"},
+};
 
 
-static void init_video_codec (ff_video_decoder_t *this, int codec_type) {
-  int i;
+static void init_video_codec (ff_video_decoder_t *this, unsigned int codec_type) {
+  size_t i;
 
   /* find the decoder */
   this->codec = NULL;
 
   for(i = 0; i < sizeof(ff_video_lookup)/sizeof(ff_codec_t); i++)
     if(ff_video_lookup[i].type == codec_type) {
+      pthread_mutex_lock(&ffmpeg_lock);
       this->codec = avcodec_find_decoder(ff_video_lookup[i].id);
+      pthread_mutex_unlock(&ffmpeg_lock);
       _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC,
                             ff_video_lookup[i].name);
       break;
@@ -301,8 +339,11 @@ static void init_video_codec (ff_video_decoder_t *this, int codec_type) {
    * this flag here in case we are going to use direct rendering */
   if(this->codec->capabilities & CODEC_CAP_DR1) {
     this->context->flags |= CODEC_FLAG_EMU_EDGE;
-  } 
+  }
+ 
+  pthread_mutex_lock(&ffmpeg_lock);
   if (avcodec_open (this->context, this->codec) < 0) {
+    pthread_mutex_unlock(&ffmpeg_lock);
     xprintf (this->stream->xine, XINE_VERBOSITY_LOG, 
              _("ffmpeg_video_dec: couldn't open decoder\n"));
     free(this->context);
@@ -310,6 +351,7 @@ static void init_video_codec (ff_video_decoder_t *this, int codec_type) {
     _x_stream_info_set(this->stream, XINE_STREAM_INFO_VIDEO_HANDLED, 0);
     return;
   }
+  pthread_mutex_unlock(&ffmpeg_lock);
 
   lprintf("lavc decoder opened\n");
 
@@ -802,7 +844,7 @@ static void ff_handle_header_buffer (ff_video_decoder_t *this, buf_element_t *bu
       memcpy ( &this->bih, this->buf, sizeof(xine_bmiheader) );
 
       if (this->bih.biSize > sizeof(xine_bmiheader)) {
-        this->context->extradata_size = this->bih.biSize - sizeof(xine_bmiheader);
+      this->context->extradata_size = this->bih.biSize - sizeof(xine_bmiheader);
         this->context->extradata = malloc(this->context->extradata_size + 
                                           FF_INPUT_BUFFER_PADDING_SIZE);
         memcpy(this->context->extradata, this->buf + sizeof(xine_bmiheader),
@@ -840,8 +882,6 @@ static void ff_handle_header_buffer (ff_video_decoder_t *this, buf_element_t *bu
 }
 
 static void ff_handle_special_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
-  int i;
-
   /* take care of all the various types of special buffers 
   * note that order is important here */
   lprintf("special buffer\n");
@@ -867,6 +907,7 @@ static void ff_handle_special_buffer (ff_video_decoder_t *this, buf_element_t *b
       buf->decoder_info[2]);
       
   } else if (buf->decoder_info[1] == BUF_SPECIAL_PALETTE) {
+    unsigned int i;
 
     palette_entry_t *demuxer_palette;
     AVPaletteControl *decoder_palette;
@@ -885,6 +926,7 @@ static void ff_handle_special_buffer (ff_video_decoder_t *this, buf_element_t *b
     decoder_palette->palette_changed = 1;
 
   } else if (buf->decoder_info[1] == BUF_SPECIAL_RV_CHUNK_TABLE) {
+    int i;
   
     lprintf("BUF_SPECIAL_RV_CHUNK_TABLE\n");
     this->context->slice_count = buf->decoder_info[2]+1;
@@ -1075,6 +1117,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
     vo_frame_t *img;
     int         free_img;
     int         got_picture, len;
+    int         got_one_picture = 0;
     int         offset = 0;
     int         codec_type = buf->type & 0xFFFF0000;
 
@@ -1091,11 +1134,11 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
         /* skip decoding b frames if too late */
         this->context->hurry_up = (this->skipframes > 0);
 
-	lprintf("buffer size: %d\n", this->size);
+        lprintf("buffer size: %d\n", this->size);
         len = avcodec_decode_video (this->context, this->av_frame,
                                     &got_picture, &chunk_buf[offset],
                                     this->size);
-	lprintf("consumed size: %d\n", len);
+        lprintf("consumed size: %d, got_picture: %d\n", len, got_picture);
         if ((len <= 0) || (len > this->size)) {
           xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, 
                     "ffmpeg_video_dec: error decompressing frame\n");
@@ -1127,6 +1170,7 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
 
       if (got_picture && this->av_frame->data[0]) {
         /* got a picture, draw it */
+        got_one_picture = 1;
         if(!this->av_frame->opaque) {
 	  /* indirect rendering */
 
@@ -1215,22 +1259,23 @@ static void ff_handle_buffer (ff_video_decoder_t *this, buf_element_t *buf) {
 
         if(free_img)
           img->free(img);
-
-      } else {
-        /* no frame, output a bad frame */
-        img = this->stream->video_out->get_frame (this->stream->video_out,
-                                                  this->bih.biWidth,
-                                                  this->bih.biHeight,
-                                                  this->aspect_ratio, 
-                                                  this->output_format,
-                                                  VO_BOTH_FIELDS|this->frame_flags);
-        img->pts       = 0;
-        img->duration  = this->video_step;
-        img->bad_frame = 1;
-        this->skipframes = img->draw(img, this->stream);
-        img->free(img);
       }
     }
+
+    if (!got_one_picture) {
+      /* skipped frame, output a bad frame */
+      img = this->stream->video_out->get_frame (this->stream->video_out,
+                                                this->bih.biWidth,
+                                                this->bih.biHeight,
+                                                this->aspect_ratio, 
+                                                this->output_format,
+                                                VO_BOTH_FIELDS|this->frame_flags);
+      img->pts       = 0;
+      img->duration  = this->video_step;
+      img->bad_frame = 1;
+      this->skipframes = img->draw(img, this->stream);
+      img->free(img);
+    }
   }
 }
 
@@ -1317,7 +1362,9 @@ static void ff_dispose (video_decoder_t *this_gen) {
   lprintf ("ff_dispose\n");
 
   if (this->decoder_ok) {
+    pthread_mutex_lock(&ffmpeg_lock);
     avcodec_close (this->context);
+    pthread_mutex_unlock(&ffmpeg_lock);
 
     this->stream->video_out->close(this->stream->video_out, this->stream);
     this->decoder_ok = 0;
@@ -1474,6 +1521,7 @@ static uint32_t supported_video_types[] = {
   BUF_VIDEO_SMC,
   BUF_VIDEO_VMD,
   BUF_VIDEO_DUCKTM1,
+  BUF_VIDEO_DUCKTM2,
   BUF_VIDEO_ZLIB,
   BUF_VIDEO_MSZH,
   BUF_VIDEO_ASV1,
@@ -1482,6 +1530,26 @@ static uint32_t supported_video_types[] = {
   BUF_VIDEO_FLV1,
   BUF_VIDEO_QTRLE,
   BUF_VIDEO_H264,
+  BUF_VIDEO_H261,
+  BUF_VIDEO_AASC,
+  BUF_VIDEO_LOCO,
+  BUF_VIDEO_QDRW,
+  BUF_VIDEO_QPEG,
+  BUF_VIDEO_TSCC,
+  BUF_VIDEO_ULTI,
+  BUF_VIDEO_WNV1,
+  BUF_VIDEO_XL,
+  BUF_VIDEO_RT21,
+  BUF_VIDEO_FPS1,
+  BUF_VIDEO_CSCD,
+  BUF_VIDEO_ALGMM,
+  BUF_VIDEO_ZMBV,
+  BUF_VIDEO_AVS,
+  BUF_VIDEO_SMACKER,
+  BUF_VIDEO_NUV,
+  BUF_VIDEO_KMVC,
+  BUF_VIDEO_FLASHSV,
+  BUF_VIDEO_CAVS,
   0 
 };
 
@@ -1490,6 +1558,11 @@ static uint32_t wmv8_video_types[] = {
   0 
 };
 
+static uint32_t wmv9_video_types[] = { 
+  BUF_VIDEO_WMV9,
+  0 
+};
+
 decoder_info_t dec_info_ffmpeg_video = {
   supported_video_types,   /* supported types */
   6                        /* priority        */
@@ -1499,3 +1572,8 @@ decoder_info_t dec_info_ffmpeg_wmv8 = {
   wmv8_video_types,        /* supported types */
   0                        /* priority        */
 };
+
+decoder_info_t dec_info_ffmpeg_wmv9 = {
+  wmv9_video_types,        /* supported types */
+  0                        /* priority        */
+};
