@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.168 2006/05/03 19:46:07 dsalt Exp $
+ * $Id: xine_decoder.c,v 1.171 2006/10/11 12:53:12 dgp85 Exp $
  *
  * xine decoder plugin using ffmpeg
  *
@@ -38,6 +38,7 @@
 pthread_once_t once_control = PTHREAD_ONCE_INIT;
 pthread_mutex_t ffmpeg_lock;
 
+#ifndef HAVE_FFMPEG
 void avcodec_register_all(void)
 {
     static int inited = 0;
@@ -131,7 +132,22 @@ void avcodec_register_all(void)
     register_avcodec(&shorten_decoder);
     register_avcodec(&qdm2_decoder);
     register_avcodec(&truemotion2_decoder);
+    register_avcodec(&wmv3_decoder);
+    register_avcodec(&cscd_decoder);
+    register_avcodec(&mmvideo_decoder);
+    register_avcodec(&zmbv_decoder);
+    register_avcodec(&avs_decoder);
+    register_avcodec(&smacker_decoder);
+    register_avcodec(&smackaud_decoder);
+    register_avcodec(&nuv_decoder);
+    register_avcodec(&kmvc_decoder);
+    register_avcodec(&flashsv_decoder);
+    register_avcodec(&cavs_decoder);
+    register_avcodec(&cook_decoder);
+    register_avcodec(&truespeech_decoder);
+    register_avcodec(&tta_decoder);
 }
+#endif
 
 void init_once_routine(void) {
   pthread_mutex_init(&ffmpeg_lock, NULL);
@@ -143,10 +159,11 @@ void init_once_routine(void) {
  * exported plugin catalog entry
  */
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_VIDEO_DECODER | PLUGIN_MUST_PRELOAD, 18, "ffmpegvideo", XINE_VERSION_CODE, &dec_info_ffmpeg_video, init_video_plugin },
   { PLUGIN_VIDEO_DECODER, 18, "ffmpeg-wmv8", XINE_VERSION_CODE, &dec_info_ffmpeg_wmv8, init_video_plugin },
+  { PLUGIN_VIDEO_DECODER, 18, "ffmpeg-wmv9", XINE_VERSION_CODE, &dec_info_ffmpeg_wmv9, init_video_plugin },
   { PLUGIN_AUDIO_DECODER, 15, "ffmpegaudio", XINE_VERSION_CODE, &dec_info_ffmpeg_audio, init_audio_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
