@@ -248,23 +248,7 @@ flac_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
         this->bits_per_sample = buf->decoder_info[2];
         this->channels        = buf->decoder_info[3];
 
-        switch (this->channels)
-        {
-            case 1:
-                mode = AO_CAP_MODE_MONO;
-            break;
-            case 2:
-                mode = AO_CAP_MODE_STEREO;
-            break;
-            case 4:
-                mode = AO_CAP_MODE_4CHANNEL;
-            break;
-            case 5:
-                mode = AO_CAP_MODE_5CHANNEL;
-            break;
-            case 6:
-                mode = AO_CAP_MODE_5_1CHANNEL;
-        }
+	mode = _x_ao_channels2mode(this->channels);
 
         if (!this->output_open)
         {
@@ -371,7 +355,10 @@ open_plugin (audio_decoder_class_t *class_gen, xine_stream_t *stream) {
 
     FLAC__stream_decoder_set_client_data (this->flac_decoder, this);
 
-    FLAC__stream_decoder_init (this->flac_decoder);
+    if (FLAC__stream_decoder_init (this->flac_decoder) == FLAC__STREAM_DECODER_SEARCH_FOR_METADATA) {
+	    free (this);
+	    return NULL;
+    }
 
     return (audio_decoder_t *) this;
 }
@@ -411,14 +398,14 @@ static uint32_t audio_types[] = {
   BUF_AUDIO_FLAC, 0
  };
 
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   5                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 25, "flac", XINE_VERSION_CODE, NULL, demux_flac_init_class },
+  { PLUGIN_DEMUX, 26, "flac", XINE_VERSION_CODE, NULL, demux_flac_init_class },
   { PLUGIN_AUDIO_DECODER, 15, "flacdec", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
