@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.38 2002/11/20 11:57:43 mroi Exp $
+ * $Id: xine_decoder.c,v 1.61 2006/07/10 22:08:29 dgp85 Exp $
  * 
  * 31-8-2001 Added LPCM rate sensing.
  *   (c) 2001 James Courtier-Dutton James@superbug.demon.co.uk
@@ -26,6 +26,8 @@
 #ifndef __sun
 #define _XOPEN_SOURCE 500
 #endif
+/* avoid compiler warnings */
+#define _BSD_SOURCE 1
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -39,6 +41,11 @@
 #include "audio_out.h"
 #include "buffer.h"
 
+#ifdef WIN32
+#include <winsock.h>
+/*#include <Winsock2.h>*/ /* htons */
+#endif
+
 typedef struct {
   audio_decoder_class_t   decoder_class;
 } lpcm_class_t;
@@ -57,16 +64,16 @@ typedef struct lpcm_decoder_s {
   int		   cpu_be;	/* TRUE, if we're a Big endian CPU */
 } lpcm_decoder_t;
 
-void lpcm_reset (audio_decoder_t *this_gen) {
+static void lpcm_reset (audio_decoder_t *this_gen) {
 
   /* lpcm_decoder_t *this = (lpcm_decoder_t *) this_gen; */
 
 }
 
-void lpcm_discontinuity (audio_decoder_t *this_gen) {
+static void lpcm_discontinuity (audio_decoder_t *this_gen) {
 }
 
-void lpcm_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
+static void lpcm_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
   lpcm_decoder_t *this = (lpcm_decoder_t *) this_gen;
   int16_t        *sample_buffer=(int16_t *)buf->content;
@@ -81,12 +88,17 @@ void lpcm_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
   /* get config byte from mpeg2 stream */
   if ( (buf->decoder_flags & BUF_FLAG_SPECIAL) &&
         buf->decoder_info[1] == BUF_SPECIAL_LPCM_CONFIG ) {
-    int bits_per_sample = 16;
-    int sample_rate;
-    int num_channels;
+    unsigned int bits_per_sample = 16;
+    unsigned int sample_rate = 0;
+    unsigned int num_channels;
       
     num_channels = (buf->decoder_info[2] & 0x7) + 1;
-    sample_rate = buf->decoder_info[2] & 0x10 ? 96000 : 48000;
+    switch ((buf->decoder_info[2]>>4) & 3) {
+    case 0: sample_rate = 48000; break;
+    case 1: sample_rate = 96000; break;
+    case 2: sample_rate = 44100; break;
+    case 3: sample_rate = 32000; break;
+    }
     switch ((buf->decoder_info[2]>>6) & 3) {
       case 0: bits_per_sample = 16; break;
       case 1: bits_per_sample = 20; break;
@@ -104,15 +116,11 @@ void lpcm_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
     } 
   }
   
-  if( buf->decoder_flags & BUF_FLAG_HEADER ) {
+  if( buf->decoder_flags & BUF_FLAG_STDHEADER ) {
     this->rate=buf->decoder_info[1];
     this->bits_per_sample=buf->decoder_info[2] ; 
     this->number_of_channels=buf->decoder_info[3] ; 
     format_changed++;
-
-    /* stream/meta info */
-    this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] = strdup("Linear PCM");
-    this->stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED] = 1;
   }
   
   /*
@@ -122,23 +130,35 @@ void lpcm_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
     if (this->output_open)
         this->stream->audio_out->close (this->stream->audio_out, this->stream);
 
-    this->ao_cap_mode=(this->number_of_channels == 2) ? AO_CAP_MODE_STEREO : AO_CAP_MODE_MONO; 
-   
-    this->output_open = this->stream->audio_out->open (this->stream->audio_out, this->stream,
-                                               (this->bits_per_sample>16)?16:this->bits_per_sample,
+    this->ao_cap_mode=_x_ao_channels2mode(this->number_of_channels);
+
+    /* force 24-bit samples into 16 bits for now */
+    if (this->bits_per_sample == 24)
+      this->output_open = this->stream->audio_out->open (this->stream->audio_out, this->stream,
+                                               16,
                                                this->rate,
                                                this->ao_cap_mode) ;
+    else
+      this->output_open = this->stream->audio_out->open (this->stream->audio_out, this->stream,
+                                               this->bits_per_sample,
+                                               this->rate,
+                                               this->ao_cap_mode) ;
+
+    /* stream/meta info */
+    _x_meta_info_set_utf8(this->stream, XINE_META_INFO_AUDIOCODEC, "Linear PCM");
+    _x_stream_info_set(this->stream, XINE_STREAM_INFO_AUDIO_BITRATE, 
+        this->bits_per_sample * this->rate * this->number_of_channels);
   }
 
   if (!this->output_open || (buf->decoder_flags & BUF_FLAG_HEADER) ) 
     return;
 
   audio_buffer = this->stream->audio_out->get_buffer (this->stream->audio_out);
-  
+
   /* Swap LPCM samples into native byte order, if necessary */
   buf->type &= 0xffff0000;
   stream_be = ( buf->type == BUF_AUDIO_LPCM_BE );
-  
+ 
   if( this->bits_per_sample == 16 ){
     if (stream_be != this->cpu_be)
       swab (sample_buffer, audio_buffer->mem, buf->size);
@@ -165,8 +185,28 @@ void lpcm_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
         n -= 10; 
       }
     }
-  }
-  else {
+  } else if( this->bits_per_sample == 24 ) {
+    uint8_t *s = (uint8_t *)sample_buffer;
+    uint8_t *d = (uint8_t *)audio_buffer->mem;
+    int n = buf->size;
+
+    while (n >= 0) {
+      *d++ = s[8];
+      *d++ = s[0];
+
+      *d++ = s[9];
+      *d++ = s[2];
+
+      *d++ = s[10];
+      *d++ = s[4];
+
+      *d++ = s[11];
+      *d++ = s[6];
+
+      s += 12;
+      n -= 12;
+    }
+  } else {
     memcpy (audio_buffer->mem, sample_buffer, buf->size);
   }
   
@@ -191,7 +231,7 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
 
   lpcm_decoder_t *this ;
 
-  this = (lpcm_decoder_t *) malloc (sizeof (lpcm_decoder_t));
+  this = (lpcm_decoder_t *) xine_xmalloc (sizeof (lpcm_decoder_t));
 
   this->audio_decoder.decode_data         = lpcm_decode_data;
   this->audio_decoder.reset               = lpcm_reset;
@@ -226,7 +266,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   lpcm_class_t *this ;
 
-  this = (lpcm_class_t *) malloc (sizeof (lpcm_class_t));
+  this = (lpcm_class_t *) xine_xmalloc (sizeof (lpcm_class_t));
 
   this->decoder_class.open_plugin     = open_plugin;
   this->decoder_class.get_identifier  = get_identifier;
@@ -240,13 +280,13 @@ static uint32_t audio_types[] = {
   BUF_AUDIO_LPCM_BE, BUF_AUDIO_LPCM_LE, 0
 };
 
-static decoder_info_t dec_info_audio = {
+static const decoder_info_t dec_info_audio = {
   audio_types,         /* supported types */
   1                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_AUDIO_DECODER, 12, "pcm", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
+  { PLUGIN_AUDIO_DECODER, 15, "pcm", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
