@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.56 2006/06/02 22:18:57 dsalt Exp $
+ * $Id: xine_decoder.c,v 1.57 2006/07/10 22:08:29 dgp85 Exp $
  *
  * stuff needed to turn libmad into a xine decoder plugin
  */
@@ -368,7 +368,7 @@ static const decoder_info_t dec_info_audio = {
   7                    /* priority        */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_AUDIO_DECODER, 15, "mad", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
