@@ -1,4 +1,4 @@
- /*
+/*
  * decode.c
  * Copyright (C) 2000-2002 Michel Lespinasse <walken@zoy.org>
  * Copyright (C) 1999-2000 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
@@ -29,6 +29,7 @@
 #include <string.h>	/* memcpy/memset, try to remove */
 #include <stdlib.h>
 #include <inttypes.h>
+#include <math.h>
 
 #define LOG_MODULE "decode"
 #define LOG_VERBOSE
@@ -41,7 +42,7 @@
 #include "mpeg2.h"
 #include "mpeg2_internal.h"
 #include "xineutils.h"
-#include "xvmc_vld.h"
+#include "libmpeg2_accel.h"
 
 /*
 #define LOG_PAN_SCAN
@@ -64,14 +65,15 @@ void mpeg2_init (mpeg2dec_t * mpeg2dec,
 	mpeg2_cpu_state_init (mm_accel);
 	mpeg2_idct_init (mm_accel);
 	mpeg2_mc_init (mm_accel);
+	libmpeg2_accel_scan(&mpeg2dec->accel, mpeg2_scan_norm, mpeg2_scan_alt);
     }
 
     if( !mpeg2dec->chunk_buffer )
       mpeg2dec->chunk_buffer = xine_xmalloc_aligned (16, BUFFER_SIZE + 4, 
-						     (void**)&mpeg2dec->chunk_base);
+						     &mpeg2dec->chunk_base);
     if( !mpeg2dec->picture )
       mpeg2dec->picture = xine_xmalloc_aligned (16, sizeof (picture_t),
-						(void**)&mpeg2dec->picture_base);
+						&mpeg2dec->picture_base);
 
     mpeg2dec->shift = 0xffffff00;
     mpeg2dec->new_sequence = 0;
@@ -103,40 +105,22 @@ void mpeg2_init (mpeg2dec_t * mpeg2dec,
 
 static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
 {
-  switch (mpeg2dec->picture->frame_rate_code) {
-  case 1: /* 23.976 fps */
-    frame->duration      = 3754;  /* actually it's 3753.75 */
-    break;
-  case 2: /* 24 fps */
-    frame->duration      = 3750;
-    break;
-  case 3: /* 25 fps */
-    frame->duration      = 3600;
-    break;
-  case 4: /* 29.97 fps */
-    frame->duration      = 3003;
-    break;
-  case 5: /* 30 fps */
-    frame->duration      = 3000;
-    break;
-  case 6: /* 50 fps */
-    frame->duration      = 1800;
-    break;
-  case 7: /* 59.94 fps */
-    frame->duration      = 1502;  /* actually it's 1501.5 */
-    break;
-  case 8: /* 60 fps */
-    frame->duration      = 1500;
-    break;
-  default:
-       /* printf ("invalid/unknown frame rate code : %d \n",
-               frame->frame_rate_code); */
-    frame->duration      = 0;
-  }
+  static const double durations[] = {
+    0,		/* invalid */
+    3753.75,	/* 23.976 fps */
+    3750,	/* 24 fps */
+    3600,	/* 25 fps */
+    3003,	/* 29.97 fps */
+    3000,	/* 30 fps */
+    1800,	/* 50 fps */
+    1501.5,	/* 59.94 fps */
+    1500,	/* 60 fps */
+  };
+  double duration = ((unsigned) mpeg2dec->picture->frame_rate_code > 8u)
+		    ? 0 : durations[mpeg2dec->picture->frame_rate_code];
   
-  frame->duration = frame->duration *
-    ((mpeg2dec->picture->frame_rate_ext_n + 1) /
-     (mpeg2dec->picture->frame_rate_ext_d + 1));
+  duration = duration * (mpeg2dec->picture->frame_rate_ext_n + 1.0) /
+			(mpeg2dec->picture->frame_rate_ext_d + 1.0);
  
   /* this should be used to detect any special rff pattern */
   mpeg2dec->rff_pattern = mpeg2dec->rff_pattern << 1;
@@ -146,7 +130,7 @@ static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
       (mpeg2dec->rff_pattern & 0xff) == 0x55) &&
       !mpeg2dec->picture->progressive_sequence ) {
     /* special case for ntsc 3:2 pulldown */
-    frame->duration += frame->duration/4;
+    duration *= 5 / 4;
   }
   else
   {  
@@ -155,14 +139,16 @@ static inline void get_frame_duration (mpeg2dec_t * mpeg2dec, vo_frame_t *frame)
            frame->progressive_frame ) {
         /* decoder should output 3 fields, so adjust duration to
            count on this extra field time */
-        frame->duration += frame->duration/2;     
+        duration *= 3 / 2;
       } else if( mpeg2dec->picture->progressive_sequence ) {
         /* for progressive sequences the output should repeat the
            frame 1 or 2 times depending on top_field_first flag. */
-        frame->duration *= (frame->top_field_first)?3:2;
+        duration *= (frame->top_field_first) ? 3 : 2;
       }
     }
   }
+
+  frame->duration = (int) ceil (duration);
   _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_FRAME_DURATION, frame->duration);
   /*printf("mpeg2dec: rff=%u\n",frame->repeat_first_field);*/
 } 
@@ -175,6 +161,10 @@ static double get_aspect_ratio(mpeg2dec_t *mpeg2dec)
     1.0, 0.6735, 0.7031, 0.7615, 0.8055, 0.8437, 0.8935, 0.9157,
     0.9815, 1.0255, 1.0695, 1.0950, 1.1575, 1.2015, 1.0 /*reserved*/ };
 
+  /* TODO: For slower machines the value of this function should be computed
+   *       once and cached!
+   */
+
   if( !picture->mpeg1 ) {
     /* these hardcoded values are defined on mpeg2 standard for
      * aspect ratio. other values are reserved or forbidden.  */
@@ -245,12 +235,12 @@ static void remember_metainfo (mpeg2dec_t *mpeg2dec) {
   _x_meta_info_set_utf8(mpeg2dec->stream, XINE_META_INFO_VIDEOCODEC, "MPEG (libmpeg2)");
 }
 
-
 static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 			       uint8_t * buffer)
 {
     picture_t * picture;
     int is_frame_done;
+    double ratio;
     
     /* wait for sequence_header_code */
     if (mpeg2dec->is_sequence_needed) {
@@ -277,37 +267,8 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     
     if (is_frame_done && picture->current_frame != NULL) {
 
-      /*
-       * This frame completion code will move to a separate libmpeg2_accel.c file?
-       * int libmpeg2_accel_frame_completion(mpeg2dec_t *, picture_t *, int);
-       */
-
-      if (mpeg2dec->frame_format == XINE_IMGFMT_XXMC) {
-	xine_xxmc_t *xxmc = (xine_xxmc_t *) 
-	  picture->current_frame->accel_data;
-	switch(picture->current_frame->format) {
-	case XINE_IMGFMT_XXMC:
-	  switch(xxmc->acceleration) {
-	  case XINE_XVMC_ACCEL_VLD:
-	    mpeg2_xxmc_vld_frame_complete(mpeg2dec, picture, code);
-	    break;
-	  case XINE_XVMC_ACCEL_IDCT:
-	  case XINE_XVMC_ACCEL_MOCOMP:
-	    xxmc->decoded = !picture->current_frame->bad_frame;
-	    xxmc->proc_xxmc_flush( picture->current_frame );
-	    break;
-	  default:
-	    break;
-	  }
-	default:
-	  break;
-	}
-      }
-
-      /*
-       * End of frame completion code.
-       */
-
+	libmpeg2_accel_frame_completion(&mpeg2dec->accel, mpeg2dec->frame_format, 
+					picture, code);
 
 	if (((picture->picture_structure == FRAME_PICTURE) ||
 	     (picture->second_field)) ) {
@@ -439,6 +400,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	}
 
 	if (mpeg2dec->is_sequence_needed 
+	    || (picture->aspect_ratio_information != picture->saved_aspect_ratio)
 	    || (picture->frame_width != picture->coded_picture_width)
 	    || (picture->frame_height != picture->coded_picture_height)) {
 	    xine_event_t event;
@@ -475,6 +437,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 
 	    picture->frame_width = picture->coded_picture_width;
 	    picture->frame_height = picture->coded_picture_height;
+	    picture->saved_aspect_ratio = picture->aspect_ratio_information;
 	}
 	break;
 
@@ -495,9 +458,9 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	  abort();
 	}
     default:
-	if (code >= 0xb9)
-	  printf ("libmpeg2: stream not demultiplexed ?\n");
-
+        if ((code >= 0xb9) && (code != 0xe4)) {
+	  printf("Not multiplexed? 0x%x\n",code);
+	}
 	if (code >= 0xb0)
 	    break;
 
@@ -512,6 +475,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 		mpeg2dec->drop_frame = 1;		
 	    } else {
 		int flags = picture->picture_structure;
+
 		if (!picture->mpeg1) flags |= VO_INTERLACED_FLAG;
 		if (mpeg2dec->force_pan_scan) flags |= VO_PAN_SCAN_FLAG;
 		if (mpeg2dec->new_sequence) flags |= VO_NEW_SEQUENCE_FLAG;
@@ -522,39 +486,29 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 			picture->current_frame->free (picture->current_frame);
 		}
 		if (picture->picture_coding_type == B_TYPE) {
+                    ratio = get_aspect_ratio(mpeg2dec);
 		    picture->current_frame =
 		        mpeg2dec->stream->video_out->get_frame (mpeg2dec->stream->video_out,
 						     picture->coded_picture_width,
 						     picture->coded_picture_height,
-						     get_aspect_ratio(mpeg2dec),
+						     ratio,
 						     mpeg2dec->frame_format,
 						     flags);
-		    /*
-		     * Move to libmpeg2_accel.c
-		     * int libmpeg2_accel_new_frame(mpeg2dec_t *, picture_t *)
-		     */
-		    mpeg2_xxmc_choose_coding(mpeg2dec, picture, get_aspect_ratio(mpeg2dec),
-					     flags);
-		    /*
-		     * End of new frame accel code.
-		     */
+		    libmpeg2_accel_new_frame( &mpeg2dec->accel, mpeg2dec->frame_format,
+					      picture, ratio, flags);
 		} else {
+                    ratio = get_aspect_ratio(mpeg2dec);
 		    picture->current_frame =
 		        mpeg2dec->stream->video_out->get_frame (mpeg2dec->stream->video_out,
 						     picture->coded_picture_width,
 						     picture->coded_picture_height,
-						     get_aspect_ratio(mpeg2dec),
+						     ratio,
 						     mpeg2dec->frame_format,
 						     flags);
-		    /*
-		     * Move to libmpeg2_accel.c
-		     * int libmpeg2_accel_new_frame(mpeg2dec_t *, picture_t *)
-		     */
-		    mpeg2_xxmc_choose_coding(mpeg2dec, picture, 
-					     get_aspect_ratio(mpeg2dec), flags);
-		    /*
-		     * End of new frame accel code.
-		     */
+
+		    libmpeg2_accel_new_frame( &mpeg2dec->accel, mpeg2dec->frame_format,
+					      picture, ratio, flags);
+
 		    if (picture->forward_reference_frame &&
 		        picture->forward_reference_frame != picture->backward_reference_frame)
 		      picture->forward_reference_frame->free (picture->forward_reference_frame);
@@ -564,28 +518,10 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 		    picture->backward_reference_frame = picture->current_frame;
 		}
 
-		/*
-		 * Move to libmpeg2_accel.c
-		 * int libmpeg2_accel_new_sequence(mpeg2dec_t *, picture_t *)
-		 */
-
-		if(mpeg2dec->new_sequence) {
-		  switch(mpeg2dec->frame_format) {
-		    case XINE_IMGFMT_XXMC:
-		    case XINE_IMGFMT_XVMC: {
-		      xine_xvmc_t *xvmc = (xine_xvmc_t *) 
-		        picture->current_frame->accel_data;
-		      picture->mc = xvmc->macroblocks;
-		      mpeg2dec->new_sequence = 0;
-		      break;
-		    }
-		    default:
-		      break;
-		  }
-		}
-		/*
-		 * End of new sequence accel code.
-		 */
+		if(mpeg2dec->new_sequence) 
+		    mpeg2dec->new_sequence = 
+			libmpeg2_accel_new_sequence(&mpeg2dec->accel, mpeg2dec->frame_format, 
+						    picture);
 
 		picture->current_frame->bad_frame          = 1;
 		picture->current_frame->drawn              = 0;
@@ -624,49 +560,11 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	  printf("slice target %08x past %08x future %08x\n",picture->current_frame,picture->forward_reference_frame,picture->backward_reference_frame);
 	  fflush(stdout);
 #endif
-	  /*
-	   * The below accelerated slice function choice will move to libmpeg2_accel.c ?
-	   * int libmpeg2_accel_slice(mpeg2dec_t *, picture_t *, int , char *)
-	   */
+	  libmpeg2_accel_slice(&mpeg2dec->accel, picture, code, buffer, mpeg2dec->chunk_size, 
+			       mpeg2dec->chunk_buffer);
 
-	  switch( mpeg2dec->frame_format ) {
-	    case XINE_IMGFMT_XXMC:
-	    {
-	      xine_xxmc_t *xxmc = (xine_xxmc_t *) 
-		picture->current_frame->accel_data;
-	      switch(picture->current_frame->format) {
-	      case XINE_IMGFMT_XXMC:
-		switch(xxmc->acceleration) {
-		case XINE_XVMC_ACCEL_VLD:
-		  mpeg2_xxmc_slice(mpeg2dec, picture, code, buffer);
-		  break;
-		case XINE_XVMC_ACCEL_IDCT:
-		case XINE_XVMC_ACCEL_MOCOMP:
-		  mpeg2_xvmc_slice (mpeg2dec, picture, code, buffer);
-		  break;
-		default:
-		  mpeg2_slice (picture, code, buffer);
-		  break;
-		}
-		break;
-	      default:
-		mpeg2_slice (picture, code, buffer);
-		break;
-	      }
-	      break;
-	    }
-	    case XINE_IMGFMT_XVMC:
-	      mpeg2_xvmc_slice (mpeg2dec, picture, code, buffer);
-	      break;
-	    default:
-	      mpeg2_slice (picture, code, buffer);
-	      break;
-	  }
-	  /*
-	   * End of acceleration code.
-	   */
-
-	  if( picture->v_offset > picture->limit_y ) { 
+	  if( picture->v_offset > picture->limit_y || 
+	      picture->v_offset + 16 > picture->display_height ) { 
 	    picture->current_frame->bad_frame = 0;
 	  }
 	}
@@ -738,11 +636,14 @@ int mpeg2_decode_data (mpeg2dec_t * mpeg2dec, uint8_t * current, uint8_t * end,
     while (current != end) {
 	code = mpeg2dec->code;
 	current = copy_chunk (mpeg2dec, current, end);
-	if (current == NULL)
-	    return ret;
+	if (current == NULL) 
+	    break;
 	ret += parse_chunk (mpeg2dec, code, mpeg2dec->chunk_buffer);
     }
 
+    libmpeg2_accel_frame_completion(&mpeg2dec->accel, mpeg2dec->frame_format, 
+				    mpeg2dec->picture, 0xff);
+
     return ret;
 }
 
@@ -752,6 +653,7 @@ void mpeg2_discontinuity (mpeg2dec_t * mpeg2dec) {
   if( !picture )
     return;
   
+  mpeg2dec->in_slice = 0;
   mpeg2dec->pts = 0;  
   if ( picture->current_frame )
     picture->current_frame->pts = 0;
@@ -759,15 +661,8 @@ void mpeg2_discontinuity (mpeg2dec_t * mpeg2dec) {
     picture->forward_reference_frame->pts = 0;
   if ( picture->backward_reference_frame )
     picture->backward_reference_frame->pts = 0;
-  
-  /*
-   * Move to libmpeg2_accel.c
-   * int libmpeg2_accel_discontinuity(mpeg2dec_t *);
-   */
-  mpeg2dec->xvmc_last_slice_code=-1;
-  /*
-   * End of discontinuity accel code.
-   */
+
+  libmpeg2_accel_discontinuity(&mpeg2dec->accel, mpeg2dec->frame_format, picture);
 }
 
 void mpeg2_reset (mpeg2dec_t * mpeg2dec) {
@@ -1021,4 +916,11 @@ static void process_userdata(mpeg2dec_t *mpeg2dec, uint8_t *buffer)
       mpeg2dec->cc_dec->decode_data(mpeg2dec->cc_dec, &buf);
     }
   }
+  /* check Active Format Description ETSI TS 101 154 V1.5.1 */
+  else if (buffer[0] == 0x44 && buffer[1] == 0x54 && buffer[2] == 0x47 && buffer[3] == 0x31)
+  {
+    int afd = (buffer[4] & 0x40) ? (buffer[5] & 0x0f) : -1;
+    _x_stream_info_set(mpeg2dec->stream, XINE_STREAM_INFO_VIDEO_AFD, afd);
+    
+  }
 }
