@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.31 2004/12/12 22:01:10 mroi Exp $
+ * $Id: xine_decoder.c,v 1.34 2006/07/10 22:08:30 dgp85 Exp $
  *
  * closed caption spu decoder. receive data by events. 
  *
@@ -357,12 +357,12 @@ static void *init_spu_decoder_plugin (xine_t *xine, void *data) {
 /* plugin catalog information */
 static uint32_t supported_types[] = { BUF_SPU_CC, 0 };
 
-static decoder_info_t spudec_info = {
+static const decoder_info_t spudec_info = {
   supported_types,     /* supported types */
   1                    /* priority        */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_SPU_DECODER, 16, "spucc", XINE_VERSION_CODE, &spudec_info, &init_spu_decoder_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
