@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2004 the xine project
  *
  * Copyright (C) James Courtier-Dutton James@superbug.demon.co.uk - July 2001
  *
@@ -19,7 +19,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: spu.h,v 1.19 2002/12/26 21:53:42 miguelfreitas Exp $
+ * $Id: spu.h,v 1.27 2004/07/14 01:18:48 miguelfreitas Exp $
  *
  * This file was originally part of the OMS program.
  *
@@ -35,7 +35,11 @@
 #include <inttypes.h>
 #include "video_out.h"
 #include "video_overlay.h"
-#include "nav_types.h"
+#ifdef HAVE_DVDNAV
+#  include <dvdnav/nav_types.h>
+#else
+#  include "nav_types.h"
+#endif
 
 #define NUM_SEQ_BUFFERS 50
 #define MAX_STREAMS 32
@@ -69,7 +73,7 @@ typedef struct {
 typedef struct {
   uint8_t *cmd_ptr;
 
-  int32_t field_offs[2];
+  uint32_t field_offs[2];
   int32_t b_top,    o_top;
   int32_t b_bottom, o_bottom;
   int32_t b_left,   o_left;
@@ -81,13 +85,13 @@ typedef struct {
   int32_t delay;        /* Delay in 90Khz / 1000 */
   int32_t need_clut;    /* doesn't have the right clut yet */
   int32_t cur_colors[4];/* current 4 colors been used */
+  int32_t vobsub;       /* vobsub must be aligned to bottom */
 
   uint32_t clut[16];
 } spudec_state_t;
 
 typedef struct spudec_stream_state_s {
   spudec_seq_t        ra_seq;
-  uint32_t         stream_filter;
   spudec_state_t      state;
   int64_t          vpts;
   int64_t          pts;
@@ -98,6 +102,13 @@ typedef struct {
   spu_decoder_class_t   decoder_class;
 } spudec_class_t;
 
+typedef struct pci_node_s pci_node_t;
+struct pci_node_s {
+  pci_t         pci;
+  uint64_t      vpts;
+  pci_node_t   *next;
+};
+
 typedef struct spudec_decoder_s {
   spu_decoder_t    spu_decoder;
 
@@ -114,17 +125,21 @@ typedef struct spudec_decoder_s {
   vo_overlay_t     overlay;
   int              ovl_caps;
   int              output_open;
-  pci_t            pci;
+  pthread_mutex_t  nav_pci_lock;
+  pci_node_t       pci_cur;
   uint32_t         buttonN;  /* Current button number for highlights */
-  int32_t button_filter; /* Allow highlight changes or not */
-  pthread_mutex_t nav_pci_lock;
+  int32_t          button_filter; /* Allow highlight changes or not */
   int64_t          last_event_vpts;
 } spudec_decoder_t;
 
-void spudec_reassembly (spudec_seq_t *seq, uint8_t *pkt_data, u_int pkt_len);
-void spudec_process( spudec_decoder_t *this, uint32_t stream_id);
+void spudec_reassembly (xine_t *xine, spudec_seq_t *seq, uint8_t *pkt_data, u_int pkt_len);
+void spudec_process( spudec_decoder_t *this, int stream_id);
+/* the nav functions must be called with the nav_pci_lock held */
 void spudec_decode_nav( spudec_decoder_t *this, buf_element_t *buf);
-void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button, int32_t mode,
+void spudec_clear_nav_list(spudec_decoder_t *this);
+void spudec_update_nav(spudec_decoder_t *this);
+void spudec_process_nav(spudec_decoder_t *this);
+int  spudec_copy_nav_to_overlay(xine_t *xine, pci_t* nav_pci, uint32_t* clut, int32_t button, int32_t mode,
                                 vo_overlay_t * overlay, vo_overlay_t * base );
 
 #endif
