@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_sputext.c,v 1.43 2004/12/17 13:38:24 mroi Exp $
+ * $Id: demux_sputext.c,v 1.49 2006/07/10 22:08:30 dgp85 Exp $
  *
  * code based on old libsputext/xine_decoder.c
  *
@@ -358,7 +358,8 @@ static subtitle_t *sub_read_line_subrip(demux_sputext_t *this,subtitle_t *curren
   do {
     if(!read_line_from_input(this,line,LINE_LEN))
       return NULL;
-  } while(sscanf(line,"%d:%d:%d,%d --> %d:%d:%d,%d",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4) < 8);
+    i = sscanf(line,"%d:%d:%d%*[,.]%d --> %d:%d:%d%*[,.]%d",&a1,&a2,&a3,&a4,&b1,&b2,&b3,&b4);
+  } while(i < 8);
   current->start = a1*360000+a2*6000+a3*100+a4/10;
   current->end   = b1*360000+b2*6000+b3*100+b4/10;
   i=0;
@@ -1051,7 +1052,7 @@ static int sub_autodetect (demux_sputext_t *this) {
       return FORMAT_MICRODVD;
     }
 
-    if (sscanf (line, "%d:%d:%d,%d --> %d:%d:%d,%d", &i, &i, &i, &i, &i, &i, &i, &i)==8) {
+    if (sscanf (line, "%d:%d:%d%*[,.]%d --> %d:%d:%d%*[,.]%d", &i, &i, &i, &i, &i, &i, &i, &i)==8) {
       this->uses_time=1;
       xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "subrip subtitle format detected\n");
       return FORMAT_SUBRIP;
@@ -1240,7 +1241,7 @@ static subtitle_t *sub_read_file (demux_sputext_t *this) {
     else
       strcat(buffer, "\n");
     
-    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, buffer);
+    xprintf (this->stream->xine, XINE_VERBOSITY_DEBUG, "%s", buffer);
   }
   
   return first;
@@ -1512,8 +1513,8 @@ static void *init_sputext_demux_class (xine_t *xine, void *data) {
   return this;
 }
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 25, "sputext", XINE_VERSION_CODE, NULL, &init_sputext_demux_class },
+  { PLUGIN_DEMUX, 26, "sputext", XINE_VERSION_CODE, NULL, &init_sputext_demux_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
