@@ -28,7 +28,7 @@
  *   8 (long and short)
  * - untested (found no testfiles) IFF-ANIM OPT 3, 4 and 6
  *
- * $Id: bitplane.c,v 1.14 2006/06/02 22:18:58 dsalt Exp $
+ * $Id: bitplane.c,v 1.15 2006/07/10 22:08:43 dgp85 Exp $
  */
 
 #include <stdio.h>
@@ -1585,7 +1585,7 @@ static const decoder_info_t dec_info_video = {
   1                    /* priority        */
 };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_DECODER, 18, "bitplane", XINE_VERSION_CODE, &dec_info_video, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
