@@ -27,6 +27,10 @@
 #ifndef _MMX_H
 #define _MMX_H
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "goom_graphic.h"
 
 /*	Warning:  at this writing, the version of GAS packaged
@@ -69,11 +73,15 @@ mm_support(void)
 		 13 if AMD Extended MMX, &3dNow supported
 	   0 if hardware does not support any of these
 	*/
+#ifdef ARCH_X86_64
+	return 13;
+#else
 	register int rval = 0;
 
 	__asm__ __volatile__ (
 		/* See if CPUID instruction is supported ... */
 		/* ... Get copies of EFLAGS into eax and ecx */
+    "pushl %%ebx\n\t"
 		"pushf\n\t"
 		"popl %%eax\n\t"
 		"movl %%eax, %%ecx\n\t"
@@ -214,13 +222,15 @@ mm_support(void)
 		"movl $0, %0\n\n\t"
 
 		"Return:\n\t"
+    "popl %%ebx\n\t"
 		: "=X" (rval)
 		: /* no input */
-		: "eax", "ebx", "ecx", "edx"
+		: "eax", "ecx", "edx"
 	);
 
 	/* Return */
 	return(rval);
+#endif
 }
 
 /*	Function to test if mmx instructions are supported...
