@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: switch.c,v 1.17 2006/06/02 22:18:58 dsalt Exp $
+ * $Id: switch.c,v 1.18 2006/07/10 22:08:44 dgp85 Exp $
  */
  
 /*
@@ -41,7 +41,7 @@ static void *switch_init_plugin(xine_t *xine, void *);
 /* plugin catalog information */
 static const post_info_t switch_special_info = { XINE_POST_TYPE_VIDEO_COMPOSE };
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
   { PLUGIN_POST, 9, "switch", XINE_VERSION_CODE, &switch_special_info, &switch_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
