@@ -1,7 +1,7 @@
  /*
  * Copyright (C) 2001 the xine project
  *
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  *
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@
 static void deinterlace_bob_yuv_mmx( uint8_t *pdst, uint8_t *psrc[],
     int width, int height )
 {
-#ifdef ARCH_X86
+#if defined(ARCH_X86) || defined(ARCH_X86_64)
   int Line;
   uint64_t *YVal1;
   uint64_t *YVal2;
@@ -190,7 +190,7 @@ static void deinterlace_bob_yuv_mmx( uint8_t *pdst, uint8_t *psrc[],
 static int deinterlace_weave_yuv_mmx( uint8_t *pdst, uint8_t *psrc[],
     int width, int height )
 {
-#ifdef ARCH_X86
+#if defined(ARCH_X86) || defined(ARCH_X86_64)
 
   int Line;
   uint64_t *YVal1;
@@ -389,7 +389,7 @@ static int deinterlace_weave_yuv_mmx( uint8_t *pdst, uint8_t *psrc[],
 static int deinterlace_greedy_yuv_mmx( uint8_t *pdst, uint8_t *psrc[],
     int width, int height )
 {
-#ifdef ARCH_X86
+#if defined(ARCH_X86) || defined(ARCH_X86_64)
   int Line;
   int	LoopCtr;
   uint64_t *L1;					// ptr to Line1, of 3
@@ -552,7 +552,7 @@ static int deinterlace_greedy_yuv_mmx( uint8_t *pdst, uint8_t *psrc[],
 static void deinterlace_onefield_yuv_mmx( uint8_t *pdst, uint8_t *psrc[],
     int width, int height )
 {
-#ifdef ARCH_X86
+#if defined(ARCH_X86) || defined(ARCH_X86_64)
   int Line;
   uint64_t *YVal1;
   uint64_t *YVal3;
@@ -633,7 +633,7 @@ static void deinterlace_onefield_yuv_mmx( uint8_t *pdst, uint8_t *psrc[],
 static void deinterlace_linearblend_yuv_mmx( uint8_t *pdst, uint8_t *psrc[],
     int width, int height )
 {
-#ifdef ARCH_X86
+#if defined(ARCH_X86) || defined(ARCH_X86_64)
   int Line;
   uint64_t *YVal1;
   uint64_t *YVal2;
@@ -753,7 +753,7 @@ static void deinterlace_linearblend_yuv( uint8_t *pdst, uint8_t *psrc[],
 
 static int check_for_mmx(void)
 {
-#ifdef ARCH_X86
+#if defined(ARCH_X86) || defined(ARCH_X86_64)
 static int config_flags = -1;
 
   if ( config_flags == -1 )
@@ -766,13 +766,6 @@ static int config_flags = -1;
 #endif
 }
 
-static void abort_mmx_missing(void)
-{
-  printf("deinterlace: Fatal error, MMX instruction set needed!\n");
-  /* FIXME: is it possible to call some "nicer" xine exit function? */
-  abort();
-}
-
 /* generic YUV deinterlacer
    pdst -> pointer to destination bitmap
    psrc -> array of pointers to source bitmaps ([0] = most recent)
@@ -791,7 +784,7 @@ void deinterlace_yuv( uint8_t *pdst, uint8_t *psrc[],
       if( check_for_mmx() )
         deinterlace_bob_yuv_mmx(pdst,psrc,width,height);
       else /* FIXME: provide an alternative? */
-        abort_mmx_missing();
+        xine_fast_memcpy(pdst,psrc[0],width*height);
       break;
     case DEINTERLACE_WEAVE:
       if( check_for_mmx() )
@@ -800,7 +793,7 @@ void deinterlace_yuv( uint8_t *pdst, uint8_t *psrc[],
           xine_fast_memcpy(pdst,psrc[0],width*height);
       }
       else /* FIXME: provide an alternative? */
-        abort_mmx_missing();
+        xine_fast_memcpy(pdst,psrc[0],width*height);
       break;
     case DEINTERLACE_GREEDY:
       if( check_for_mmx() )
@@ -809,16 +802,16 @@ void deinterlace_yuv( uint8_t *pdst, uint8_t *psrc[],
           xine_fast_memcpy(pdst,psrc[0],width*height);
       }
       else /* FIXME: provide an alternative? */
-        abort_mmx_missing();
+        xine_fast_memcpy(pdst,psrc[0],width*height);
       break;
     case DEINTERLACE_ONEFIELD:
       if( check_for_mmx() )
         deinterlace_onefield_yuv_mmx(pdst,psrc,width,height);
       else /* FIXME: provide an alternative? */
-        abort_mmx_missing();
+        xine_fast_memcpy(pdst,psrc[0],width*height);
       break;
     case DEINTERLACE_ONEFIELDXV:
-      printf("deinterlace: ONEFIELDXV must be handled by the video driver.\n");
+      lprintf("ONEFIELDXV must be handled by the video driver.\n");
       break;
     case DEINTERLACE_LINEARBLEND:
       if( check_for_mmx() )
@@ -827,11 +820,31 @@ void deinterlace_yuv( uint8_t *pdst, uint8_t *psrc[],
         deinterlace_linearblend_yuv(pdst,psrc,width,height);
       break;
     default:
-      printf("deinterlace: unknow method %d.\n",method);
+      lprintf("unknow method %d.\n",method);
       break;
   }
 }
 
+int deinterlace_yuv_supported ( int method )
+{
+  switch( method ) {
+    case DEINTERLACE_NONE:
+      return 1;
+    case DEINTERLACE_BOB:
+    case DEINTERLACE_WEAVE:
+    case DEINTERLACE_GREEDY:
+    case DEINTERLACE_ONEFIELD:
+      return check_for_mmx();
+    case DEINTERLACE_ONEFIELDXV:
+      lprintf ("ONEFIELDXV must be handled by the video driver.\n");
+      return 0;
+    case DEINTERLACE_LINEARBLEND:
+      return 1;
+  }
+
+  return 0;
+}
+
 char *deinterlace_methods[] = {
   "none", 
   "bob",
