@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2005 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -20,7 +20,7 @@
  * video_out_directx.c, direct draw video output plugin for xine
  * by Matthew Grooms <elon@altavista.com>
  *
- * $Id: video_out_directx.c,v 1.21 2004/12/01 07:23:55 athp Exp $
+ * $Id: video_out_directx.c,v 1.31 2006/07/10 22:08:44 dgp85 Exp $
  */
 
 typedef unsigned char boolean;
@@ -28,9 +28,7 @@ typedef unsigned char boolean;
 #include <windows.h>
 #include <ddraw.h>
 
-#include "xine.h"
-#include "video_out.h"
-#include "alphablend.h"
+#include <pthread.h>
 
 #define LOG_MODULE "video_out_directx"
 #define LOG_VERBOSE
@@ -38,6 +36,10 @@ typedef unsigned char boolean;
 #define LOG
 */
 
+#include "xine.h"
+#include "video_out.h"
+#include "xine_internal.h"
+
 #include "xine_internal.h"
 #include "yuv2rgb.h"
 
@@ -54,7 +56,7 @@ typedef unsigned char boolean;
  * Defining them here allows us to get rid of the dxguid library during
  * the linking stage.
  *****************************************************************************/
-#if 0
+#if 1
 static const GUID IID_IDirectDraw = {
 	0x6C14DB80,0xA733,0x11CE,{0xA5,0x21,0x00,0x20,0xAF,0x0B,0xE5,0x60}
 };
@@ -66,7 +68,7 @@ static const GUID IID_IDirectDraw2 = {
 };
 #endif
 
-#if 1
+#if 0
 static const GUID IID_IDirectDraw4 = {
 	0x9C59509A,0x39BD,0x11D1,{0x8C,0x4A,0x00,0xC0,0x4F,0xD9,0x30,0xC5}
 };
@@ -100,9 +102,9 @@ typedef struct {
 
   xine_t                  *xine;
 
-  LPDIRECTDRAW4		   ddobj;	    /* direct draw object */
-  LPDIRECTDRAWSURFACE4	   primary;	    /* primary dd surface */
-  LPDIRECTDRAWSURFACE4 	   secondary;	    /* secondary dd surface  */
+  LPDIRECTDRAW		   ddobj;	    /* direct draw object */
+  LPDIRECTDRAWSURFACE	   primary;	    /* primary dd surface */
+  LPDIRECTDRAWSURFACE 	   secondary;	    /* secondary dd surface  */
   LPDIRECTDRAWCLIPPER	   ddclipper;	    /* dd clipper object */
   uint8_t *		   contents;	    /* secondary contents */
   win32_frame_t           *current;         /* current frame */
@@ -137,7 +139,7 @@ typedef struct {
 /* Display formatted error message in 
  * popup message box.*/
 
-void Error( HWND hwnd, LPSTR szfmt, ... )
+static void Error( HWND hwnd, LPSTR szfmt, ... )
 {
   char tempbuff[ 256 ];
   *tempbuff = 0;
@@ -148,7 +150,7 @@ void Error( HWND hwnd, LPSTR szfmt, ... )
 /* Update our drivers current knowledge
  * of our windows video out posistion */
 
-void UpdateRect( win32_visual_t * win32_visual )
+static void UpdateRect( win32_visual_t * win32_visual )
 {
   if( win32_visual->FullScreen )
     {
@@ -172,10 +174,10 @@ void UpdateRect( win32_visual_t * win32_visual )
  * window and will always have a pixel format 
  * identical to the current display mode. */
 
-boolean CreatePrimary( win32_driver_t * win32_driver )
+static boolean CreatePrimary( win32_driver_t * win32_driver )
 {
   LPDIRECTDRAW			ddobj;
-  DDSURFACEDESC2			ddsd;
+  DDSURFACEDESC			ddsd;
   HRESULT					result;
 
   /* create direct draw object */
@@ -193,16 +195,16 @@ boolean CreatePrimary( win32_driver_t * win32_driver )
   result = IDirectDraw_SetCooperativeLevel( ddobj, win32_driver->win32_visual->WndHnd, DDSCL_NORMAL );
   if( result != DD_OK )
     {
-      Error( 0, "SetCooperativeLevel : error %i", result );
+      Error( 0, "SetCooperativeLevel : error 0x%lx", result );
       return 0;
     }
 
   /* try to get new interface */
 
-  result = IDirectDraw_QueryInterface( ddobj, &IID_IDirectDraw4, (LPVOID *) &win32_driver->ddobj );
+  result = IDirectDraw_QueryInterface( ddobj, &IID_IDirectDraw, (LPVOID *) &win32_driver->ddobj );
   if( result != DD_OK )
     {
-      Error( 0, "ddobj->QueryInterface : DirectX 4 or higher required" );
+      Error( 0, "ddobj->QueryInterface : DirectX required" );
       return 0;
     }
 
@@ -217,19 +219,19 @@ boolean CreatePrimary( win32_driver_t * win32_driver )
   ddsd.dwFlags        = DDSD_CAPS;
   ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
 
-  result = IDirectDraw4_CreateSurface( win32_driver->ddobj, &ddsd, &win32_driver->primary, 0 );
+  result = IDirectDraw_CreateSurface( win32_driver->ddobj, &ddsd, &win32_driver->primary, 0 );
   if( result != DD_OK )
     {
-      Error( 0, "CreateSurface ( primary ) : error %i ", result );
+      Error( 0, "CreateSurface ( primary ) : error 0x%lx", result );
       return 0;
     }
 
   /* create our clipper object */
 
-  result = IDirectDraw4_CreateClipper( win32_driver->ddobj, 0, &win32_driver->ddclipper, 0 );
+  result = IDirectDraw_CreateClipper( win32_driver->ddobj, 0, &win32_driver->ddclipper, 0 );
   if( result != DD_OK )
     {
-      Error( 0, "CreateClipper : error %i", result );
+      Error( 0, "CreateClipper : error 0x%lx", result );
       return 0;
     }
 
@@ -238,16 +240,16 @@ boolean CreatePrimary( win32_driver_t * win32_driver )
   result = IDirectDrawClipper_SetHWnd( win32_driver->ddclipper, 0, win32_driver->win32_visual->WndHnd );
   if( result != DD_OK )
     {
-      Error( 0, "ddclipper->SetHWnd : error %i", result );
+      Error( 0, "ddclipper->SetHWnd : error 0x%lx", result );
       return 0;
     }
 
   /* associate our primary surface with our clipper */
 
-  result = IDirectDrawSurface4_SetClipper( win32_driver->primary, win32_driver->ddclipper );
+  result = IDirectDrawSurface_SetClipper( win32_driver->primary, win32_driver->ddclipper );
   if( result != DD_OK )
     {
-      Error( 0, "ddclipper->SetHWnd : error %i", result );
+      Error( 0, "ddclipper->SetHWnd : error 0x%lx", result );
       return 0;
     }
 
@@ -268,9 +270,10 @@ boolean CreatePrimary( win32_driver_t * win32_driver )
  * as the primary surface. At least then we
  * can use h/w scaling if supported. */
 
-boolean CreateSecondary( win32_driver_t * win32_driver, int width, int height, int format )
+static boolean CreateSecondary( win32_driver_t * win32_driver, int width, int height, int format )
 {
-  DDSURFACEDESC2			ddsd;
+  DDSURFACEDESC ddsd;
+  HRESULT result;
 
   if( format == XINE_IMGFMT_YV12 )
     xprintf(win32_driver->xine, XINE_VERBOSITY_DEBUG, "vo_out_directx : switching to YV12 overlay type\n" );
@@ -297,7 +300,7 @@ boolean CreateSecondary( win32_driver_t * win32_driver, int width, int height, i
    * surface then release it */
 
   if( win32_driver->secondary )
-    IDirectDrawSurface4_Release( win32_driver->secondary );
+    IDirectDrawSurface_Release( win32_driver->secondary );
 
   memset( &ddsd, 0, sizeof( ddsd ) );
   ddsd.dwSize         = sizeof( ddsd );
@@ -356,47 +359,49 @@ boolean CreateSecondary( win32_driver_t * win32_driver, int width, int height, i
     } 
 #endif /* RGB_SUPPORT */
 
-  lprintf("CreateSecondary() - IDirectDraw4_CreateSurface()\n");
-
-  if( IDirectDraw4_CreateSurface( win32_driver->ddobj, &ddsd, &win32_driver->secondary, 0 ) == DD_OK )
+  lprintf("CreateSecondary() - IDirectDraw_CreateSurface()\n");
+  if( IDirectDraw_CreateSurface( win32_driver->ddobj, &ddsd, &win32_driver->secondary, 0 ) == DD_OK )
     return TRUE;
 
   /*  Our fallback method is to create a back buffer
    *  with the same image format as the primary surface */
 
   lprintf("CreateSecondary() - Falling back to back buffer same as primary\n");
+  lprintf("CreateSecondary() - act_format = (NATIVE) %d\n", IMGFMT_NATIVE);
 
   ddsd.dwFlags        = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
   ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
-
-  lprintf("CreateSecondary() - act_format = (NATIVE) %d\n", IMGFMT_NATIVE);
-
   win32_driver->act_format = IMGFMT_NATIVE;
 
-  if( IDirectDraw4_CreateSurface( win32_driver->ddobj, &ddsd, &win32_driver->secondary, 0 ) == DD_OK )
+  if( IDirectDraw_CreateSurface( win32_driver->ddobj, &ddsd, &win32_driver->secondary, 0 ) == DD_OK )
+    return TRUE;
+
+  /*  Our second fallback - all w/o HW acceleration */
+  lprintf("CreateSecondary() - Falling back, disabling HW acceleration \n");
+  ddsd.dwFlags        = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
+  ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
+  if( (result = IDirectDraw_CreateSurface( win32_driver->ddobj, &ddsd, &win32_driver->secondary, 0 )) == DD_OK )
     return TRUE;
 
   /* This is bad. We cant even create a surface with
    * the same format as the primary surface. */
-
-  Error( 0, "CreateSurface ( Secondary ) : unable to create a suitable rendering surface" );
-
+  Error( 0, "CreateSurface ( Secondary ) : unable to create a suitable rendering surface: 0x%08lX", result );
   return FALSE;
 }
 
 /* Destroy all direct draw driver allocated
  * resources. */
 
-void Destroy( win32_driver_t * win32_driver )
+static void Destroy( win32_driver_t * win32_driver )
 {
   if( win32_driver->ddclipper )
     IDirectDrawClipper_Release( win32_driver->ddclipper );
 
   if( win32_driver->primary )
-    IDirectDrawSurface4_Release( win32_driver->primary );
+    IDirectDrawSurface_Release( win32_driver->primary );
 
   if( win32_driver->secondary )
-    IDirectDrawSurface4_Release( win32_driver->secondary );
+    IDirectDrawSurface_Release( win32_driver->secondary );
 
   if( win32_driver->ddobj )
     IDirectDraw_Release( win32_driver->ddobj );
@@ -411,7 +416,7 @@ void Destroy( win32_driver_t * win32_driver )
  * the h/w does not support an overlay for
  * the native frame format. */
 
-boolean CheckPixelFormat( win32_driver_t * win32_driver )
+static boolean CheckPixelFormat( win32_driver_t * win32_driver )
 {
   DDPIXELFORMAT	ddpf;
   HRESULT			result;
@@ -420,10 +425,10 @@ boolean CheckPixelFormat( win32_driver_t * win32_driver )
 
   memset( &ddpf, 0, sizeof( DDPIXELFORMAT ));
   ddpf.dwSize = sizeof( DDPIXELFORMAT );
-  result = IDirectDrawSurface4_GetPixelFormat( win32_driver->primary, &ddpf );
+  result = IDirectDrawSurface_GetPixelFormat( win32_driver->primary, &ddpf );
   if( result != DD_OK )
     {
-      Error( 0, "IDirectDrawSurface4_GetPixelFormat ( CheckPixelFormat ) : error %u", result );
+      Error( 0, "IDirectDrawSurface_GetPixelFormat ( CheckPixelFormat ) : error 0x%lx", result );
       return 0;
     }
 
@@ -486,10 +491,10 @@ boolean CheckPixelFormat( win32_driver_t * win32_driver )
  * being pushed to the backend. */
 
 
-LPDIRECTDRAWSURFACE4 CreateBMP( win32_driver_t * win32_driver, int resource )
+static LPDIRECTDRAWSURFACE CreateBMP( win32_driver_t * win32_driver, int resource )
 {
-  LPDIRECTDRAWSURFACE4	bmp_surf;
-  DDSURFACEDESC2	bmp_ddsd;
+  LPDIRECTDRAWSURFACE	bmp_surf;
+  DDSURFACEDESC	bmp_ddsd;
   HBITMAP		bmp_hndl;
   BITMAP		bmp_head;
   HDC			hdc_dds;
@@ -515,7 +520,7 @@ LPDIRECTDRAWSURFACE4 CreateBMP( win32_driver_t * win32_driver, int resource )
   bmp_ddsd.dwWidth        = bmp_head.bmWidth;
   bmp_ddsd.dwHeight       = bmp_head.bmHeight;
 
-  if( IDirectDraw4_CreateSurface( win32_driver->ddobj, &bmp_ddsd, &bmp_surf, 0 ) != DD_OK )
+  if( IDirectDraw_CreateSurface( win32_driver->ddobj, &bmp_ddsd, &bmp_surf, 0 ) != DD_OK )
     {
       Error( 0, "CreateSurface ( bitmap ) : could not create dd surface" );
       return 0;
@@ -525,7 +530,7 @@ LPDIRECTDRAWSURFACE4 CreateBMP( win32_driver_t * win32_driver, int resource )
    * create a compat dc and load
    * our bitmap into the compat dc */
 
-  IDirectDrawSurface4_GetDC( bmp_surf, &hdc_dds );
+  IDirectDrawSurface_GetDC( bmp_surf, &hdc_dds );
   hdc_mem = CreateCompatibleDC( hdc_dds );
   SelectObject( hdc_mem, bmp_hndl );
 
@@ -539,7 +544,7 @@ LPDIRECTDRAWSURFACE4 CreateBMP( win32_driver_t * win32_driver, int resource )
 
   DeleteDC( hdc_mem );
   DeleteObject( bmp_hndl );
-  IDirectDrawSurface4_ReleaseDC( bmp_surf, hdc_dds );
+  IDirectDrawSurface_ReleaseDC( bmp_surf, hdc_dds );
 
   return bmp_surf;
 }
@@ -549,8 +554,8 @@ LPDIRECTDRAWSURFACE4 CreateBMP( win32_driver_t * win32_driver, int resource )
  * a h/w overlay of the current frame type
  * is supported. */
 
-boolean Overlay( LPDIRECTDRAWSURFACE4 src_surface, RECT * src_rect,
-		 LPDIRECTDRAWSURFACE4 dst_surface, RECT * dst_rect,
+static boolean Overlay( LPDIRECTDRAWSURFACE src_surface, RECT * src_rect,
+		 LPDIRECTDRAWSURFACE dst_surface, RECT * dst_rect,
 		 COLORREF color_key )
 {
   DWORD			dw_color_key;
@@ -564,10 +569,10 @@ boolean Overlay( LPDIRECTDRAWSURFACE4 src_surface, RECT * src_rect,
 
   memset( &ddpf, 0, sizeof( DDPIXELFORMAT ));
   ddpf.dwSize = sizeof( DDPIXELFORMAT );
-  result = IDirectDrawSurface4_GetPixelFormat( dst_surface, &ddpf );
+  result = IDirectDrawSurface_GetPixelFormat( dst_surface, &ddpf );
   if( result != DD_OK )
     {
-      Error( 0, "IDirectDrawSurface4_GetPixelFormat : could not get surface pixel format" );
+      Error( 0, "IDirectDrawSurface_GetPixelFormat : could not get surface pixel format" );
       return FALSE;
     }
 
@@ -585,19 +590,19 @@ boolean Overlay( LPDIRECTDRAWSURFACE4 src_surface, RECT * src_rect,
 
   /* attempt to overlay the surface */
 
-  result = IDirectDrawSurface4_UpdateOverlay( src_surface, src_rect, dst_surface, dst_rect, flags, &ddofx );
+  result = IDirectDrawSurface_UpdateOverlay( src_surface, src_rect, dst_surface, dst_rect, flags, &ddofx );
   if( result != DD_OK )
     {
       if( result == DDERR_SURFACELOST )
 	{
-	  IDirectDrawSurface4_Restore( src_surface );
-	  IDirectDrawSurface4_Restore( dst_surface );
+	  IDirectDrawSurface_Restore( src_surface );
+	  IDirectDrawSurface_Restore( dst_surface );
 
-	  IDirectDrawSurface4_UpdateOverlay( src_surface, src_rect, dst_surface, dst_rect, flags, &ddofx );
+	  IDirectDrawSurface_UpdateOverlay( src_surface, src_rect, dst_surface, dst_rect, flags, &ddofx );
 	}
       else
 	{
-	  Error( 0, "IDirectDrawSurface4_UpdateOverlay : error %i", result );
+	  Error( 0, "IDirectDrawSurface_UpdateOverlay : error 0x%lx", result );
 	  return FALSE;
 	}
     }
@@ -610,8 +615,8 @@ boolean Overlay( LPDIRECTDRAWSURFACE4 src_surface, RECT * src_rect,
  * h/w overlay of the current frame format is
  * not supported. */
 
-boolean BltCopy( LPDIRECTDRAWSURFACE4 src_surface, RECT * src_rect,
-		 LPDIRECTDRAWSURFACE4 dst_surface, RECT * dst_rect )
+static boolean BltCopy( LPDIRECTDRAWSURFACE src_surface, RECT * src_rect,
+		 LPDIRECTDRAWSURFACE dst_surface, RECT * dst_rect )
 {
   DDSURFACEDESC	ddsd_target;
   HRESULT	result;
@@ -621,19 +626,19 @@ boolean BltCopy( LPDIRECTDRAWSURFACE4 src_surface, RECT * src_rect,
 
   /* attempt to blt the surface sontents */
 
-  result = IDirectDrawSurface4_Blt( dst_surface, dst_rect, src_surface, src_rect, DDBLT_WAIT, 0 );
+  result = IDirectDrawSurface_Blt( dst_surface, dst_rect, src_surface, src_rect, DDBLT_WAIT, 0 );
   if( result != DD_OK )
     {
       if( result != DDERR_SURFACELOST )
 	{
-	  IDirectDrawSurface4_Restore( src_surface );
-	  IDirectDrawSurface4_Restore( dst_surface );
+	  IDirectDrawSurface_Restore( src_surface );
+	  IDirectDrawSurface_Restore( dst_surface );
 
-	  IDirectDrawSurface4_Blt( dst_surface, dst_rect, src_surface, src_rect, DDBLT_WAIT, 0 );
+	  IDirectDrawSurface_Blt( dst_surface, dst_rect, src_surface, src_rect, DDBLT_WAIT, 0 );
 	}
       else
 	{
-	  Error( 0, "IDirectDrawSurface4_Blt : error %i", result );
+	  Error( 0, "IDirectDrawSurface_Blt : error 0x%lx", result );
 	  return FALSE;
 	}
     }
@@ -647,7 +652,7 @@ boolean BltCopy( LPDIRECTDRAWSURFACE4 src_surface, RECT * src_rect,
  * moving the image contents contained in our
  * secondary surface to our primary surface. */
 
-boolean DisplayFrame( win32_driver_t * win32_driver )
+static boolean DisplayFrame( win32_driver_t * win32_driver )
 {
   int					view_width;
   int					view_height;
@@ -752,10 +757,10 @@ boolean DisplayFrame( win32_driver_t * win32_driver )
 
 /* Lock our back buffer to update its contents. */
 
-void * Lock( void * surface )
+static void * Lock( void * surface )
 {
-  LPDIRECTDRAWSURFACE4	lock_surface = ( LPDIRECTDRAWSURFACE4 ) surface;
-  DDSURFACEDESC2	ddsd;
+  LPDIRECTDRAWSURFACE	lock_surface = ( LPDIRECTDRAWSURFACE ) surface;
+  DDSURFACEDESC	ddsd;
   HRESULT		result;
 
   if( !surface )
@@ -764,11 +769,11 @@ void * Lock( void * surface )
   memset( &ddsd, 0, sizeof( ddsd ) );
   ddsd.dwSize = sizeof( ddsd );
 
-  result = IDirectDrawSurface4_Lock( lock_surface, 0, &ddsd, DDLOCK_WAIT | DDLOCK_NOSYSLOCK, 0 );
+  result = IDirectDrawSurface_Lock( lock_surface, 0, &ddsd, DDLOCK_WAIT | DDLOCK_NOSYSLOCK, 0 );
   if( result == DDERR_SURFACELOST )
     {
-      IDirectDrawSurface4_Restore( lock_surface );
-      result = IDirectDrawSurface4_Lock( lock_surface, 0, &ddsd, DDLOCK_WAIT | DDLOCK_NOSYSLOCK, 0 );
+      IDirectDrawSurface_Restore( lock_surface );
+      result = IDirectDrawSurface_Lock( lock_surface, 0, &ddsd, DDLOCK_WAIT | DDLOCK_NOSYSLOCK, 0 );
 
       if( result != DD_OK )
 	return 0;
@@ -784,20 +789,18 @@ void * Lock( void * surface )
     }
 
   return ddsd.lpSurface;
-
-  return 0;
 }
 
 /* Unlock our back buffer to prepair for display. */
 
-void Unlock( void * surface )
+static void Unlock( void * surface )
 {
-  LPDIRECTDRAWSURFACE4 lock_surface = ( LPDIRECTDRAWSURFACE4 ) surface;
+  LPDIRECTDRAWSURFACE lock_surface = ( LPDIRECTDRAWSURFACE ) surface;
 
   if( !surface )
     return;
 
-  IDirectDrawSurface4_Unlock( lock_surface, 0 );
+  IDirectDrawSurface_Unlock( lock_surface, 0 );
 }
 
 /* -----------------------------------------
@@ -851,8 +854,7 @@ static void win32_frame_dispose( vo_frame_t * vo_frame )
 {
   win32_frame_t * win32_frame = ( win32_frame_t * ) vo_frame;
 
-  if( win32_frame->buffer )
-    free( win32_frame->buffer );
+  free(win32_frame->buffer);
 
   win32_free_framedata(vo_frame);
 
@@ -867,6 +869,8 @@ static vo_frame_t * win32_alloc_frame( vo_driver_t * vo_driver )
   if (!win32_frame)
     return NULL;
 
+  pthread_mutex_init(&win32_frame->vo_frame.mutex, NULL);
+
   win32_frame->vo_frame.proc_slice = NULL;
   win32_frame->vo_frame.proc_frame = NULL;
   win32_frame->vo_frame.field      = win32_frame_field;
@@ -885,6 +889,7 @@ static void win32_update_frame_format( vo_driver_t * vo_driver, vo_frame_t * vo_
 
   /*printf("vo_out_directx : win32_update_frame_format() - width = %d, height=%d, ratio_code=%d, format=%d, flags=%d\n", width, height, ratio_code, format, flags);*/
 
+
   if( ( win32_frame->format	!= format	) ||
       ( win32_frame->width	!= width	) ||
       ( win32_frame->height	!= height	) )
@@ -936,6 +941,7 @@ static void win32_update_frame_format( vo_driver_t * vo_driver, vo_frame_t * vo_
       win32_frame->height	= height;
       win32_frame->ratio	= ratio;
     }
+
 }
 
 static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame )
@@ -945,6 +951,7 @@ static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame
   int              offset;
   int              size;
 
+
   /* if the required width, height or format has changed
    * then recreate the secondary buffer */
 
@@ -988,13 +995,13 @@ static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame
 #if NEW_YUV
 	  win32_driver->yuv2rgb->configure( win32_driver->yuv2rgb,
 					    win32_driver->width, win32_driver->height,
-					    win32_driver->width, win32_driver->width/2,
+				 	    win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[1],
 					    win32_driver->width, win32_driver->height,
-					    win32_driver->width * win32_driver->bytespp );
+					    win32_driver->width * win32_driver->bytespp);
 #else
 	  yuv2rgb_setup( win32_driver->yuv2rgb,
 			 win32_driver->width, win32_driver->height,
-			 win32_driver->width, win32_driver->width/2,
+		 	 win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[1],
 			 win32_driver->width, win32_driver->height,
 			 win32_driver->width * win32_driver->bytespp );
 
@@ -1014,14 +1021,14 @@ static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame
 #if NEW_YUV
 	  win32_driver->yuv2rgb->configure( win32_driver->yuv2rgb,
 					    win32_driver->width, win32_driver->height,
-					    win32_driver->width, win32_driver->width/2,
+				 	    win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[0] / 2,
 					    win32_driver->width, win32_driver->height,
 					    win32_driver->width * win32_driver->bytespp );
 #else
 
 	  yuv2rgb_setup( win32_driver->yuv2rgb,
 			 win32_driver->width, win32_driver->height,
-			 win32_driver->width, win32_driver->width/2,
+			 win32_frame->vo_frame.pitches[0], win32_frame->vo_frame.pitches[0] / 2,
 			 win32_driver->width, win32_driver->height,
 			 win32_driver->width * win32_driver->bytespp );
 
@@ -1058,22 +1065,22 @@ static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame
 
 	    offset = 0;
 	    size   = frame->pitches[0] * frame->height;
-	    memcpy( img+offset, frame->base[0], size);
+	    xine_fast_memcpy( img+offset, frame->base[0], size);
 
 	    offset += size;
 	    size   = frame->pitches[2]* frame->height / 2;
-	    memcpy( img+offset, frame->base[2], size);
+	    xine_fast_memcpy( img+offset, frame->base[2], size);
 				
 	    offset += size;
 	    size   = frame->pitches[1] * frame->height / 2;
-	    memcpy( img+offset, frame->base[1], size);
+	    xine_fast_memcpy( img+offset, frame->base[1], size);
 	  }
 	  break;
 	case XINE_IMGFMT_YUY2:
-	  memcpy( win32_driver->contents, win32_frame->vo_frame.base[0], win32_frame->vo_frame.pitches[0] * win32_frame->vo_frame.height * 2);
+	  xine_fast_memcpy( win32_driver->contents, win32_frame->vo_frame.base[0], win32_frame->vo_frame.pitches[0] * win32_frame->vo_frame.height * 2);
 	  break;
 	default:
-	  memcpy( win32_driver->contents, win32_frame->vo_frame.base[0], win32_frame->vo_frame.pitches[0] * win32_frame->vo_frame.height * 2);
+	  xine_fast_memcpy( win32_driver->contents, win32_frame->vo_frame.base[0], win32_frame->vo_frame.pitches[0] * win32_frame->vo_frame.height * 2);
 	  break;
 	}
     }
@@ -1091,6 +1098,7 @@ static void win32_display_frame( vo_driver_t * vo_driver, vo_frame_t * vo_frame
     vo_frame->free(&win32_driver->current->vo_frame);
   }
   win32_driver->current = (win32_frame_t *)vo_frame;
+
 }
 
 static void win32_overlay_blend( vo_driver_t * vo_driver, vo_frame_t * vo_frame, vo_overlay_t * vo_overlay )
@@ -1098,15 +1106,18 @@ static void win32_overlay_blend( vo_driver_t * vo_driver, vo_frame_t * vo_frame,
   win32_frame_t * win32_frame = ( win32_frame_t * ) vo_frame;
   win32_driver_t * win32_driver = ( win32_driver_t * ) vo_driver;
 
+  win32_driver->alphablend_extra_data.offset_x = vo_frame->overlay_offset_x;
+  win32_driver->alphablend_extra_data.offset_y = vo_frame->overlay_offset_y;
+
   /* temporary overlay support, somthing more appropriate
    * for win32 will be devised at a later date */
 
   if( vo_overlay->rle )
     {
       if( vo_frame->format == XINE_IMGFMT_YV12 )
-	blend_yuv( win32_frame->vo_frame.base, vo_overlay, win32_frame->width, win32_frame->height, win32_frame->vo_frame.pitches, &win32_driver->alphablend_extra_data );
+	_x_blend_yuv( win32_frame->vo_frame.base, vo_overlay, win32_frame->width, win32_frame->height, win32_frame->vo_frame.pitches, &win32_driver->alphablend_extra_data );
       else
-	blend_yuy2( win32_frame->vo_frame.base[0], vo_overlay, win32_frame->width, win32_frame->height, win32_frame->vo_frame.pitches[0], &win32_driver->alphablend_extra_data );
+	_x_blend_yuy2( win32_frame->vo_frame.base[0], vo_overlay, win32_frame->width, win32_frame->height, win32_frame->vo_frame.pitches[0], &win32_driver->alphablend_extra_data );
     }
 }
 
@@ -1176,6 +1187,7 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *wi
   directx_class_t *class = (directx_class_t *)class_gen;
   win32_driver_t  *win32_driver = ( win32_driver_t * ) xine_xmalloc ( sizeof( win32_driver_t ) );
 
+
   _x_alphablend_init(&win32_driver->alphablend_extra_data, class->xine);
   
   win32_driver->xine = class->xine;
@@ -1196,12 +1208,15 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *wi
   win32_driver->vo_driver.dispose		= win32_exit;
   win32_driver->vo_driver.redraw_needed         = win32_redraw_needed;
 
-  CreatePrimary( win32_driver );
+  if (!CreatePrimary( win32_driver )) {
+      Destroy( win32_driver );
+      return NULL;
+  }
   if( !CheckPixelFormat( win32_driver ) )
     {
       Error( 0, "vo_directx : Your screen pixel format is not supported" );
       Destroy( win32_driver );
-      return 0;
+      return NULL;
     }
 
 #if (NEW_YUV)
@@ -1212,7 +1227,7 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *wi
 #endif
 
   return ( vo_driver_t * ) win32_driver;
-}    
+}
 
 
 static char* get_identifier (video_driver_class_t *this_gen) {
@@ -1232,7 +1247,7 @@ static void dispose_class (video_driver_class_t *this_gen) {
 static void *init_class (xine_t *xine, void *visual_gen) {
 
   directx_class_t    *directx;
-  
+
   /*
    * from this point on, nothing should go wrong anymore
    */
@@ -1249,7 +1264,7 @@ static void *init_class (xine_t *xine, void *visual_gen) {
   return directx;
 }
 
-static vo_info_t vo_info_win32 = {
+static const vo_info_t vo_info_win32 = {
   7,                        /* priority    */
   XINE_VISUAL_TYPE_DIRECTX  /* visual type */
 };
@@ -1258,8 +1273,8 @@ static vo_info_t vo_info_win32 = {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_VIDEO_OUT, 20, "vo_directx", XINE_VERSION_CODE, &vo_info_win32, init_class },
+  { PLUGIN_VIDEO_OUT, 21, "vo_directx", XINE_VERSION_CODE, &vo_info_win32, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
