@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_fb.c,v 1.40 2004/12/12 22:01:27 mroi Exp $
+ * $Id: video_out_fb.c,v 1.48 2006/07/10 22:08:44 dgp85 Exp $
  * 
  * video_out_fb.c, frame buffer xine driver by Miguel Freitas
  *
@@ -81,7 +81,6 @@
 */
 
 #include "xine_internal.h"
-#include "alphablend.h"
 #include "yuv2rgb.h"
 #include "xineutils.h"
 #include "vo_scale.h"
@@ -127,6 +126,8 @@ typedef struct fb_driver_s
   int                yuv2rgb_mode;
   int                yuv2rgb_swap;
   int                yuv2rgb_brightness;
+  int                yuv2rgb_contrast;
+  int                yuv2rgb_saturation;
   uint8_t           *yuv2rgb_cmap;
   yuv2rgb_factory_t *yuv2rgb_factory;
 
@@ -492,9 +493,9 @@ static void fb_overlay_clut_yuv2rgb(fb_driver_t *this,
     overlay->rgb_clut++;
   }
 	
-  if(!overlay->clip_rgb_clut)
+  if(!overlay->hili_rgb_clut)
   {
-    clut = (clut_t*) overlay->clip_color;
+    clut = (clut_t*) overlay->hili_color;
 		
     for(i = 0;
 	i < sizeof(overlay->color)/sizeof(overlay->color[0]);
@@ -507,7 +508,7 @@ static void fb_overlay_clut_yuv2rgb(fb_driver_t *this,
 				 clut[i].cb,
 				 clut[i].cr);
     }
-    overlay->clip_rgb_clut++;
+    overlay->hili_rgb_clut++;
   }
 }
 
@@ -517,16 +518,19 @@ static void fb_overlay_blend (vo_driver_t *this_gen, vo_frame_t *frame_gen,
   fb_driver_t *this = (fb_driver_t *)this_gen;
   fb_frame_t *frame = (fb_frame_t *)frame_gen;
 
+  this->alphablend_extra_data.offset_x = frame_gen->overlay_offset_x;
+  this->alphablend_extra_data.offset_y = frame_gen->overlay_offset_y;
+
   /* Alpha Blend here */
   if(overlay->rle)
   {
-    if(!overlay->rgb_clut || !overlay->clip_rgb_clut)
+    if(!overlay->rgb_clut || !overlay->hili_rgb_clut)
        fb_overlay_clut_yuv2rgb(this,overlay,frame);
 
     switch(this->bpp)
     {
        case 16:
-	blend_rgb16(frame->data,
+	_x_blend_rgb16(frame->data,
 		    overlay,
 		    frame->sc.output_width,
 		    frame->sc.output_height,
@@ -536,7 +540,7 @@ static void fb_overlay_blend (vo_driver_t *this_gen, vo_frame_t *frame_gen,
         break;
 				
        case 24:
-	blend_rgb24(frame->data,
+	_x_blend_rgb24(frame->data,
 		    overlay,
 		    frame->sc.output_width,
 		    frame->sc.output_height,
@@ -546,7 +550,7 @@ static void fb_overlay_blend (vo_driver_t *this_gen, vo_frame_t *frame_gen,
         break;
 				
        case 32:
-	blend_rgb32(frame->data,
+	_x_blend_rgb32(frame->data,
 		    overlay,
 		    frame->sc.output_width,
 		    frame->sc.output_height,
@@ -632,6 +636,10 @@ static int fb_get_property(vo_driver_t *this_gen, int property)
 
     case VO_PROP_BRIGHTNESS:
       return this->yuv2rgb_brightness;
+    case VO_PROP_CONTRAST:
+      return this->yuv2rgb_contrast;
+    case VO_PROP_SATURATION:
+      return this->yuv2rgb_saturation;
 
     case VO_PROP_WINDOW_WIDTH:
       return this->sc.gui_width;
@@ -663,9 +671,26 @@ static int fb_set_property(vo_driver_t *this_gen, int property, int value)
 
     case VO_PROP_BRIGHTNESS:
       this->yuv2rgb_brightness = value;
-      this->yuv2rgb_factory->
-	set_csc_levels(this->yuv2rgb_factory, value, 128, 128);
-      xprintf(this->xine, XINE_VERBOSITY_DEBUG, "video_out_fb: brightness changed to %d\n", value);
+      this->yuv2rgb_factory->set_csc_levels (this->yuv2rgb_factory,
+                                             this->yuv2rgb_brightness,
+                                             this->yuv2rgb_contrast,
+                                             this->yuv2rgb_saturation);
+      break;
+
+    case VO_PROP_CONTRAST:
+      this->yuv2rgb_contrast = value;
+      this->yuv2rgb_factory->set_csc_levels (this->yuv2rgb_factory,
+                                             this->yuv2rgb_brightness,
+                                             this->yuv2rgb_contrast,
+                                             this->yuv2rgb_saturation);
+      break;
+
+    case VO_PROP_SATURATION:
+      this->yuv2rgb_saturation = value;
+      this->yuv2rgb_factory->set_csc_levels (this->yuv2rgb_factory,
+                                             this->yuv2rgb_brightness,
+                                             this->yuv2rgb_contrast,
+                                             this->yuv2rgb_saturation);
       break;
 
     default:
@@ -680,16 +705,21 @@ static void fb_get_property_min_max(vo_driver_t *this_gen,
 				    int property, int *min, int *max)
 {
   /* fb_driver_t *this = (fb_driver_t *) this_gen;  */
-	
-  if(property == VO_PROP_BRIGHTNESS)
-  {
-    *min = -100;
-    *max = +100;
-  }
-  else
-  {
+
+  switch (property) {
+  case VO_PROP_BRIGHTNESS:
+    *min = -128;
+    *max = +127;
+    break;
+  case VO_PROP_CONTRAST:
+  case VO_PROP_SATURATION:
+    *min = 0;
+    *max = 255;
+    break;
+  default:
     *min = 0;
     *max = 0;
+    break;
   }
 }
 
@@ -767,6 +797,32 @@ static int get_fb_fix_screeninfo(int fd, struct fb_fix_screeninfo *fix, xine_t *
   return 1;
 }
 
+static int set_fb_palette (int fd, const struct fb_var_screeninfo *var)
+{
+  unsigned short red[256], green[256], blue[256];
+  const struct fb_cmap fb_cmap = {
+    0, 256, red, green, blue, NULL
+  };
+  int i, mask;
+
+  if (!var->red.offset && !var->green.offset && !var->blue.offset)
+    return 1; /* we only handle true-colour modes */
+
+  /* Fill in the palette data */
+  mask = (1 << var->red.length) - 1;
+  for (i = 0; i < 256; ++i)
+    red[i] = (i & mask) * 65535.0 / mask;
+  mask = (1 << var->green.length) - 1;
+  for (i = 0; i < 256; ++i)
+    green[i] = (i & mask) * 65535.0 / mask;
+  mask = (1 << var->blue.length) - 1;
+  for (i = 0; i < 256; ++i)
+    blue[i] = (i & mask) * 65535.0 / mask;
+
+  /* Set the palette; return true on success */
+  return !ioctl (fd, FBIOPUTCMAP, &fb_cmap);
+}
+
 static void register_callbacks(fb_driver_t *this)
 {
   this->vo_driver.get_capabilities     = fb_get_capabilities;
@@ -880,21 +936,17 @@ static int setup_yuv2rgb(fb_driver_t *this, config_values_t *config,
   if(!this->yuv2rgb_mode)
     return 0;
 
-  this->yuv2rgb_swap  = 0;
-  this->yuv2rgb_brightness =
-    config->register_range(config, "video.output.fb_gamma", 0,
-			   -100, 100,
-			   _("brightness correction"),
-			   _("The brightness correction can be used to lighten or darken the image. "
-			     "It changes the blacklevel without modifying the contrast, but it "
-			     "limits the tonal range."),
-			   0, NULL, NULL);
+  this->yuv2rgb_swap       = 0;
+  this->yuv2rgb_brightness = 0;
+  this->yuv2rgb_contrast   = this->yuv2rgb_saturation = 128;
 
   this->yuv2rgb_factory = yuv2rgb_factory_init(this->yuv2rgb_mode,
 					       this->yuv2rgb_swap, 
 					       this->yuv2rgb_cmap);
-  this->yuv2rgb_factory->set_csc_levels(this->yuv2rgb_factory,
-					this->yuv2rgb_brightness, 128, 128);
+  this->yuv2rgb_factory->set_csc_levels (this->yuv2rgb_factory,
+                                         this->yuv2rgb_brightness,
+                                         this->yuv2rgb_contrast,
+                                         this->yuv2rgb_saturation);
   
   return 1;
 }
@@ -989,6 +1041,8 @@ static vo_driver_t *fb_open_plugin(video_driver_class_t *class_gen,
     goto error;
   if(!get_fb_fix_screeninfo(this->fd, &this->fb_fix, class->xine))
     goto error;
+  if (!set_fb_palette (this->fd, &this->fb_var))
+    goto error;
    
   this->xine = class->xine;
 
@@ -1068,7 +1122,7 @@ static void *fb_init_class(xine_t *xine, void *visual_gen)
   return this;
 }
 
-static vo_info_t vo_info_fb =
+static const vo_info_t vo_info_fb =
 {
   1,                    /* priority    */
 #ifdef USE_X11_VISUAL
@@ -1079,8 +1133,8 @@ static vo_info_t vo_info_fb =
 };
 
 /* exported plugin catalog entry */
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_VIDEO_OUT, 20, "fb", XINE_VERSION_CODE, &vo_info_fb, fb_init_class },
+  { PLUGIN_VIDEO_OUT, 21, "fb", XINE_VERSION_CODE, &vo_info_fb, fb_init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
