@@ -17,11 +17,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_none.c,v 1.8 2003/01/11 10:49:47 f1rmb Exp $
+ * $Id: video_out_none.c,v 1.27 2006/07/10 22:08:44 dgp85 Exp $
  *
  * Was originally part of toxine frontend.
  * ...but has now been adapted to xine coding style standards ;)
- * ......what changes, impressing!
+ * ......what changes, impressive!
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,7 +35,6 @@
 
 #include "xine.h"
 
-#include "xine.h"
 #include "video_out.h"
 #include "xine_internal.h"
 #include "xineutils.h"
@@ -45,19 +44,22 @@ typedef struct {
   vo_frame_t           vo_frame;
   int                  width;
   int                  height;
-  int                  ratio_code;
+  double               ratio;
   int                  format;
+  xine_t               *xine;
 } none_frame_t;
 
 typedef struct {
   vo_driver_t          vo_driver;
   config_values_t     *config;
-  int                   ratio;
+  int                  ratio;
+  xine_t               *xine;
 } none_driver_t;
 
 typedef struct {
   video_driver_class_t  driver_class;
   config_values_t      *config;
+  xine_t               *xine;
 } none_class_t;
 
 
@@ -65,10 +67,9 @@ static void free_framedata(none_frame_t* frame) {
   if(frame->vo_frame.base[0]) {
     free(frame->vo_frame.base[0]);
     frame->vo_frame.base[0] = NULL;
+    frame->vo_frame.base[1] = NULL;
+    frame->vo_frame.base[2] = NULL;
   }
-  
-  frame->vo_frame.base[1] = NULL;
-  frame->vo_frame.base[2] = NULL;
 }
 
 static void none_frame_dispose(vo_frame_t *vo_frame) {
@@ -86,35 +87,36 @@ static uint32_t none_get_capabilities(vo_driver_t *vo_driver) {
 }
 
 static vo_frame_t *none_alloc_frame(vo_driver_t *vo_driver) {
+  /* none_driver_t *this = (none_driver_t *) vo_driver; */
   none_frame_t  *frame;
   
-  frame = (none_frame_t *) malloc(sizeof(none_frame_t));
-  memset(frame, 0, sizeof(none_frame_t));
-  
-  if(frame == NULL) {
-    printf ("video_out_none: out of memory in none_alloc_frame\n");
-    abort();
-  }
-  
+  frame = (none_frame_t *) xine_xmalloc(sizeof(none_frame_t));
+  if(!frame)
+    return NULL;
+
+  pthread_mutex_init(&frame->vo_frame.mutex, NULL);
+
   frame->vo_frame.base[0] = NULL;
   frame->vo_frame.base[1] = NULL;
   frame->vo_frame.base[2] = NULL;
   
-  frame->vo_frame.copy    = NULL;
-  frame->vo_frame.field   = none_frame_field;
-  frame->vo_frame.dispose = none_frame_dispose;
-  frame->vo_frame.driver  = vo_driver;
+  frame->vo_frame.proc_slice = NULL;
+  frame->vo_frame.proc_frame = NULL;
+  frame->vo_frame.field      = none_frame_field;
+  frame->vo_frame.dispose    = none_frame_dispose;
+  frame->vo_frame.driver     = vo_driver;
   
   return (vo_frame_t *)frame;
 }
 
 static void none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_frame,
 				     uint32_t width, uint32_t height, 
-				     int ratio_code, int format, int flags) {
-  none_frame_t  *frame = (none_frame_t *)vo_frame;
-  uint32_t      frame_size = (width * height);
+				     double ratio, int format, int flags) {
+  none_driver_t *this = (none_driver_t *) vo_driver;
+  none_frame_t  *frame = (none_frame_t *) vo_frame;
 
   if((frame->width != width) || (frame->height != height) || (frame->format != format)) {
+    
     free_framedata(frame);
     
     frame->width  = width;
@@ -123,24 +125,32 @@ static void none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_fram
     
     switch(format) {
 
-    case XINE_IMGFMT_YV12:
-      frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
-      frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
-      frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
-      frame->vo_frame.base[0] = malloc(frame_size * 3/2);
-      frame->vo_frame.base[2] = frame->vo_frame.base[0]+frame_size;
-      frame->vo_frame.base[1] = frame->vo_frame.base[0]+frame_size*5/4;
+    case XINE_IMGFMT_YV12: 
+      {
+	int y_size, uv_size;
+	
+	frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
+	frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
+	frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
+	
+	y_size  = frame->vo_frame.pitches[0] * height;
+	uv_size = frame->vo_frame.pitches[1] * ((height+1)/2);
+	
+	frame->vo_frame.base[0] = malloc (y_size + 2*uv_size);
+	frame->vo_frame.base[1] = frame->vo_frame.base[0]+y_size+uv_size;
+	frame->vo_frame.base[2] = frame->vo_frame.base[0]+y_size;
+      }
       break;
 
     case XINE_IMGFMT_YUY2:
       frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
-      frame->vo_frame.base[0] = malloc((frame_size * 2));
+      frame->vo_frame.base[0] = malloc(frame->vo_frame.pitches[0] * height);
       frame->vo_frame.base[1] = NULL;
       frame->vo_frame.base[2] = NULL;
       break;
 
     default:
-      printf ("video_out_none: unknown frame format %04x)\n", format);
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG, "video_out_none: unknown frame format %04x)\n", format);
       break;
 
     }
@@ -150,19 +160,20 @@ static void none_update_frame_format(vo_driver_t *vo_driver, vo_frame_t *vo_fram
 	    || frame->vo_frame.base[1] == NULL 
 	    || frame->vo_frame.base[2] == NULL))
        || (format == XINE_IMGFMT_YUY2 && frame->vo_frame.base[0] == NULL)) {
-      printf ("video_out_none: error. (framedata allocation failed: out of memory)\n"); 
+      xprintf (this->xine, XINE_VERBOSITY_DEBUG,
+	       "video_out_none: error. (framedata allocation failed: out of memory)\n"); 
       free_framedata(frame);
     }
   }
 
-  frame->ratio_code = ratio_code;
+  frame->ratio = ratio;
 }
 
 static void none_display_frame(vo_driver_t *vo_driver, vo_frame_t *vo_frame) {
   /* none_driver_t  *driver = (none_driver_t *)vo_driver; */
   none_frame_t   *frame = (none_frame_t *)vo_frame;
   
-  frame->vo_frame.displayed(&frame->vo_frame);
+  frame->vo_frame.free(&frame->vo_frame);
 }
 
 static int none_get_property(vo_driver_t *vo_driver, int property) {
@@ -187,8 +198,8 @@ static int none_set_property(vo_driver_t *vo_driver, int property, int value) {
   switch(property) {
 
   case VO_PROP_ASPECT_RATIO:
-    if(value >= NUM_ASPECT_RATIOS)
-      value = ASPECT_AUTO;
+    if(value >= XINE_VO_ASPECT_NUM_RATIOS)
+      value = XINE_VO_ASPECT_AUTO;
 
     driver->ratio = value;
     break;
@@ -234,10 +245,11 @@ static vo_driver_t *open_plugin(video_driver_class_t *driver_class, const void *
   none_class_t    *class = (none_class_t *) driver_class;
   none_driver_t   *driver;
   
-  driver = (none_driver_t *) malloc(sizeof(none_driver_t));
+  driver = (none_driver_t *) xine_xmalloc(sizeof(none_driver_t));
   
   driver->config = class->config;
-  driver->ratio  = ASPECT_AUTO;
+  driver->xine   = class->xine;
+  driver->ratio  = XINE_VO_ASPECT_AUTO;
   
   driver->vo_driver.get_capabilities     = none_get_capabilities;
   driver->vo_driver.alloc_frame          = none_alloc_frame ;
@@ -276,7 +288,7 @@ static void dispose_class (video_driver_class_t *driver_class) {
 static void *init_class (xine_t *xine, void *visual) {
   none_class_t        *this;
   
-  this = (none_class_t *) malloc(sizeof(none_class_t));
+  this = (none_class_t *) xine_xmalloc(sizeof(none_class_t));
 
   this->driver_class.open_plugin     = open_plugin;
   this->driver_class.get_identifier  = get_identifier;
@@ -284,17 +296,18 @@ static void *init_class (xine_t *xine, void *visual) {
   this->driver_class.dispose         = dispose_class;
 
   this->config                       = xine->config;
+  this->xine                         = xine;
 
   return this;
 }
 
-static vo_info_t vo_info_none = {
+static const vo_info_t vo_info_none = {
   5,                        /* Priority    */
   XINE_VISUAL_TYPE_NONE     /* Visual type */
 };
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_VIDEO_OUT, 14, "none", XINE_VERSION_CODE, &vo_info_none, init_class },
+  { PLUGIN_VIDEO_OUT, 21, "none", XINE_VERSION_CODE, &vo_info_none, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
