@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_vidix.c,v 1.73 2006/06/02 22:18:59 dsalt Exp $
+ * $Id: video_out_vidix.c,v 1.74 2006/07/10 22:08:44 dgp85 Exp $
  * 
  * video_out_vidix.c
  *
@@ -1310,7 +1310,7 @@ static const vo_info_t vo_info_vidixfb = {
  * exported plugin catalog entry
  */
 
-const plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
 #ifdef HAVE_X11
   { PLUGIN_VIDEO_OUT, 21, "vidix", XINE_VERSION_CODE, &vo_info_vidix, vidix_init_class },
