@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: buffer_types.c,v 1.104 2006/03/06 00:19:50 hadess Exp $
+ * $Id: buffer_types.c,v 1.106 2006/10/02 15:56:06 valtri Exp $
  *
  *
  * contents:
@@ -739,6 +739,30 @@ static video_db_t video_db[] = {
   BUF_VIDEO_DUCKTM2,
   "Duck TrueMotion 2"
 },
+{
+  {
+    meFOURCC('C','S','C','D'),
+    0
+  },
+  BUF_VIDEO_CSCD,
+  "CamStudio"
+},
+{
+  {
+    meFOURCC('Z','M','B','V'),
+    0
+  },
+  BUF_VIDEO_ZMBV,
+  "Zip Motion Blocks Video"
+},
+{
+  {
+    meFOURCC('K','M','V','C'),
+    0
+  },
+  BUF_VIDEO_KMVC,
+  "Karl Morton's Video Codec"
+},
 { { 0 }, 0, "last entry" }
 };
 
@@ -1076,6 +1100,13 @@ static audio_db_t audio_db[] = {
   BUF_AUDIO_ALAC,
   "Apple Lossless Audio Codec"
 },
+{
+  {
+    0x22,
+  },
+  BUF_AUDIO_TRUESPEECH,
+  "Truespeech"
+},
 {
   {
     0
