@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2005 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -20,7 +20,7 @@
  * Demuxer helper functions
  * hide some xine engine details from demuxers and reduce code duplication
  *
- * $Id: demux.c,v 1.54.2.1 2005/04/20 17:21:15 mroi Exp $ 
+ * $Id: demux.c,v 1.65 2006/12/03 19:23:16 miguelfreitas Exp $ 
  */
 
 
@@ -67,6 +67,9 @@ void _x_demux_flush_engine (xine_stream_t *stream) {
 
   buf_element_t *buf;
   
+  if( stream->gapless_switch )
+    return;
+  
   stream->xine->port_ticket->acquire(stream->xine->port_ticket, 1);
 
   /* only flush/discard output ports on master streams */
@@ -82,6 +85,8 @@ void _x_demux_flush_engine (xine_stream_t *stream) {
   stream->video_fifo->clear(stream->video_fifo);
   stream->audio_fifo->clear(stream->audio_fifo);
   
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_RESET_DECODER;
   stream->video_fifo->put (stream->video_fifo, buf);
@@ -90,6 +95,8 @@ void _x_demux_flush_engine (xine_stream_t *stream) {
   buf->type = BUF_CONTROL_RESET_DECODER;
   stream->audio_fifo->put (stream->audio_fifo, buf);
   
+  pthread_mutex_unlock(&stream->demux_mutex);  
+
   /* on seeking we must wait decoder fifos to process before doing flush. 
    * otherwise we flush too early (before the old data has left decoders)
    */
@@ -121,6 +128,8 @@ void _x_demux_control_newpts( xine_stream_t *stream, int64_t pts, uint32_t flags
 
   buf_element_t *buf;
       
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_NEWPTS;
   buf->decoder_flags = flags;
@@ -132,6 +141,8 @@ void _x_demux_control_newpts( xine_stream_t *stream, int64_t pts, uint32_t flags
   buf->decoder_flags = flags;
   buf->disc_off = pts;
   stream->audio_fifo->put (stream->audio_fifo, buf);
+
+  pthread_mutex_unlock(&stream->demux_mutex);  
 }
 
 /* sync with decoder fifos, making sure everything gets processed */
@@ -150,24 +161,28 @@ void _x_demux_control_headers_done (xine_stream_t *stream) {
 
   pthread_mutex_lock (&stream->counter_lock);
 
-  if (stream->video_thread) {
+  if (stream->video_thread_created) {
     header_count_video = stream->header_count_video + 1;
   } else {
     header_count_video = 0;
   }
 
-  if (stream->audio_thread) {
+  if (stream->audio_thread_created) {
     header_count_audio = stream->header_count_audio + 1;
   } else {
     header_count_audio = 0;
   }
   
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf_video->type = BUF_CONTROL_HEADERS_DONE;
   stream->video_fifo->put (stream->video_fifo, buf_video);
 
   buf_audio->type = BUF_CONTROL_HEADERS_DONE;
   stream->audio_fifo->put (stream->audio_fifo, buf_audio);
 
+  pthread_mutex_unlock(&stream->demux_mutex);  
+
   while ((stream->header_count_audio < header_count_audio) || 
          (stream->header_count_video < header_count_video)) {
     struct timeval tv;
@@ -195,6 +210,8 @@ void _x_demux_control_start( xine_stream_t *stream ) {
 
   buf_element_t *buf;
 
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_START;
   stream->video_fifo->put (stream->video_fifo, buf);
@@ -202,12 +219,16 @@ void _x_demux_control_start( xine_stream_t *stream ) {
   buf = stream->audio_fifo->buffer_pool_alloc (stream->audio_fifo);
   buf->type = BUF_CONTROL_START;
   stream->audio_fifo->put (stream->audio_fifo, buf);
+
+  pthread_mutex_unlock(&stream->demux_mutex);  
 }
 
 void _x_demux_control_end( xine_stream_t *stream, uint32_t flags ) {
 
   buf_element_t *buf;
 
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_END;
   buf->decoder_flags = flags;
@@ -217,12 +238,16 @@ void _x_demux_control_end( xine_stream_t *stream, uint32_t flags ) {
   buf->type = BUF_CONTROL_END;
   buf->decoder_flags = flags;
   stream->audio_fifo->put (stream->audio_fifo, buf);
+
+  pthread_mutex_unlock(&stream->demux_mutex);  
 }
 
 void _x_demux_control_nop( xine_stream_t *stream, uint32_t flags ) {
 
   buf_element_t *buf;
 
+  pthread_mutex_lock(&stream->demux_mutex);  
+
   buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
   buf->type = BUF_CONTROL_NOP;
   buf->decoder_flags = flags;
@@ -232,6 +257,8 @@ void _x_demux_control_nop( xine_stream_t *stream, uint32_t flags ) {
   buf->type = BUF_CONTROL_NOP;
   buf->decoder_flags = flags;
   stream->audio_fifo->put (stream->audio_fifo, buf);
+
+  pthread_mutex_unlock(&stream->demux_mutex);  
 }
 
 static void *demux_loop (void *stream_gen) {
@@ -286,19 +313,30 @@ static void *demux_loop (void *stream_gen) {
       status = stream->demux_plugin->get_status(stream->demux_plugin);
     }
 
+    /* delay sending finished event - used for image presentations */
+    while(stream->demux_thread_running &&
+          status == DEMUX_FINISHED && stream->delay_finish_event != 0){
+      pthread_mutex_unlock( &stream->demux_lock );
+      xine_usec_sleep(100000);
+      if( stream->delay_finish_event > 0 )
+        stream->delay_finish_event--;
+      pthread_mutex_lock( &stream->demux_lock );
+      status = stream->demux_plugin->get_status(stream->demux_plugin);
+    }
+
   } while( status == DEMUX_OK && stream->demux_thread_running &&
            !stream->emergency_brake);
 
   lprintf ("loop finished (status: %d)\n", status);
 
   pthread_mutex_lock (&stream->counter_lock);
-  if (stream->audio_thread)
+  if (stream->audio_thread_created)
     finished_count_audio = stream->finished_count_audio + 1;
-  if (stream->video_thread)
+  if (stream->video_thread_created)
     finished_count_video = stream->finished_count_video + 1;
   pthread_mutex_unlock (&stream->counter_lock);
 
-  /* demux_thread_running is zero if demux loop has being stopped by user */
+  /* demux_thread_running is zero if demux loop has been stopped by user */
   non_user = stream->demux_thread_running;
   stream->demux_thread_running = 0;
   
@@ -332,12 +370,13 @@ int _x_demux_start_thread (xine_stream_t *stream) {
   
   if( !stream->demux_thread_running ) {
 
-    if (stream->demux_thread) {
+    if (stream->demux_thread_created) {
       void *p;
       pthread_join(stream->demux_thread, &p);
     }
 
     stream->demux_thread_running = 1;
+    stream->demux_thread_created = 1;
     if ((err = pthread_create (&stream->demux_thread,
 			       NULL, demux_loop, (void *)stream)) != 0) {
       printf ("demux: can't create new thread (%s)\n", strerror(err));
@@ -368,9 +407,9 @@ int _x_demux_stop_thread (xine_stream_t *stream) {
 
   lprintf ("joining thread %ld\n", stream->demux_thread );
   
-  if( stream->demux_thread ) {
+  if( stream->demux_thread_created ) {
     pthread_join (stream->demux_thread, &p);
-    stream->demux_thread = 0;
+    stream->demux_thread_created = 0;
   }
   
   /*
@@ -521,7 +560,8 @@ void _x_demux_send_data(fifo_buffer_t *fifo, uint8_t *data, int size,
 
   decoder_flags |= BUF_FLAG_FRAME_START;
 
-  while (fifo && size) {
+  _x_assert(size > 0);
+  while (fifo && size > 0) {
 
     buf = fifo->buffer_pool_alloc (fifo);
 
@@ -566,7 +606,8 @@ int _x_demux_read_send_data(fifo_buffer_t *fifo, input_plugin_t *input,
 
   decoder_flags |= BUF_FLAG_FRAME_START;
 
-  while (fifo && size) {
+  _x_assert(size > 0);
+  while (fifo && size > 0) {
 
     buf = fifo->buffer_pool_alloc (fifo);
 
@@ -600,3 +641,49 @@ int _x_demux_read_send_data(fifo_buffer_t *fifo, input_plugin_t *input,
   
   return 0;
 }
+
+/*
+ * Helper function for sending MRL reference events
+ */
+void _x_demux_send_mrl_reference (xine_stream_t *stream, int alternative,
+				  const char *mrl, const char *title,
+				  int start_time, int duration)
+{
+  xine_event_t event;
+  union {
+    xine_mrl_reference_data_ext_t *e;
+    xine_mrl_reference_data_t *b;
+  } data;
+  int mrl_len = strlen (mrl);
+
+  if (!title)
+    title = "";
+
+  /* extended MRL reference event */
+
+  event.stream = stream;
+  event.data_length = offsetof (xine_mrl_reference_data_ext_t, mrl) +
+                      mrl_len + strlen (title) + 2;
+  data.e = event.data = malloc (event.data_length);
+    
+  data.e->alternative = alternative;
+  data.e->start_time = start_time;
+  data.e->duration = duration;
+  strcpy((char *)data.e->mrl, mrl);
+  strcpy((char *)data.e->mrl + mrl_len + 1, title ? title : "");
+
+  event.type = XINE_EVENT_MRL_REFERENCE_EXT;
+  xine_event_send (stream, &event);
+
+  /* plain MRL reference event */
+
+  event.data_length = offsetof (xine_mrl_reference_data_t, mrl) + mrl_len + 1;
+
+  /*data.b->alternative = alternative;*/
+  strcpy (data.b->mrl, mrl);
+
+  event.type = XINE_EVENT_MRL_REFERENCE;
+  xine_event_send (stream, &event);
+
+  free (data.e);
+}
