@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: scratch.c,v 1.17 2006/06/29 12:28:06 dgp85 Exp $
+ * $Id: scratch.c,v 1.22 2006/10/18 18:46:17 hadess Exp $
  *
  * top-level xine functions
  *
@@ -37,13 +37,28 @@
 #define LOG
 */
 
-#include "scratch.h"
 #include "xineutils.h"
+#include "scratch.h"
 
 static void __attribute__((__format__(__printf__, 2, 0)))
   scratch_printf (scratch_buffer_t *this, const char *format, va_list argp)
 {
-  vsnprintf (this->lines[this->cur], SCRATCH_LINE_LEN_MAX, format, argp);
+  time_t t;
+  struct tm tm;
+  size_t l;
+
+  time (&t);
+  localtime_r (&t, &tm);
+
+  if ( ! this->lines[this->cur] )
+    this->lines[this->cur] = xine_xmalloc(SCRATCH_LINE_LEN_MAX+1);
+  if ( ! this->lines[this->cur] )
+    return;
+
+  strftime (this->lines[this->cur], SCRATCH_LINE_LEN_MAX, "%X: ", &tm);
+  l = strlen (this->lines[this->cur]);
+  vsnprintf (this->lines[this->cur] + l, SCRATCH_LINE_LEN_MAX - l, format, argp);
+
   lprintf ("printing format %s to line %d\n", format, this->cur);
   this->cur = (this->cur + 1) % this->num_lines;
 }
@@ -69,10 +84,11 @@ static void scratch_dispose (scratch_buffer_t *this) {
   int   i;
   
   mem = (char *) this->lines[0];
-  free(mem);
   
-  for(i = 0; i < this->num_lines; i++ )
+  for(i = 0; i < this->num_lines; i++ ) {
+    free(this->lines[i]);
     this->lines[i] = NULL;
+  }
   
   free (this->lines);
   free (this->ordered);
@@ -82,17 +98,14 @@ static void scratch_dispose (scratch_buffer_t *this) {
 scratch_buffer_t *_x_new_scratch_buffer (int num_lines) {
   scratch_buffer_t *this;
   int               i;
-  char             *mem;
 
   this = xine_xmalloc (sizeof (scratch_buffer_t));
 
   this->lines   = xine_xmalloc (sizeof (char *) * (num_lines + 1));
   this->ordered = xine_xmalloc (sizeof (char *) * (num_lines + 1));
 
-  mem = (char *) xine_xmalloc((sizeof(char) * SCRATCH_LINE_LEN_MAX) * num_lines);
-
   for (i = 0; i < num_lines; i++)
-    this->lines[i] = (char *) (mem + (i * SCRATCH_LINE_LEN_MAX));
+    this->lines[i] = NULL;
 
   this->ordered[i]     = NULL;
   this->lines[i]       = NULL;
